@@ -1,9 +1,10 @@
-require "common"
+shard_include "common"
 
 AntinukeBehaviour = class(Behaviour)
 
 local DebugEnabled = false
 
+
 local function EchoDebug(inStr)
 	if DebugEnabled then
 		game:SendToConsole("AntinukeBehaviour: " .. inStr)
