@@ -1,7 +1,8 @@
-require "common"
+shard_include "common"
 
 local DebugEnabled = false
 
+
 local function EchoDebug(inStr)
 	if DebugEnabled then
 		game:SendToConsole("AssistHandler: " .. inStr)
@@ -261,6 +262,7 @@ function AssistHandler:Release(builder, bid, dead)
 				ai.nonAssistant[asstbehaviour.id] = true
 			end
 		end
+		-- self.ai:UnitIdle(asstbehaviour.unit:Internal())
 		EchoDebug(asstbehaviour.name .. " released to available assistants")
 	end
 	self.working[bid] = nil
@@ -329,6 +331,7 @@ function AssistHandler:RemoveWorking(asstbehaviour)
 end
 
 function AssistHandler:AssignIDByName(asstbehaviour)
+	-- game:SendToConsole("assisthandler:assignidbyname", ai, ai.id, self.ai, self.ai.id)
 	local uname = asstbehaviour.name
 	if self.IDByNameTaken[uname] == nil then
 		asstbehaviour.IDByName = 1
