@@ -89,7 +89,7 @@ end
 
 function AngleDist(angle1, angle2)
 	return abs((angle1 + pi -  angle2) % twicePi - pi)
-	-- Spring.Echo(math.floor(angleDist * 57.29), math.floor(high * 57.29), math.floor(low * 57.29))
+	-- game:SendToConsole(math.floor(angleDist * 57.29), math.floor(high * 57.29), math.floor(low * 57.29))
 end
 
 function AngleAdd(angle1, angle2)
@@ -289,7 +289,7 @@ function WhatHurtsUnit(unitName, mtype, position)
 	if mtype then whatHurtsMtype[mtype] = hurts end
 	if mtype == "amp" and position ~= nil then
 		-- special case: amphibious need to check whether underwater or not
-		local underwater = ai.maphandler:IsUnderWater(position)
+		local underwater = position.y < 0
 		if underwater then
 			return { ground = false, air = false, submerged = true}
 		else
