@@ -1,7 +1,8 @@
-require "common"
+shard_include "common"
 
 local DebugEnabled = false
 
+
 local function EchoDebug(inStr)
 	if DebugEnabled then
 		game:SendToConsole("DefendHandler: " .. inStr)
@@ -49,7 +50,7 @@ function DefendHandler:Init()
 	self.wardsByDefenderID = {}
 	self.defendersByID = {}
 	self.unitGuardDistances = {}
-	ai.frontPosition = {}
+	self.ai.frontPosition = {}
 end
 
 function DefendHandler:AddWard(behaviour, turtle)
@@ -276,10 +277,15 @@ function DefendHandler:AssignAll(GAS, mtype) -- Ground Air Submerged (weapon), m
 				end
 				if okay then
 					local defender = dfndbehaviour.unit:Internal()
-					if ai.maphandler:UnitCanGoHere(defender, wardPos) then
-						local defenderPos = defender:GetPosition()
-						local dist = Distance(defenderPos, wardPos)
-						bydistance[dist] = dfndbehaviour -- the probability of the same distance is near zero
+					local ux, uy, uz = defender:GetPosition()
+					if ux then
+						if self.ai.maphandler:UnitCanGoHere(defender, wardPos) then
+							local defenderPos = defender:GetPosition()
+							local dist = Distance(defenderPos, wardPos)
+							bydistance[dist] = dfndbehaviour -- the probability of the same distance is near zero
+						end
+					else
+						-- game:SendToConsole(self.ai.id, "defender unit nil position", defender:ID(), defender:Name())
 					end
 				end
 			end
@@ -366,7 +372,8 @@ function DefendHandler:AssignLoiterers(ward)
 end
 
 function DefendHandler:FreeDefenders(ward, GAS, mtype)
-	for i, dfndbehaviour in pairs(ward.defenders) do
+	for i = #ward.defenders, 1, -1 do
+		local dfndbehaviour = ward.defenders[i]
 		if dfndbehaviour.hits == GAS and dfndbehaviour.mtype == mtype then
 			table.remove(ward.defenders, i)
 		end
@@ -433,20 +440,23 @@ function DefendHandler:AddDefender(dfndbehaviour)
 end
 
 function DefendHandler:RemoveDefender(dfndbehaviour)
+	-- Spring.Echo(self.ai.id, "remove defender", dfndbehaviour.hits, dfndbehaviour.mtype, self.defenders[dfndbehaviour.hits])
 	if dfndbehaviour.tough or dfndbehaviour.hits == "air" then
-		for i, db in pairs(self.defenders[dfndbehaviour.hits][dfndbehaviour.mtype]) do
+		for i = #self.defenders[dfndbehaviour.hits][dfndbehaviour.mtype], 1, -1 do
+			local db = self.defenders[dfndbehaviour.hits][dfndbehaviour.mtype][i]
 			if db == dfndbehaviour then
 				table.remove(self.defenders[dfndbehaviour.hits][dfndbehaviour.mtype], i)
 				self.needAssignment[dfndbehaviour.hits][dfndbehaviour.mtype] = true
-				return
+				-- return
 			end
 		end
 	else
-		for i, db in pairs(self.loiterers[dfndbehaviour.mtype]) do
+		for i = #self.loiterers[dfndbehaviour.mtype], 1, -1 do
+			local db = self.loiterers[dfndbehaviour.mtype][i]
 			if db == dfndbehaviour then
 				table.remove(self.loiterers[dfndbehaviour.mtype], i)
 				self.needLoitererAssignment[dfndbehaviour.mtype] = true
-				return
+				-- return
 			end
 		end
 	end
@@ -540,10 +550,10 @@ function DefendHandler:FindFronts(troublingCells)
 						turtle.threatForecastAngle = AngleAtoB(turtle.position.x, turtle.position.z, cell.pos.x, cell.pos.z)
 						turtle.front = true
 						self:Danger(nil, turtle, GAS)
-						ai.incomingThreat = cell.response[GAS]
-						ai.frontPosition[GAS] = turtle.position
+						self.ai.incomingThreat = cell.response[GAS]
+						self.ai.frontPosition[GAS] = turtle.position
 					else
-						ai.incomingThreat = 0
+						self.ai.incomingThreat = 0
 					end
 				end
 				if nearestMobile ~= nil then
@@ -568,7 +578,7 @@ end
 -- receive a signal that a building is threatened or a turtle is on the front
 function DefendHandler:Danger(behaviour, turtle, GAS)
 	local f = game:Frame()
-	if turtle == nil and behaviour ~= nil then turtle = ai.turtlehandler:GetUnitTurtle(behaviour.id) end
+	if turtle == nil and behaviour ~= nil then turtle = self.ai.turtlehandler:GetUnitTurtle(behaviour.id) end
 	if turtle ~= nil then
 		for i, ward in pairs(self.wards) do
 			if ward.turtle == turtle then
