@@ -1,5 +1,3 @@
-require "unittable"
-
 local DebugEnabled = false
 
 local function EchoDebug(inStr)
@@ -35,6 +33,8 @@ factoryMobilities = {
 	asubpen = {"sub", "amp"},
 	corgant = {"bot"},
 	armshltx = {"bot"},
+	corgantuw = {"bot"},
+	armshltxuw = {"bot"},
 }
 
 -- for calculating what factories to build
@@ -88,6 +88,8 @@ factoryExitSides = {
 	asubpen = 4,
 	corgant = 1,
 	armshltx = 1,
+	corgantuw = 1,
+	armshltxuw = 1,
 }
 
 littlePlasmaList = {
@@ -124,6 +126,9 @@ mexUpgrade = {
 	armmex = "armmoho",
 	coruwmex = "coruwmme",
 	armuwmex = "armuwmme",
+	armamex = "armmoho",
+	corexp = "cormexp",
+	
 }
 
 -- these will be abandoned faster
@@ -161,8 +166,8 @@ turtleList = {
 	armfmkr = 0.5,
 	cormmkr = 4,
 	armmmkr = 4,
-	coruwmmm = 4,
-	armuwmmm = 4,
+	corfmmm = 4,
+	armfmmm = 4,
 	corestor = 0.5,
 	armestor = 0.5,
 	cormstor = 0.5,
@@ -195,6 +200,8 @@ advFactories = {
 expFactories = {
 	corgant = 1,
 	armshltx = 1,
+	corgantuw = 1,
+	armshltxuw = 1,
 }
 
 -- leads to experimental
@@ -203,6 +210,10 @@ leadsToExpFactories = {
 	armlab = 1,
 	coralab = 1,
 	armalab = 1,
+	corsy = 1,
+	armsy = 1,
+	corasy = 1,
+	armasy = 1,
 }
 
 -- sturdy, cheap units to be built in larger numbers than siege units
@@ -517,6 +528,111 @@ seaplaneConList = {
 	armcsa = 1,
 }
 
+
+Eco1={
+	armsolar=1,
+	armwin=1,
+	armadvsol=1,
+	armtide=1,
+
+	corsolar=1,
+	corwin=1,
+	coradvsol=1,
+	cortide=1,
+
+	-- corgeo=1,
+	-- armgeo=1,
+
+	--store
+
+	armestor=1,
+	armmstor=1,
+	armuwes=1,
+	armuwms=1,
+
+	corestor=1,
+	cormstor=1,
+	coruwes=1,
+	coruwms=1,
+
+	--conv
+	armmakr=1,
+	cormakr=1,
+	armfmkr=1,
+	corfmkr=1,
+
+
+	--metalli
+	corexp=1,
+	armamex=1,
+
+	cormex=1,
+	armmex=1,
+
+	armuwmex=1,
+	coruwmex=1,
+
+	armnanotc=1,
+	cornanotc=1,
+}
+
+Eco2={
+	--metalli
+	armmoho=1,
+	cormoho=1,
+	cormexp=1,
+
+	coruwmme=1,
+	armuwmme=1,
+
+	--magazzini 
+	armuwadves=1,
+	armuwadvms=1,
+
+	coruwadves=1,
+	coruwadvms=1,
+
+	cmgeo = 1,
+	amgeo = 1,
+	corbhmth =1,
+	armgmm =1,
+
+	corfus = 1,
+	armfus = 1,
+	cafus = 1,
+	aafus = 1,
+	armuwfus = 1,
+	coruwfus = 1,
+
+	--convertitori
+	cormmkr=1,
+	armmmkr=1,
+	corfmmm=1,
+	armfmmm=1,
+}
+
+cleaners = {
+	armbeaver = 1,
+	cormuskrat = 1,
+	armcom = 1,
+	corcom = 1,
+	armdecom = 1,
+	cordecom = 1,
+}
+
+cleanable = {
+	armsolar='ground',
+	corsolar='ground',
+	armadvsol = 'ground',
+	coradvsol = 'ground',
+	armtide = 'floating',
+	cortite = 'floating',
+	armfmkr = 'floating',
+	corfmkr = 'floating',
+	cormakr = 'ground',
+	armmakr = 'ground',
+}
+
 -- minimum, maximum, starting point units required to attack, bomb
 minAttackCounter = 8
 maxAttackCounter = 30
@@ -548,16 +664,26 @@ FactoryUnitName = "buildfactory"
 -- this unit is used to check for underwater metal spots
 UWMetalSpotCheckUnit = "coruwmex"
 
-mobUnitName = {}
-mobUnitName["veh"] = "armllt" -- this looks wrong, but it gives us a better picture of where vehicles can go
-mobUnitName["bot"] = "corck"
-mobUnitName["amp"] = "cormuskrat"
-mobUnitName["hov"] = "corsh"
-mobUnitName["shp"] = "corcs"
-mobUnitName["sub"] = "coracsub"
+-- for non-lua only; tests build orders of these units to determine mobility there
+-- multiple units for one mtype function as OR
+mobUnitNames = {
+	veh = {"corcv", "armllt"},
+	bot = {"corck", "armeyes"},
+	amp = {"cormuskrat"},
+	hov = {"corsh", "armfdrag"},
+	shp = {"corcs"},
+	sub = {"coracsub"},
+}
 
--- this unit is used to check for hoverable water
-WaterSurfaceUnitName = "armfdrag"
+-- for ShardSpringLua only; tests move orders of these units to determine mobility there
+mobUnitExampleName = {
+	veh = "armcv",
+	bot = "armck",
+	amp = "armbeaver",
+	hov = "armch",
+	shp = "armcs",
+	sub = "armacsub"
+}
 
 -- side names
 CORESideName = "core"
