@@ -4,7 +4,7 @@
 
 #include "AIException.h"
 
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
 #include <string>
 
