@@ -15,7 +15,7 @@ extern "C" {
  * Each Skirmish AI instance will receive an instance of this struct
  * in its init(skirmishAIId) function and with the SInitEvent.
  *
- * This struct contians only activities that leave the game state as it is,
+ * This struct contains only activities that leave the game state as it is,
  * in spring terms: unsynced events
  * Activities that change game state (-> synced events) are handled through
  * AI commands, defined in AISCommands.h.
@@ -48,6 +48,9 @@ struct SSkirmishAICallback {
 	 */
 	int               (CALLING_CONV *Engine_handleCommand)(int skirmishAIId, int toId, int commandId, int commandTopic, void* commandData);
 
+	int               (CALLING_CONV *Engine_executeCommand)(int skirmishAIId, int unitId, int groupId, void* commandData);
+
+
 	/// Returns the major engine revision number (e.g. 83)
 	const char*       (CALLING_CONV *Engine_Version_getMajor)(int skirmishAIId);
 
@@ -362,11 +365,19 @@ struct SSkirmishAICallback {
 	 */
 	float             (CALLING_CONV *Game_getTeamResourceStorage)(int skirmishAIId, int otherTeamId, int resourceId);
 
+	float             (CALLING_CONV *Game_getTeamResourcePull)(int skirmishAIId, int otherTeamId, int resourceId);
+
+	float             (CALLING_CONV *Game_getTeamResourceShare)(int skirmishAIId, int otherTeamId, int resourceId);
+
+	float             (CALLING_CONV *Game_getTeamResourceSent)(int skirmishAIId, int otherTeamId, int resourceId);
+
+	float             (CALLING_CONV *Game_getTeamResourceReceived)(int skirmishAIId, int otherTeamId, int resourceId);
+
+	float             (CALLING_CONV *Game_getTeamResourceExcess)(int skirmishAIId, int otherTeamId, int resourceId);
+
 	/// Returns true, if the two supplied ally-teams are currently allied
 	bool              (CALLING_CONV *Game_isAllied)(int skirmishAIId, int firstAllyTeamId, int secondAllyTeamId);
 
-	bool              (CALLING_CONV *Game_isExceptionHandlingEnabled)(int skirmishAIId);
-
 	bool              (CALLING_CONV *Game_isDebugModeEnabled)(int skirmishAIId);
 
 	int               (CALLING_CONV *Game_getMode)(int skirmishAIId);
@@ -398,6 +409,16 @@ struct SSkirmishAICallback {
 	 */
 	void              (CALLING_CONV *Game_getCategoryName)(int skirmishAIId, int categoryFlag, char* name, int name_sizeMax);
 
+	/**
+	 * @return float value of parameter if it's set, defaultValue otherwise.
+	 */
+	float             (CALLING_CONV *Game_getRulesParamFloat)(int skirmishAIId, const char* gameRulesParamName, float defaultValue);
+
+	/**
+	 * @return string value of parameter if it's set, defaultValue otherwise.
+	 */
+	const char*       (CALLING_CONV *Game_getRulesParamString)(int skirmishAIId, const char* gameRulesParamName, const char* defaultValue);
+
 // END misc callback functions
 
 
@@ -415,14 +436,6 @@ struct SSkirmishAICallback {
 // END Visualization related callback functions
 
 
-// BEGINN kind of deprecated; it is recommended not to use these
-//	bool              (CALLING_CONV *getProperty)(int skirmishAIId, int id, int property, void* dst);
-
-//	bool              (CALLING_CONV *getValue)(int skirmishAIId, int id, void* dst);
-
-// END kind of deprecated; it is recommended not to use these
-
-
 // BEGINN OBJECT Cheats
 	/**
 	 * Returns whether this AI may use active cheats.
@@ -467,6 +480,16 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *Economy_getStorage)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
 
+	float             (CALLING_CONV *Economy_getPull)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
+
+	float             (CALLING_CONV *Economy_getShare)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
+
+	float             (CALLING_CONV *Economy_getSent)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
+
+	float             (CALLING_CONV *Economy_getReceived)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
+
+	float             (CALLING_CONV *Economy_getExcess)(int skirmishAIId, int resourceId); //$ REF:resourceId->Resource
+
 // END OBJECT Resource
 
 
@@ -495,7 +518,6 @@ struct SSkirmishAICallback {
 
 	int               (CALLING_CONV *getUnitDefByName)(int skirmishAIId, const char* unitName); //$ REF:RETURN->UnitDef
 
-//	int               (CALLING_CONV *UnitDef_getId)(int skirmishAIId, int unitDefId);
 
 	/** Forces loading of the unit model */
 	float             (CALLING_CONV *UnitDef_getHeight)(int skirmishAIId, int unitDefId);
@@ -507,18 +529,6 @@ struct SSkirmishAICallback {
 
 	const char*       (CALLING_CONV *UnitDef_getHumanName)(int skirmishAIId, int unitDefId);
 
-	const char*       (CALLING_CONV *UnitDef_getFileName)(int skirmishAIId, int unitDefId);
-
-	/** @deprecated */
-	int               (CALLING_CONV *UnitDef_getAiHint)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getCobId)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getTechLevel)(int skirmishAIId, int unitDefId);
-
-	/** @deprecated */
-	const char*       (CALLING_CONV *UnitDef_getGaia)(int skirmishAIId, int unitDefId);
-
 	float             (CALLING_CONV *UnitDef_getUpkeep)(int skirmishAIId, int unitDefId, int resourceId); //$ REF:resourceId->Resource
 
 	/** This amount of the resource will always be created. */
@@ -542,7 +552,6 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_getStorage)(int skirmishAIId, int unitDefId, int resourceId); //$ REF:resourceId->Resource
 
-	bool              (CALLING_CONV *UnitDef_isSquareResourceExtractor)(int skirmishAIId, int unitDefId, int resourceId); //$ REF:resourceId->Resource
 
 	float             (CALLING_CONV *UnitDef_getBuildTime)(int skirmishAIId, int unitDefId);
 
@@ -640,12 +649,6 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_getSlideTolerance)(int skirmishAIId, int unitDefId);
 
-	/**
-	 * Build location relevant maximum steepness of the underlaying terrain.
-	 * Used to calculate the maxHeightDif.
-	 */
-	float             (CALLING_CONV *UnitDef_getMaxSlope)(int skirmishAIId, int unitDefId);
-
 	/**
 	 * Maximum terra-form height this building allows.
 	 * If this value is 0.0, you can only build this structure on
@@ -695,16 +698,14 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_getMaxWeaponRange)(int skirmishAIId, int unitDefId);
 
-	/** @deprecated */
-	const char*       (CALLING_CONV *UnitDef_getType)(int skirmishAIId, int unitDefId);
 
 	const char*       (CALLING_CONV *UnitDef_getTooltip)(int skirmishAIId, int unitDefId);
 
 	const char*       (CALLING_CONV *UnitDef_getWreckName)(int skirmishAIId, int unitDefId);
 
-	const char*       (CALLING_CONV *UnitDef_getDeathExplosion)(int skirmishAIId, int unitDefId);
+	int               (CALLING_CONV *UnitDef_getDeathExplosion)(int skirmishAIId, int unitDefId); //$ REF:RETURN->WeaponDef
 
-	const char*       (CALLING_CONV *UnitDef_getSelfDExplosion)(int skirmishAIId, int unitDefId);
+	int               (CALLING_CONV *UnitDef_getSelfDExplosion)(int skirmishAIId, int unitDefId); //$ REF:RETURN->WeaponDef
 
 	/**
 	 * Returns the name of the category this unit is in.
@@ -774,7 +775,6 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_getWingAngle)(int skirmishAIId, int unitDefId);
 
-	float             (CALLING_CONV *UnitDef_getDrag)(int skirmishAIId, int unitDefId);
 
 	float             (CALLING_CONV *UnitDef_getFrontToSpeed)(int skirmishAIId, int unitDefId);
 
@@ -792,18 +792,12 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_getVerticalSpeed)(int skirmishAIId, int unitDefId);
 
-	/**
-	 * @deprecated
-	 */
-	bool              (CALLING_CONV *UnitDef_isAbleToCrash)(int skirmishAIId, int unitDefId);
 
-	/**
-	 * @deprecated
-	 */
 	bool              (CALLING_CONV *UnitDef_isHoverAttack)(int skirmishAIId, int unitDefId);
 
 	bool              (CALLING_CONV *UnitDef_isAirStrafe)(int skirmishAIId, int unitDefId);
 
+
 	/**
 	 * @return  < 0:  it can land
 	 *          >= 0: how much the unit will move during hovering on the spot
@@ -837,8 +831,6 @@ struct SSkirmishAICallback {
 
 	int               (CALLING_CONV *UnitDef_getZSize)(int skirmishAIId, int unitDefId);
 
-	/** @deprecated */
-	int               (CALLING_CONV *UnitDef_getBuildAngle)(int skirmishAIId, int unitDefId);
 
 // beginn: transports stuff
 	float             (CALLING_CONV *UnitDef_getLoadingRadius)(int skirmishAIId, int unitDefId);
@@ -922,7 +914,6 @@ struct SSkirmishAICallback {
 
 	bool              (CALLING_CONV *UnitDef_isHideDamage)(int skirmishAIId, int unitDefId);
 
-	bool              (CALLING_CONV *UnitDef_isCommander)(int skirmishAIId, int unitDefId);
 
 	bool              (CALLING_CONV *UnitDef_isShowPlayerName)(int skirmishAIId, int unitDefId);
 
@@ -947,17 +938,6 @@ struct SSkirmishAICallback {
 	 */
 	int               (CALLING_CONV *UnitDef_getNoChaseCategory)(int skirmishAIId, int unitDefId);
 
-	bool              (CALLING_CONV *UnitDef_isLeaveTracks)(int skirmishAIId, int unitDefId);
-
-	float             (CALLING_CONV *UnitDef_getTrackWidth)(int skirmishAIId, int unitDefId);
-
-	float             (CALLING_CONV *UnitDef_getTrackOffset)(int skirmishAIId, int unitDefId);
-
-	float             (CALLING_CONV *UnitDef_getTrackStrength)(int skirmishAIId, int unitDefId);
-
-	float             (CALLING_CONV *UnitDef_getTrackStretch)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getTrackType)(int skirmishAIId, int unitDefId);
 
 	bool              (CALLING_CONV *UnitDef_isAbleToDropFlare)(int skirmishAIId, int unitDefId);
 
@@ -985,27 +965,6 @@ struct SSkirmishAICallback {
 	 */
 	bool              (CALLING_CONV *UnitDef_isLevelGround)(int skirmishAIId, int unitDefId);
 
-	bool              (CALLING_CONV *UnitDef_isUseBuildingGroundDecal)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getBuildingDecalType)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getBuildingDecalSizeX)(int skirmishAIId, int unitDefId);
-
-	int               (CALLING_CONV *UnitDef_getBuildingDecalSizeY)(int skirmishAIId, int unitDefId);
-
-	float             (CALLING_CONV *UnitDef_getBuildingDecalDecaySpeed)(int skirmishAIId, int unitDefId);
-
-	/**
-	 * Maximum flight time in seconds before the aircraft needs
-	 * to return to an air repair bay to refuel.
-	 */
-	float             (CALLING_CONV *UnitDef_getMaxFuel)(int skirmishAIId, int unitDefId);
-
-	/** Time to fully refuel the unit */
-	float             (CALLING_CONV *UnitDef_getRefuelTime)(int skirmishAIId, int unitDefId);
-
-	/** Minimum build power of airbases that this aircraft can land on */
-	float             (CALLING_CONV *UnitDef_getMinAirBasePower)(int skirmishAIId, int unitDefId);
 
 	/** Number of units of this type allowed simultaneously in the game */
 	int               (CALLING_CONV *UnitDef_getMaxThisUnit)(int skirmishAIId, int unitDefId);
@@ -1026,18 +985,6 @@ struct SSkirmishAICallback {
 
 	bool              (CALLING_CONV *UnitDef_isMoveDataAvailable)(int skirmishAIId, int unitDefId); //$ AVAILABLE:MoveData
 
-	/// @deprecated
-	float             (CALLING_CONV *UnitDef_MoveData_getMaxAcceleration)(int skirmishAIId, int unitDefId);
-
-	/// @deprecated
-	float             (CALLING_CONV *UnitDef_MoveData_getMaxBreaking)(int skirmishAIId, int unitDefId);
-
-	/// @deprecated
-	float             (CALLING_CONV *UnitDef_MoveData_getMaxSpeed)(int skirmishAIId, int unitDefId);
-
-	/// @deprecated
-	short             (CALLING_CONV *UnitDef_MoveData_getMaxTurnRate)(int skirmishAIId, int unitDefId);
-
 	int               (CALLING_CONV *UnitDef_MoveData_getXSize)(int skirmishAIId, int unitDefId);
 
 	int               (CALLING_CONV *UnitDef_MoveData_getZSize)(int skirmishAIId, int unitDefId);
@@ -1048,14 +995,12 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_MoveData_getSlopeMod)(int skirmishAIId, int unitDefId);
 
-	float             (CALLING_CONV *UnitDef_MoveData_getDepthMod)(int skirmishAIId, int unitDefId);
+	float             (CALLING_CONV *UnitDef_MoveData_getDepthMod)(int skirmishAIId, int unitDefId, float height);
 
 	int               (CALLING_CONV *UnitDef_MoveData_getPathType)(int skirmishAIId, int unitDefId);
 
 	float             (CALLING_CONV *UnitDef_MoveData_getCrushStrength)(int skirmishAIId, int unitDefId);
 
-	/** enum MoveType { Ground_Move=0, Hover_Move=1, Ship_Move=2 }; */
-	int               (CALLING_CONV *UnitDef_MoveData_getMoveType)(int skirmishAIId, int unitDefId);
 
 	/** enum SpeedModClass { Tank=0, KBot=1, Hover=2, Ship=3 }; */
 	int               (CALLING_CONV *UnitDef_MoveData_getSpeedModClass)(int skirmishAIId, int unitDefId);
@@ -1082,11 +1027,6 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *UnitDef_WeaponMount_getMaxAngleDif)(int skirmishAIId, int unitDefId, int weaponMountId);
 
-	/**
-	 * How many seconds of fuel it costs for the owning unit to fire this weapon.
-	 */
-	float             (CALLING_CONV *UnitDef_WeaponMount_getFuelUsage)(int skirmishAIId, int unitDefId, int weaponMountId);
-
 	/**
 	 * Returns the bit field value denoting the categories this weapon should
 	 * not target.
@@ -1192,51 +1132,24 @@ struct SSkirmishAICallback {
 	int               (CALLING_CONV *Unit_getDef)(int skirmishAIId, int unitId); //$ REF:RETURN->UnitDef
 
 	/**
-	 * This is a set of parameters that is initialized
-	 * in CreateUnitRulesParams() and may change during the game.
-	 * Each parameter is uniquely identified only by its id
-	 * (which is the index in the vector).
-	 * Parameters may or may not have a name.
+	 * @return float value of parameter if it's set, defaultValue otherwise.
 	 */
-	int               (CALLING_CONV *Unit_getModParams)(int skirmishAIId, int unitId); //$ FETCHER:MULTI:NUM:ModParam
+	float             (CALLING_CONV *Unit_getRulesParamFloat)(int skirmishAIId, int unitId, const char* unitRulesParamName, float defaultValue);
 
 	/**
-	 * Not every mod parameter has a name.
+	 * @return string value of parameter if it's set, defaultValue otherwise.
 	 */
-	const char*       (CALLING_CONV *Unit_ModParam_getName)(int skirmishAIId, int unitId, int modParamId);
-
-	float             (CALLING_CONV *Unit_ModParam_getValue)(int skirmishAIId, int unitId, int modParamId);
+	const char*       (CALLING_CONV *Unit_getRulesParamString)(int skirmishAIId, int unitId, const char* unitRulesParamName, const char* defaultValue);
 
 	int               (CALLING_CONV *Unit_getTeam)(int skirmishAIId, int unitId);
 
 	int               (CALLING_CONV *Unit_getAllyTeam)(int skirmishAIId, int unitId);
 
-	/**
-	 * Indicates the units main function.
-	 * This can be used as help for (skirmish) AIs.
-	 *
-	 * example:
-	 * A unit can shoot, build and transport other units.
-	 * To human players, it is obvious that transportation is the units
-	 * main function, as it can transport a lot of units,
-	 * but has only weak build- and fire-power.
-	 * Instead of letting the AI developers write complex
-	 * algorithms to find out the same, mod developers can set this value.
-	 *
-	 * @return  0: ???
-	 *          1: ???
-	 *          2: ???
-	 *          ...
-	 * @deprecated
-	 */
-	int               (CALLING_CONV *Unit_getAiHint)(int skirmishAIId, int unitId);
 
 	int               (CALLING_CONV *Unit_getStockpile)(int skirmishAIId, int unitId);
 
 	int               (CALLING_CONV *Unit_getStockpileQueued)(int skirmishAIId, int unitId);
 
-	float             (CALLING_CONV *Unit_getCurrentFuel)(int skirmishAIId, int unitId);
-
 	/** The unit's max speed */
 	float             (CALLING_CONV *Unit_getMaxSpeed)(int skirmishAIId, int unitId);
 
@@ -1296,6 +1209,12 @@ struct SSkirmishAICallback {
 	/** The unit's current health */
 	float             (CALLING_CONV *Unit_getHealth)(int skirmishAIId, int unitId);
 
+	float             (CALLING_CONV *Unit_getParalyzeDamage)(int skirmishAIId, int unitId);
+
+	float             (CALLING_CONV *Unit_getCaptureProgress)(int skirmishAIId, int unitId);
+
+	float             (CALLING_CONV *Unit_getBuildProgress)(int skirmishAIId, int unitId);
+
 	float             (CALLING_CONV *Unit_getSpeed)(int skirmishAIId, int unitId);
 
 	/**
@@ -1332,9 +1251,32 @@ struct SSkirmishAICallback {
 	/** Number of the last frame this unit received an order from a player. */
 	int               (CALLING_CONV *Unit_getLastUserOrderFrame)(int skirmishAIId, int unitId);
 
+	int               (CALLING_CONV *Unit_getWeapons)(int skirmishAIId, int unitId); //$ FETCHER:MULTI:NUM:Weapon
+
+	int               (CALLING_CONV *Unit_getWeapon)(int skirmishAIId, int unitId, int weaponMountId); //$ REF:weaponMountId->WeaponMount REF:RETURN->Weapon
 // END OBJECT Unit
 
 
+// BEGINN OBJECT Team
+	bool              (CALLING_CONV *Team_hasAIController)(int skirmishAIId, int teamId);
+
+	int               (CALLING_CONV *getEnemyTeams)(int skirmishAIId, int* teamIds, int teamIds_sizeMax); //$ FETCHER:MULTI:IDs:Team:teamIds
+
+	int               (CALLING_CONV *getAllyTeams)(int skirmishAIId, int* teamIds, int teamIds_sizeMax); //$ FETCHER:MULTI:IDs:Team:teamIds
+
+	/**
+	 * @return float value of parameter if it's set, defaultValue otherwise.
+	 */
+	float             (CALLING_CONV *Team_getRulesParamFloat)(int skirmishAIId, int teamId, const char* teamRulesParamName, float defaultValue);
+
+	/**
+	 * @return string value of parameter if it's set, defaultValue otherwise.
+	 */
+	const char*       (CALLING_CONV *Team_getRulesParamString)(int skirmishAIId, int teamId, const char* teamRulesParamName, const char* defaultValue);
+
+// END OBJECT Team
+
+
 // BEGINN OBJECT Group
 	int               (CALLING_CONV *getGroups)(int skirmishAIId, int* groupIds, int groupIds_sizeMax); //$ FETCHER:MULTI:IDs:Group:groupIds
 
@@ -1434,7 +1376,6 @@ struct SSkirmishAICallback {
 
 	const char*       (CALLING_CONV *Mod_getDescription)(int skirmishAIId);
 
-	bool              (CALLING_CONV *Mod_getAllowTeamColors)(int skirmishAIId);
 
 	/**
 	 * Should constructions without builders decay?
@@ -1564,14 +1505,9 @@ struct SSkirmishAICallback {
 	int               (CALLING_CONV *Mod_getAirMipLevel)(int skirmishAIId);
 
 	/**
-	 * units sightdistance will be multiplied with this, for testing purposes
+	 * miplevel for radar
 	 */
-	float             (CALLING_CONV *Mod_getLosMul)(int skirmishAIId);
-
-	/**
-	 * units airsightdistance will be multiplied with this, for testing purposes
-	 */
-	float             (CALLING_CONV *Mod_getAirLosMul)(int skirmishAIId);
+	int               (CALLING_CONV *Mod_getRadarMipLevel)(int skirmishAIId);
 
 	/**
 	 * when underwater, units are not in LOS unless also in sonar
@@ -1654,7 +1590,7 @@ struct SSkirmishAICallback {
 
 	/**
 	 * @brief the level of sight map
-	 * gs->mapx >> losMipLevel
+	 * mapDims.mapx >> losMipLevel
 	 * A square with value zero means you do not have LOS coverage on it.
 	 *Mod_getLosMipLevel
 	 * - do NOT modify or delete the height-map (native code relevant only)
@@ -1671,30 +1607,32 @@ struct SSkirmishAICallback {
 	 */
 	int               (CALLING_CONV *Map_getLosMap)(int skirmishAIId, int* losValues, int losValues_sizeMax); //$ ARRAY:losValues
 
+	/**
+	 * @brief the level of sight map
+	 * mapDims.mapx >> airMipLevel
+	 * @see getLosMap()
+	 */
+	int               (CALLING_CONV *Map_getAirLosMap)(int skirmishAIId, int* airLosValues, int airLosValues_sizeMax); //$ ARRAY:airLosValues
+
 	/**
 	 * @brief the radar map
-	 * A square with value 0 means you do not have radar coverage on it.
-	 *
-	 * - do NOT modify or delete the height-map (native code relevant only)
-	 * - index 0 is top left
-	 * - each data position is 8*8 in size
-	 * - the value for the full resolution position (x, z) is at index ((z * width + x) / 8)
-	 * - the last value, bottom right, is at index (width/8 * height/8 - 1)
+	 * mapDims.mapx >> radarMipLevel
+	 * @see getLosMap()
 	 */
 	int               (CALLING_CONV *Map_getRadarMap)(int skirmishAIId, int* radarValues, int radarValues_sizeMax); //$ ARRAY:radarValues
 
-	/**
-	 * @brief the radar jammer map
-	 * A square with value 0 means you do not have radar jamming coverage.
-	 *
-	 * - do NOT modify or delete the height-map (native code relevant only)
-	 * - index 0 is top left
-	 * - each data position is 8*8 in size
-	 * - the value for the full resolution position (x, z) is at index ((z * width + x) / 8)
-	 * - the last value, bottom right, is at index (width/8 * height/8 - 1)
-	 */
+	/** @see getRadarMap() */
+	int               (CALLING_CONV *Map_getSonarMap)(int skirmishAIId, int* sonarValues, int sonarValues_sizeMax); //$ ARRAY:sonarValues
+
+	/** @see getRadarMap() */
+	int               (CALLING_CONV *Map_getSeismicMap)(int skirmishAIId, int* seismicValues, int seismicValues_sizeMax); //$ ARRAY:seismicValues
+
+	/** @see getRadarMap() */
 	int               (CALLING_CONV *Map_getJammerMap)(int skirmishAIId, int* jammerValues, int jammerValues_sizeMax); //$ ARRAY:jammerValues
 
+	/** @see getRadarMap() */
+	int               (CALLING_CONV *Map_getSonarJammerMap)(int skirmishAIId, int* sonarJammerValues, int sonarJammerValues_sizeMax); //$ ARRAY:sonarJammerValues
+
 	/**
 	 * @brief resource maps
 	 * This map shows the resource density on the map.
@@ -1774,6 +1712,37 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *Map_getGravity)(int skirmishAIId);
 
+	float             (CALLING_CONV *Map_getWaterDamage)(int skirmishAIId);
+
+	bool              (CALLING_CONV *Map_isDeformable)(int skirmishAIId);
+
+	/** Returns global map hardness */
+	float             (CALLING_CONV *Map_getHardness)(int skirmishAIId);
+
+	/**
+	 * Returns hardness modifiers of the squares adjusted by terrain type.
+	 *
+	 * - index 0 is top left
+	 * - each data position is 2*2 in size (relative to heightmap)
+	 * - the value for the full resolution position (x, z) is at index ((z * width + x) / 2)
+	 * - the last value, bottom right, is at index (width/2 * height/2 - 1)
+	 *
+	 * @see getHardness()
+	 */
+	int               (CALLING_CONV *Map_getHardnessModMap)(int skirmishAIId, float* hardMods, int hardMods_sizeMax); //$ ARRAY:hardMods
+
+	/**
+	 * Returns speed modifiers of the squares
+	 * for specific speedModClass adjusted by terrain type.
+	 *
+	 * - index 0 is top left
+	 * - each data position is 2*2 in size (relative to heightmap)
+	 * - the value for the full resolution position (x, z) is at index ((z * width + x) / 2)
+	 * - the last value, bottom right, is at index (width/2 * height/2 - 1)
+	 *
+	 * @see MoveData#getSpeedModClass
+	 */
+	int               (CALLING_CONV *Map_getSpeedModMap)(int skirmishAIId, int speedModClass, float* speedMods, int speedMods_sizeMax); //$ ARRAY:speedMods
 
 	/**
 	 * Returns all points drawn with this AIs team color,
@@ -1806,7 +1775,7 @@ struct SSkirmishAICallback {
 	/**
 	 * Returns the closest position from a given position that a building can be
 	 * built at.
-	 * @param minDist the distance in 1/(SQUARE_SIZE * 2) = 1/16 of full map
+	 * @param minDist the distance in 1/BUILD_SQUARE_SIZE = 1/16 of full map
 	 *                resolution, that the building must keep to other
 	 *                buildings; this makes it easier to keep free paths through
 	 *                a base
@@ -1822,13 +1791,11 @@ struct SSkirmishAICallback {
 // BEGINN OBJECT FeatureDef
 	int               (CALLING_CONV *getFeatureDefs)(int skirmishAIId, int* featureDefIds, int featureDefIds_sizeMax); //$ FETCHER:MULTI:IDs:FeatureDef:featureDefIds
 
-//	int (CALLING_CONV *FeatureDef_getId)(int skirmishAIId, int featureDefId);
 
 	const char*       (CALLING_CONV *FeatureDef_getName)(int skirmishAIId, int featureDefId);
 
 	const char*       (CALLING_CONV *FeatureDef_getDescription)(int skirmishAIId, int featureDefId);
 
-	const char*       (CALLING_CONV *FeatureDef_getFileName)(int skirmishAIId, int featureDefId);
 
 	float             (CALLING_CONV *FeatureDef_getContainedResource)(int skirmishAIId, int featureDefId, int resourceId); //$ REF:resourceId->Resource
 
@@ -1871,8 +1838,6 @@ struct SSkirmishAICallback {
 
 	bool              (CALLING_CONV *FeatureDef_isGeoThermal)(int skirmishAIId, int featureDefId);
 
-	/** Name of the FeatureDef that this turns into when killed (not reclaimed). */
-	const char*       (CALLING_CONV *FeatureDef_getDeathFeature)(int skirmishAIId, int featureDefId);
 
 	/**
 	 * Size of the feature along the X axis - in other words: height.
@@ -1912,6 +1877,16 @@ struct SSkirmishAICallback {
 
 	void              (CALLING_CONV *Feature_getPosition)(int skirmishAIId, int featureId, float* return_posF3_out);
 
+	/**
+	 * @return float value of parameter if it's set, defaultValue otherwise.
+	 */
+	float             (CALLING_CONV *Feature_getRulesParamFloat)(int skirmishAIId, int unitId, const char* featureRulesParamName, float defaultValue);
+
+	/**
+	 * @return string value of parameter if it's set, defaultValue otherwise.
+	 */
+	const char*       (CALLING_CONV *Feature_getRulesParamString)(int skirmishAIId, int unitId, const char* featureRulesParamName, const char* defaultValue);
+
 // END OBJECT Feature
 
 
@@ -1927,9 +1902,6 @@ struct SSkirmishAICallback {
 
 	const char*       (CALLING_CONV *WeaponDef_getDescription)(int skirmishAIId, int weaponDefId);
 
-	const char*       (CALLING_CONV *WeaponDef_getFileName)(int skirmishAIId, int weaponDefId);
-
-	const char*       (CALLING_CONV *WeaponDef_getCegTag)(int skirmishAIId, int weaponDefId);
 
 	float             (CALLING_CONV *WeaponDef_getRange)(int skirmishAIId, int weaponDefId);
 
@@ -1975,7 +1947,6 @@ struct SSkirmishAICallback {
 
 	int               (CALLING_CONV *WeaponDef_Damage_getTypes)(int skirmishAIId, int weaponDefId, float* types, int types_sizeMax); //$ ARRAY:types
 
-//	int (CALLING_CONV *WeaponDef_getId)(int skirmishAIId, int weaponDefId);
 
 	float             (CALLING_CONV *WeaponDef_getAreaOfEffect)(int skirmishAIId, int weaponDefId);
 
@@ -2088,24 +2059,10 @@ struct SSkirmishAICallback {
 
 	float             (CALLING_CONV *WeaponDef_getIntensity)(int skirmishAIId, int weaponDefId);
 
-	/** @deprecated only relevant for visualization */
-	float             (CALLING_CONV *WeaponDef_getThickness)(int skirmishAIId, int weaponDefId);
-
-	/** @deprecated only relevant for visualization */
-	float             (CALLING_CONV *WeaponDef_getLaserFlareSize)(int skirmishAIId, int weaponDefId);
-
-	/** @deprecated only relevant for visualization */
-	float             (CALLING_CONV *WeaponDef_getCoreThickness)(int skirmishAIId, int weaponDefId);
-
 	float             (CALLING_CONV *WeaponDef_getDuration)(int skirmishAIId, int weaponDefId);
 
-	/** @deprecated only relevant for visualization */
-	int               (CALLING_CONV *WeaponDef_getLodDistance)(int skirmishAIId, int weaponDefId);
-
 	float             (CALLING_CONV *WeaponDef_getFalloffRate)(int skirmishAIId, int weaponDefId);
 
-	/** @deprecated only relevant for visualization */
-	int               (CALLING_CONV *WeaponDef_getGraphicsType)(int skirmishAIId, int weaponDefId);
 
 	bool              (CALLING_CONV *WeaponDef_isSoundTrigger)(int skirmishAIId, int weaponDefId);
 
@@ -2216,14 +2173,6 @@ struct SSkirmishAICallback {
 	/** Number of frames to delay recharging by after each hit. */
 	int               (CALLING_CONV *WeaponDef_Shield_getRechargeDelay)(int skirmishAIId, int weaponDefId);
 
-	/** The color of the shield when it is at full power. */
-	void              (CALLING_CONV *WeaponDef_Shield_getGoodColor)(int skirmishAIId, int weaponDefId, short* return_colorS3_out);
-
-	/** The color of the shield when it is empty. */
-	void              (CALLING_CONV *WeaponDef_Shield_getBadColor)(int skirmishAIId, int weaponDefId, short* return_colorS3_out);
-
-	/** The shields alpha value. */
-	short             (CALLING_CONV *WeaponDef_Shield_getAlpha)(int skirmishAIId, int weaponDefId);
 
 	/**
 	 * The type of the shield (bitfield).
@@ -2300,6 +2249,24 @@ struct SSkirmishAICallback {
 
 // END OBJECT WeaponDef
 
+
+// BEGINN OBJECT Weapon
+	int               (CALLING_CONV *Unit_Weapon_getDef)(int skirmishAIId, int unitId, int weaponId); //$ REF:RETURN->WeaponDef
+
+	/** Next tick the weapon can fire again. */
+	int               (CALLING_CONV *Unit_Weapon_getReloadFrame)(int skirmishAIId, int unitId, int weaponId);
+
+	/** Time between succesive fires in ticks. */
+	int               (CALLING_CONV *Unit_Weapon_getReloadTime)(int skirmishAIId, int unitId, int weaponId);
+
+	float             (CALLING_CONV *Unit_Weapon_getRange)(int skirmishAIId, int unitId, int weaponId);
+
+	bool              (CALLING_CONV *Unit_Weapon_isShieldEnabled)(int skirmishAIId, int unitId, int weaponId);
+
+	float             (CALLING_CONV *Unit_Weapon_getShieldPower)(int skirmishAIId, int unitId, int weaponId);
+
+// END OBJECT Weapon
+
 	bool              (CALLING_CONV *Debug_GraphDrawer_isEnabled)(int skirmishAIId);
 
 };
