@@ -26,12 +26,10 @@ ClientSetup::ClientSetup()
 
 void ClientSetup::SanityCheck()
 {
-	if (myPlayerName.empty()) myPlayerName = UnnamedPlayerName;
-	StringReplaceInPlace(myPlayerName, ' ', '_');
+	if (myPlayerName.empty())
+		myPlayerName = UnnamedPlayerName;
 
-	if (hostIP == "none") {
-		hostIP = "";
-	}
+	StringReplaceInPlace(myPlayerName, ' ', '_');
 }
 
 
@@ -40,7 +38,8 @@ void ClientSetup::LoadFromStartScript(const std::string& setup)
 	TdfParser file(setup.c_str(), setup.length());
 
 	if (!file.SectionExist("GAME")) {
-		throw content_error("GAME-section does not exist in script.txt");
+		LOG_L(L_ERROR, "[ClientSetup::%s] GAME-section missing from setup-script \"%s\"", __func__, setup.c_str());
+		throw content_error("setup-script error");
 	}
 
 	// Technical parameters
@@ -50,26 +49,27 @@ void ClientSetup::LoadFromStartScript(const std::string& setup)
 	file.GetDef(myPlayerName, "", "GAME\\MyPlayerName");
 	file.GetDef(myPasswd,     "", "GAME\\MyPasswd");
 
-	if (!file.GetValue(isHost, "GAME\\IsHost")) {
-		LOG_L(L_WARNING, "script.txt is missing the IsHost-entry. Assuming this is a client.");
-	}
+	if (!file.GetValue(isHost, "GAME\\IsHost"))
+		LOG_L(L_WARNING, "[ClientSetup::%s] IsHost-entry missing from setup-script; assuming this is a client", __func__);
+
 #ifdef DEDICATED
-	if (!isHost) {
-		handleerror(NULL, "Error: Dedicated server needs to be host, but the start script does not have \"IsHost=1\".", "Start script error", MBF_OK|MBF_EXCL);
-	}
+	if (!isHost)
+		handleerror(nullptr, "setup-script error", "dedicated server needs \"IsHost=1\" in GAME-section", MBF_OK | MBF_EXCL);
 #endif
 
-	//FIXME WTF
-	std::string sourceport, autohostip, autohostport;
-	if (file.SGetValue(sourceport, "GAME\\SourcePort")) {
+	// FIXME WTF
+	std::string sourceport;
+	std::string autohostip;
+	std::string autohostport;
+
+	if (file.SGetValue(sourceport, "GAME\\SourcePort"))
 		configHandler->SetString("SourcePort", sourceport, true);
-	}
-	if (file.SGetValue(autohostip, "GAME\\AutohostIP")) {
+
+	if (file.SGetValue(autohostip, "GAME\\AutohostIP"))
 		configHandler->SetString("AutohostIP", autohostip, true);
-	}
-	if (file.SGetValue(autohostport, "GAME\\AutohostPort")) {
+
+	if (file.SGetValue(autohostport, "GAME\\AutohostPort"))
 		configHandler->SetString("AutohostPort", autohostport, true);
-	}
 
 	file.GetDef(saveFile, "", "GAME\\SaveFile");
 	file.GetDef(demoFile, "", "GAME\\DemoFile");
