@@ -27,6 +27,7 @@ bool LuaConstGL::PushEntries(lua_State* L)
 	PUSH_GL(QUADS);
 	PUSH_GL(QUAD_STRIP);
 	PUSH_GL(POLYGON);
+	PUSH_GL(PATCHES);
 
 	// BlendingFactorDest
 	PUSH_GL(ZERO);
@@ -182,6 +183,23 @@ bool LuaConstGL::PushEntries(lua_State* L)
 	PUSH_GL(LINEAR_ATTENUATION);
 	PUSH_GL(QUADRATIC_ATTENUATION);
 
+	// Shader types
+	PUSH_GL(VERTEX_SHADER);
+	PUSH_GL(TESS_CONTROL_SHADER);
+	PUSH_GL(TESS_EVALUATION_SHADER);
+	PUSH_GL(GEOMETRY_SHADER);
+	PUSH_GL(FRAGMENT_SHADER);
+
+	// Geometry shader parameters
+	PUSH_GL(GEOMETRY_INPUT_TYPE);
+	PUSH_GL(GEOMETRY_OUTPUT_TYPE);
+	PUSH_GL(GEOMETRY_VERTICES_OUT);
+
+	// Tesselation control shader parameters
+	PUSH_GL(PATCH_VERTICES);
+	PUSH_GL(PATCH_DEFAULT_OUTER_LEVEL);
+	PUSH_GL(PATCH_DEFAULT_INNER_LEVEL);
+
 	return true;
 }
 
