@@ -9,10 +9,6 @@ struct lua_State;
 
 
 class LuaFonts {
-	public:
-		LuaFonts();
-		~LuaFonts();
-
 	public:
 		static bool PushEntries(lua_State* L);
 
@@ -28,11 +24,16 @@ class LuaFonts {
 		static int DeleteFont(lua_State* L);
 
 	private: // userdata call-outs
-		static int Print(lua_State* L);
-
 		static int Begin(lua_State* L);
+		static int Print(lua_State* L);
 		static int End(lua_State* L);
 
+		static int WorldBegin(lua_State* L);
+		static int WorldPrint(lua_State* L);
+		static int WorldEnd(lua_State* L);
+
+		static int DrawBuffered(lua_State* L);
+
 		static int WrapText(lua_State* L);
 
 		static int GetTextWidth(lua_State* L);
@@ -42,6 +43,9 @@ class LuaFonts {
 		static int SetOutlineColor(lua_State* L);
 		static int SetAutoOutlineColor(lua_State* L);
 
+		static int SetTextViewMatrix(lua_State* L);
+		static int SetTextProjMatrix(lua_State* L);
+
 		static int BindTexture(lua_State* L);
 };
 
