@@ -1,7 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "lib/streflop/streflop_cond.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 #include "LuaMathExtra.h"
 #include "LuaInclude.h"
 #include "LuaUtils.h"
