@@ -11,10 +11,13 @@ struct lua_State;
 template<LuaObjType T> class LuaObjectRendering;
 
 class LuaObjectRenderingImpl {
+#if __GNUC__ == 11
+public:
+#else
 private:
 	friend class LuaObjectRendering<LUAOBJ_UNIT>;
 	friend class LuaObjectRendering<LUAOBJ_FEATURE>;
-
+#endif
 	static void CreateMatRefMetatable(lua_State* L);
 	static void PushFunction(lua_State* L, int (*fnPntr)(lua_State*), const char* fnName);
 
@@ -33,15 +36,20 @@ private:
 	static int SetLODLength(lua_State* L);
 	static int SetLODDistance(lua_State* L);
 
-	static int SetPieceList(lua_State* L);
+	static int SetPieceList(lua_State* L) { return 0; }
 
 	static int GetMaterial(lua_State* L);
 	static int SetMaterial(lua_State* L);
 	static int SetMaterialLastLOD(lua_State* L);
-	static int SetMaterialDisplayLists(lua_State* L);
+	static int SetMaterialDisplayLists(lua_State* L) { return 0; }
+	static int SetDeferredMaterialUniform(lua_State* L);
+	static int SetForwardMaterialUniform(lua_State* L);
+	static int ClearDeferredMaterialUniform(lua_State* L);
+	static int ClearForwardMaterialUniform(lua_State* L);
 
 	static int SetUnitLuaDraw(lua_State* L);
 	static int SetFeatureLuaDraw(lua_State* L);
+	static int SetProjectileLuaDraw(lua_State* L);
 
 	static int Debug(lua_State* L);
 
@@ -71,10 +79,16 @@ public:
 		PUSH_FUNCTION(SetMaterialLastLOD);
 		PUSH_FUNCTION(SetMaterialDisplayLists);
 
+		PUSH_FUNCTION(SetDeferredMaterialUniform);
+		PUSH_FUNCTION(SetForwardMaterialUniform);
+		PUSH_FUNCTION(ClearDeferredMaterialUniform);
+		PUSH_FUNCTION(ClearForwardMaterialUniform);
+
 		PUSH_FUNCTION(SetPieceList);
 
 		PUSH_FUNCTION(SetUnitLuaDraw);
 		PUSH_FUNCTION(SetFeatureLuaDraw);
+		PUSH_FUNCTION(SetProjectileLuaDraw);
 
 		PUSH_FUNCTION(Debug);
 
@@ -143,6 +157,32 @@ private:
 		return ret;
 	}
 
+	static int SetDeferredMaterialUniform(lua_State* L) {
+		LuaObjectRenderingImpl::PushObjectType(T);
+		const int ret = LuaObjectRenderingImpl::SetDeferredMaterialUniform(L);
+		LuaObjectRenderingImpl::PopObjectType();
+		return ret;
+	}
+	static int SetForwardMaterialUniform(lua_State* L) {
+		LuaObjectRenderingImpl::PushObjectType(T);
+		const int ret = LuaObjectRenderingImpl::SetForwardMaterialUniform(L);
+		LuaObjectRenderingImpl::PopObjectType();
+		return ret;
+	}
+
+	static int ClearDeferredMaterialUniform(lua_State* L) {
+		LuaObjectRenderingImpl::PushObjectType(T);
+		const int ret = LuaObjectRenderingImpl::ClearDeferredMaterialUniform(L);
+		LuaObjectRenderingImpl::PopObjectType();
+		return ret;
+	}
+	static int ClearForwardMaterialUniform(lua_State* L) {
+		LuaObjectRenderingImpl::PushObjectType(T);
+		const int ret = LuaObjectRenderingImpl::ClearForwardMaterialUniform(L);
+		LuaObjectRenderingImpl::PopObjectType();
+		return ret;
+	}
+
 
 	static int SetUnitLuaDraw(lua_State* L) {
 		if (T != LUAOBJ_UNIT)
@@ -156,6 +196,10 @@ private:
 
 		return (LuaObjectRenderingImpl::SetFeatureLuaDraw(L));
 	}
+	// NB: value of T is not relevant here, slightly foreign callout
+	static int SetProjectileLuaDraw(lua_State* L) {
+		return (LuaObjectRenderingImpl::SetProjectileLuaDraw(L));
+	}
 
 
 	static int Debug(lua_State* L) {
