@@ -4,7 +4,7 @@
 #include "LuaParser.h"
 
 #include <algorithm>
-#include <limits.h>
+#include <climits>
 
 #include "lib/streflop/streflop_cond.h"
 
@@ -12,20 +12,21 @@
 #include "System/float4.h"
 #include "LuaInclude.h"
 
+#include "LuaConstGame.h"
 #include "LuaConstEngine.h"
 #include "LuaIO.h"
+#include "LuaVFS.h"
 #include "LuaUtils.h"
 
-#include "Game/GameVersion.h"
 #include "Sim/Misc/GlobalSynced.h" // gsRNG
 #include "System/Log/ILog.h"
 #include "System/FileSystem/FileHandler.h"
 #include "System/Misc/SpringTime.h"
 #include "System/ContainerUtil.h"
+#include "System/TimeProfiler.h"
 #include "System/ScopedFPUSettings.h"
 #include "System/StringUtil.h"
 
-
 LuaParser* GetLuaParser(lua_State* L) {
 	assert(GetLuaContextData(L)->parser != nullptr);
 	return GetLuaContextData(L)->parser;
@@ -38,7 +39,7 @@ LuaParser* GetLuaParser(lua_State* L) {
 //  LuaParser
 //
 
-LuaParser::LuaParser(const string& _fileName, const string& _fileModes, const string& _accessModes, const boolean& synced)
+LuaParser::LuaParser(const std::string& _fileName, const std::string& _fileModes, const std::string& _accessModes, const boolean& synced, const boolean& setup)
 	: fileName(_fileName)
 	, fileModes(_fileModes)
 	, accessModes(_accessModes)
@@ -49,23 +50,22 @@ LuaParser::LuaParser(const string& _fileName, const string& _fileModes, const st
 	, rootRef(LUA_NOREF)
 	, currentRef(LUA_NOREF)
 
-	, valid(false)
 	, lowerKeys(true)
 	, lowerCppKeys(true)
 {
 	// be on the safe side
+	// D.synced = synced.b;
 	D.synced = true;
 	D.parser = this;
 
-	if ((L = LUA_OPEN(&D)) != nullptr) {
-		SetupEnv(synced.b);
-	}
+	if (!setup.b)
+		return;
+
+	SetupLua(synced.b, false);
 }
 
-LuaParser::LuaParser(const string& _textChunk, const string& _accessModes, const boolean& synced)
-	: fileName("")
-	, fileModes("")
-	, textChunk(_textChunk)
+LuaParser::LuaParser(const std::string& _textChunk, const std::string& _accessModes, int, const boolean& synced, const boolean& setup)
+	: textChunk(_textChunk)
 	, accessModes(_accessModes)
 
 	, D(false, false)
@@ -74,17 +74,18 @@ LuaParser::LuaParser(const string& _textChunk, const string& _accessModes, const
 	, rootRef(LUA_NOREF)
 	, currentRef(LUA_NOREF)
 
-	, valid(false)
 	, lowerKeys(true)
 	, lowerCppKeys(true)
 {
 	// be on the safe side
+	// D.synced = synced.b;
 	D.synced = true;
 	D.parser = this;
 
-	if ((L = LUA_OPEN(&D)) != nullptr) {
-		SetupEnv(synced.b);
-	}
+	if (!setup.b)
+		return;
+
+	SetupLua(synced.b, false);
 }
 
 
@@ -106,7 +107,16 @@ LuaParser::~LuaParser()
 	}
 }
 
-void LuaParser::SetupEnv(bool synced)
+
+void LuaParser::SetupLua(bool isSyncedCtxt, bool isDefsParser)
+{
+	if ((L = LUA_OPEN(&D)) == nullptr)
+		return;
+
+	SetupEnv(isSyncedCtxt, isDefsParser);
+}
+
+void LuaParser::SetupEnv(bool isSyncedCtxt, bool isDefsParser)
 {
 	LUA_OPEN_LIB(L, luaopen_base);
 	LUA_OPEN_LIB(L, luaopen_math);
@@ -128,7 +138,7 @@ void LuaParser::SetupEnv(bool synced)
 
 	{
 		lua_getglobal(L, "math");
-		if (synced) {
+		if (isSyncedCtxt) {
 			LuaPushNamedCFunc(L, "random", Random);
 			LuaPushNamedCFunc(L, "randomseed", RandomSeed);
 		} else {
@@ -146,11 +156,30 @@ void LuaParser::SetupEnv(bool synced)
 	AddFunc("TimeCheck", TimeCheck);
 	EndTable();
 
+	GetTable("Script");
+	AddFunc("IsEngineMinVersion", LuaUtils::IsEngineMinVersion);
+	EndTable();
+
+	#if (!defined(UNITSYNC) && !defined(DEDICATED))
+	if (isDefsParser) {
+		// irrelevant for most LuaParsers except defsParser
+		// adding it to others can cause infinite recursion
+		// (e.g. when scanning generated virtual archives)
+		GetTable("Game");
+		LuaConstGame::PushEntries(L);
+		EndTable();
+	}
+	#endif
+
 	GetTable("Engine");
 	LuaConstEngine::PushEntries(L);
 	EndTable();
 
 	GetTable("VFS");
+	#if (!defined(UNITSYNC) && !defined(DEDICATED))
+	// no LuaConstVFS, but this will do
+	LuaVFS::PushCommon(L);
+	#endif
 	AddFunc("DirList",    DirList);
 	AddFunc("SubDirs",    SubDirs);
 	AddFunc("Include",    Include);
@@ -169,43 +198,46 @@ void LuaParser::SetupEnv(bool synced)
 bool LuaParser::Execute()
 {
 	if (!IsValid()) {
-		errorLog = "could not initialize LUA library";
+		errorLog = "could not initialize Lua library";
 		return false;
 	}
 
-	rootRef = LUA_NOREF;
-
+	assert(rootRef == LUA_NOREF);
 	assert(initDepth == 0);
 	initDepth = -1;
 
-	string code;
-	string codeLabel;
+	std::string code;
+	std::string codeLabel;
 
 	if (!textChunk.empty()) {
 		code = textChunk;
 		codeLabel = "text chunk";
 	}
 	else if (!fileName.empty()) {
-		codeLabel = fileName;
-		CFileHandler fh(fileName, fileModes);
+		CFileHandler fh(codeLabel = fileName, fileModes);
+
 		if (!fh.LoadStringData(code)) {
 			errorLog = "could not open file: " + fileName;
+
 			LUA_CLOSE(&L);
 			return false;
 		}
 	}
 	else {
 		errorLog = "invalid format or empty file";
+
 		LUA_CLOSE(&L);
 		return false;
 	}
 
-	int error = luaL_loadbuffer(L, code.c_str(), code.size(), codeLabel.c_str());
+	char errorBuf[4096] = {0};
+	int errorNum = 0;
 
-	if (error != 0) {
-		errorLog = lua_tostring(L, -1);
-		LOG_L(L_ERROR, "%i, %s, %s", error, codeLabel.c_str(), errorLog.c_str());
+	if ((errorNum = luaL_loadbuffer(L, code.c_str(), code.size(), codeLabel.c_str())) != 0) {
+		SNPRINTF(errorBuf, sizeof(errorBuf), "[loadbuf] error %d (\"%s\") in %s", errorNum, lua_tostring(L, -1), codeLabel.c_str());
 		LUA_CLOSE(&L);
+
+		errorLog = errorBuf;
 		return false;
 	}
 
@@ -215,21 +247,33 @@ bool LuaParser::Execute()
 
 		// do not signal floating point exceptions in user Lua code
 		ScopedDisableFpuExceptions fe;
-		error = lua_pcall(L, 0, 1, 0);
 
-		if (error != 0) {
-			errorLog = lua_tostring(L, -1);
-			LOG_L(L_ERROR, "%i, %s, %s", error, fileName.c_str(), errorLog.c_str());
+		if ((errorNum = lua_pcall(L, 0, 1, 0)) != 0) {
+			SNPRINTF(errorBuf, sizeof(errorBuf), "[pcall] error %d (\"%s\") in %s", errorNum, lua_tostring(L, -1), fileName.c_str());
 			LUA_CLOSE(&L);
+
+			errorLog = errorBuf;
 			return false;
 		}
 
+		#if 0
 		if (!lua_istable(L, 1)) {
-			errorLog = "missing return table from " + fileName;
-			LOG_L(L_ERROR, "missing return table from %s", fileName.c_str());
+			errorLog = "no return table from " + fileName;
+
 			LUA_CLOSE(&L);
 			return false;
 		}
+		#else
+		// make not returning a table (which may or may not be intentional)
+		// equivalent to returning an empty one; leave it to callers whether
+		// they consider this an error
+		if (!lua_istable(L, 1)) {
+			lua_pop(L, lua_gettop(L));
+			lua_newtable(L);
+
+			errorLog = "no return table from " + fileName;
+		}
+		#endif
 
 		if (lowerKeys)
 			LuaUtils::LowerKeys(L, 1);
@@ -239,8 +283,8 @@ bool LuaParser::Execute()
 
 	rootRef = luaL_ref(L, LUA_REGISTRYINDEX);
 	lua_settop(L, 0);
-	valid = true;
-	return true;
+
+	return (valid = true);
 }
 
 
@@ -269,7 +313,7 @@ void LuaParser::PushParam()
 }
 
 
-void LuaParser::GetTable(const string& name, bool overwrite)
+void LuaParser::GetTable(const std::string& name, bool overwrite)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -278,8 +322,7 @@ void LuaParser::GetTable(const string& name, bool overwrite)
 
 	if (overwrite) {
 		lua_newtable(L);
-	}
-	else {
+	} else {
 		lua_pushsstring(L, name);
 		lua_gettable(L, (initDepth == 0) ? LUA_GLOBALSINDEX : -3);
 		if (!lua_istable(L, -1)) {
@@ -326,7 +369,7 @@ void LuaParser::EndTable()
 
 /******************************************************************************/
 
-void LuaParser::AddFunc(const string& key, int (*func)(lua_State*))
+void LuaParser::AddFunc(const std::string& key, int (*func)(lua_State*))
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -343,7 +386,7 @@ void LuaParser::AddFunc(const string& key, int (*func)(lua_State*))
 }
 
 
-void LuaParser::AddInt(const string& key, int value)
+void LuaParser::AddInt(const std::string& key, int value)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -353,7 +396,7 @@ void LuaParser::AddInt(const string& key, int value)
 }
 
 
-void LuaParser::AddBool(const string& key, bool value)
+void LuaParser::AddBool(const std::string& key, bool value)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -363,7 +406,7 @@ void LuaParser::AddBool(const string& key, bool value)
 }
 
 
-void LuaParser::AddFloat(const string& key, float value)
+void LuaParser::AddFloat(const std::string& key, float value)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -373,7 +416,7 @@ void LuaParser::AddFloat(const string& key, float value)
 }
 
 
-void LuaParser::AddString(const string& key, const string& value)
+void LuaParser::AddString(const std::string& key, const std::string& value)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -429,7 +472,7 @@ void LuaParser::AddFloat(int key, float value)
 }
 
 
-void LuaParser::AddString(int key, const string& value)
+void LuaParser::AddString(int key, const std::string& value)
 {
 	if (!IsValid() || (initDepth < 0))
 		return;
@@ -447,24 +490,27 @@ void LuaParser::AddString(int key, const string& value)
 
 int LuaParser::TimeCheck(lua_State* L)
 {
+	#if (!defined(UNITSYNC) && !defined(DEDICATED))
 	if (!lua_isstring(L, 1) || !lua_isfunction(L, 2))
 		luaL_error(L, "Invalid arguments to TimeCheck('string', func, ...)");
 
-	const string name = lua_tostring(L, 1);
-	lua_remove(L, 1);
+	{
+		ScopedOnceTimer timer(lua_tostring(L, 1));
 
-	const spring_time startTime = spring_gettime();
+		lua_remove(L, 1);
 
-	if (lua_pcall(L, lua_gettop(L) - 1, LUA_MULTRET, 0) != 0) {
-		const string errmsg = lua_tostring(L, -1);
-		lua_pop(L, 1);
-		luaL_error(L, errmsg.c_str());
+		if (lua_pcall(L, lua_gettop(L) - 1, LUA_MULTRET, 0) != 0) {
+			const std::string errmsg = lua_tostring(L, -1);
+
+			lua_pop(L, 1);
+			luaL_error(L, errmsg.c_str());
+		}
 	}
 
-	const spring_time endTime = spring_gettime();
-
-	LOG("%s %ldms", name.c_str(), (long int) spring_tomsecs(endTime - startTime));
 	return lua_gettop(L);
+	#else
+	return 0;
+	#endif
 }
 
 
@@ -543,7 +589,7 @@ int LuaParser::Include(lua_State* L)
  		lua_error(L);
 	}
 
-	string code;
+	std::string code;
 	if (!fh.LoadStringData(code)) {
 		char buf[1024];
 		SNPRINTF(buf, sizeof(buf), "Include() could not load '%s'\n", filename.c_str());
@@ -563,7 +609,7 @@ int LuaParser::Include(lua_State* L)
 	if (lua_istable(L, 2)) {
 		lua_pushvalue(L, 2); // user fenv
 	} else {
-		LuaUtils::PushCurrentFuncEnv(L, __FUNCTION__);
+		LuaUtils::PushCurrentFuncEnv(L, __func__);
 	}
 
 	// set the include fenv to the current function's fenv
@@ -595,7 +641,7 @@ int LuaParser::LoadFile(lua_State* L)
 {
 	const LuaParser* currentParser = GetLuaParser(L);
 
-	const std::string&  filename = luaL_checkstring(L, 1);
+	const std::string& filename = luaL_checkstring(L, 1);
 
 	if (!LuaIO::IsSimplePath(filename))
 		return 0;
@@ -608,7 +654,7 @@ int LuaParser::LoadFile(lua_State* L)
 		lua_pushstring(L, "missing file");
 		return 2;
 	}
-	string data;
+	std::string data;
 	if (!fh.LoadStringData(data)) {
 		lua_pushnil(L);
 		lua_pushstring(L, "could not load data");
@@ -627,12 +673,15 @@ int LuaParser::FileExists(lua_State* L)
 {
 	const LuaParser* currentParser = GetLuaParser(L);
 
-	const std::string&  filename = luaL_checkstring(L, 1);
+	const std::string& filename = luaL_checkstring(L, 1);
+	// filter any modes not within the parser's allowed set
+	const std::string& argModes = luaL_optstring(L, 2, currentParser->accessModes.c_str());
+	const std::string& vfsModes = CFileHandler::AllowModes(argModes, currentParser->accessModes);
 
 	if (!LuaIO::IsSimplePath(filename))
 		return 0;
 
-	lua_pushboolean(L, CFileHandler::FileExists(filename, currentParser->accessModes));
+	lua_pushboolean(L, CFileHandler::FileExists(filename, vfsModes));
 	return 1;
 }
 
@@ -655,8 +704,8 @@ int LuaParser::DontMessWithMyCase(lua_State* L)
 LuaTable::LuaTable()
 : path(""),
   isValid(false),
-  parser(NULL),
-  L(NULL),
+  parser(nullptr),
+  L(nullptr),
   refnum(LUA_NOREF)
 {
 }
@@ -664,7 +713,7 @@ LuaTable::LuaTable()
 
 LuaTable::LuaTable(LuaParser* _parser)
 {
-	assert(_parser != NULL);
+	assert(_parser != nullptr);
 
 	isValid = _parser->IsValid();
 	path    = "ROOT";
@@ -766,9 +815,9 @@ LuaTable LuaTable::SubTable(int key) const
 }
 
 
-LuaTable LuaTable::SubTable(const string& mixedKey) const
+LuaTable LuaTable::SubTable(const std::string& mixedKey) const
 {
-	const string key = !(parser ? parser->lowerCppKeys : true) ? mixedKey : StringToLower(mixedKey);
+	const std::string key = !((parser != nullptr)? parser->lowerCppKeys : true) ? mixedKey : StringToLower(mixedKey);
 
 	LuaTable subTable;
 	subTable.path = path + "." + key;
@@ -793,37 +842,34 @@ LuaTable LuaTable::SubTable(const string& mixedKey) const
 }
 
 
-LuaTable LuaTable::SubTableExpr(const string& expr) const
+LuaTable LuaTable::SubTableExpr(const std::string& expr) const
 {
-	if (expr.empty()) {
+	if (expr.empty())
 		return LuaTable(*this);
-	}
-	if (!isValid) {
-		return LuaTable();
-	}
 
-	string::size_type endPos;
+	if (!isValid)
+		return LuaTable();
+
+	std::string::size_type endPos;
 	LuaTable nextTable;
 
 	if (expr[0] == '[') { // numeric key
-		endPos = expr.find(']');
-		if (endPos == string::npos) {
+		if ((endPos = expr.find(']')) == std::string::npos)
 			return LuaTable(); // missing brace
-		}
+
 		const char* startPtr = expr.c_str() + 1; // skip the '['
 		char* endPtr;
 		const int index = strtol(startPtr, &endPtr, 10);
-		if (endPtr == startPtr) {
+
+		if (endPtr == startPtr)
 			return LuaTable(); // invalid index
-		}
+
 		endPos++; // eat the ']'
 		nextTable = SubTable(index);
-	}
-	else { // string key
-		endPos = expr.find_first_of(".[");
-		if (endPos == string::npos) {
+	} else { // string key
+		if ((endPos = expr.find_first_of(".[")) == std::string::npos)
 			return SubTable(expr);
-		}
+
 		nextTable = SubTable(expr.substr(0, endPos));
 	}
 
@@ -898,25 +944,25 @@ bool LuaTable::PushValue(int key) const
 }
 
 
-bool LuaTable::PushValue(const string& mixedKey) const
+bool LuaTable::PushValue(const std::string& mixedKey) const
 {
-	const string key = !(parser ? parser->lowerCppKeys : true) ? mixedKey : StringToLower(mixedKey);
-	if (!PushTable()) {
+	const std::string key = !(parser ? parser->lowerCppKeys : true) ? mixedKey : StringToLower(mixedKey);
+
+	if (!PushTable())
 		return false;
-	}
 
 	const int top = lua_gettop(L);
 
-	if (key.find(".") != std::string::npos) {
+	if (key.find('.') != std::string::npos) {
 		// nested key (e.g. "subtable.subsub.mahkey")
 		size_t lastpos = 0;
-		size_t dotpos = key.find(".");
+		size_t dotpos = key.find('.');
 
 		lua_pushvalue(L, -1);
 		do {
 			const std::string subTableName = key.substr(lastpos, dotpos);
 			lastpos = dotpos + 1;
-			dotpos = key.find(".", lastpos);
+			dotpos = key.find('.', lastpos);
 
 			lua_pushsstring(L, subTableName);
 			lua_gettable(L, -2);
@@ -987,19 +1033,19 @@ bool LuaTable::PushValue(const string& mixedKey) const
 
 bool LuaTable::KeyExists(int key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return false;
-	}
+
 	lua_pop(L, 1);
 	return true;
 }
 
 
-bool LuaTable::KeyExists(const string& key) const
+bool LuaTable::KeyExists(const std::string& key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return false;
-	}
+
 	lua_pop(L, 1);
 	return true;
 }
@@ -1013,9 +1059,9 @@ bool LuaTable::KeyExists(const string& key) const
 
 LuaTable::DataType LuaTable::GetType(int key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return NIL;
-	}
+
 	const int type = lua_type(L, -1);
 	lua_pop(L, 1);
 
@@ -1029,11 +1075,11 @@ LuaTable::DataType LuaTable::GetType(int key) const
 }
 
 
-LuaTable::DataType LuaTable::GetType(const string& key) const
+LuaTable::DataType LuaTable::GetType(const std::string& key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return NIL;
-	}
+
 	const int type = lua_type(L, -1);
 	lua_pop(L, 1);
 
@@ -1055,29 +1101,29 @@ LuaTable::DataType LuaTable::GetType(const string& key) const
 
 int LuaTable::GetLength() const
 {
-	if (!PushTable()) {
+	if (!PushTable())
 		return 0;
-	}
+
 	return lua_objlen(L, -1);
 }
 
 
 int LuaTable::GetLength(int key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return 0;
-	}
+
 	const int len = lua_objlen(L, -1);
 	lua_pop(L, 1);
 	return len;
 }
 
 
-int LuaTable::GetLength(const string& key) const
+int LuaTable::GetLength(const std::string& key) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return 0;
-	}
+
 	const int len = lua_objlen(L, -1);
 	lua_pop(L, 1);
 	return len;
@@ -1097,29 +1143,108 @@ bool LuaTable::GetKeys(std::vector<int>& data) const
 
 	const int table = lua_gettop(L);
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwnumber(L, -2)) {
-			const int value = lua_toint(L, -2);
-			data.push_back(value);
-		}
+		if (!lua_israwnumber(L, -2))
+			continue;
+
+		data.push_back(lua_toint(L, -2));
 	}
+
 	std::stable_sort(data.begin(), data.end());
 	return true;
 }
 
-
-bool LuaTable::GetKeys(std::vector<string>& data) const
+bool LuaTable::GetKeys(std::vector<std::string>& data) const
 {
 	if (!PushTable())
 		return false;
 
 	const int table = lua_gettop(L);
+
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwstring(L, -2)) {
-			const string value = lua_tostring(L, -2);
-			data.push_back(value);
+		if (lua_israwstring(L, -2))
+			data.emplace_back(lua_tostring(L, -2));
+	}
+
+	std::stable_sort(data.begin(), data.end());
+	return true;
+}
+
+
+
+bool LuaTable::GetPairs(std::vector<std::pair<int, std::string>>& data) const
+{
+	if (!PushTable())
+		return false;
+
+	const int table = lua_gettop(L);
+
+	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
+		if (!lua_israwnumber(L, -2) || !lua_isstring(L, -1))
+			continue;
+
+		if (lua_isstring(L, -1)) {
+			data.emplace_back(lua_toint(L, -2), lua_tostring(L, -1));
+			continue;
+		}
+		if (lua_isboolean(L, -1)) {
+			data.emplace_back(lua_toint(L, -2), lua_toboolean(L, -1) ? "1" : "0");
+			continue;
 		}
 	}
-	std::stable_sort(data.begin(), data.end());
+
+	using T = std::remove_reference<decltype(data)>::type;
+	using P = T::value_type;
+
+	std::stable_sort(data.begin(), data.end(), [](const P& a, const P& b) { return (a.first < b.first); });
+	return true;
+}
+
+bool LuaTable::GetPairs(std::vector<std::pair<std::string, float>>& data) const
+{
+	if (!PushTable())
+		return false;
+
+	const int table = lua_gettop(L);
+
+	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
+		if (!lua_israwstring(L, -2) || !lua_isnumber(L, -1))
+			continue;
+
+		data.emplace_back(lua_tostring(L, -2), lua_tonumber(L, -1));
+	}
+
+	using T = std::remove_reference<decltype(data)>::type;
+	using P = T::value_type;
+
+	std::stable_sort(data.begin(), data.end(), [](const P& a, const P& b) { return (a.first < b.first); });
+	return true;
+}
+
+bool LuaTable::GetPairs(std::vector<std::pair<std::string, std::string>>& data) const
+{
+	if (!PushTable())
+		return false;
+
+	const int table = lua_gettop(L);
+
+	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
+		if (!lua_israwstring(L, -2))
+			continue;
+
+		if (lua_isstring(L, -1)) { // includes numbers
+			data.emplace_back(lua_tostring(L, -2), lua_tostring(L, -1));
+			continue;
+		}
+		if (lua_isboolean(L, -1)) {
+			data.emplace_back(lua_tostring(L, -2), lua_toboolean(L, -1) ? "1" : "0");
+			continue;
+		}
+	}
+
+	using T = std::remove_reference<decltype(data)>::type;
+	using P = T::value_type;
+
+	std::stable_sort(data.begin(), data.end(), [](const P& a, const P& b) { return (a.first < b.first); });
 	return true;
 }
 
@@ -1136,76 +1261,79 @@ bool LuaTable::GetMap(spring::unordered_map<int, float>& data) const
 		return false;
 
 	const int table = lua_gettop(L);
+
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwnumber(L, -2) && lua_isnumber(L, -1)) {
-			const int   key   =   lua_toint(L, -2);
-			const float value = lua_tonumber(L, -1);
-			data[key] = value;
-		}
+		if (!lua_israwnumber(L, -2) || !lua_isnumber(L, -1))
+			continue;
+
+		data[lua_toint(L, -2)] = lua_tonumber(L, -1);
 	}
+
 	return true;
 }
 
-
-bool LuaTable::GetMap(spring::unordered_map<int, string>& data) const
+bool LuaTable::GetMap(spring::unordered_map<int, std::string>& data) const
 {
 	if (!PushTable())
 		return false;
 
 	const int table = lua_gettop(L);
+
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwnumber(L, -2) && lua_isstring(L, -1)) {
-			if (lua_isstring(L, -1)) {
-				const int    key   = lua_toint(L, -2);
-				const string value = lua_tostring(L, -1);
-				data[key] = value;
-			} else if (lua_isboolean(L, -1)) {
-				const int    key   = lua_toint(L, -2);
-				const string value = lua_toboolean(L, -1) ? "1" : "0";
-				data[key] = value;
-			}
+		if (!lua_israwnumber(L, -2) || !lua_isstring(L, -1))
+			continue;
+
+		if (lua_isstring(L, -1)) {
+			data[lua_toint(L, -2)] = lua_tostring(L, -1);
+			continue;
+		}
+		if (lua_isboolean(L, -1)) {
+			data[lua_toint(L, -2)] = lua_toboolean(L, -1) ? "1" : "0";
+			continue;
 		}
 	}
+
 	return true;
 }
 
-
-bool LuaTable::GetMap(spring::unordered_map<string, float>& data) const
+bool LuaTable::GetMap(spring::unordered_map<std::string, float>& data) const
 {
 	if (!PushTable())
 		return false;
 
 	const int table = lua_gettop(L);
+
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwstring(L, -2) && lua_isnumber(L, -1)) {
-			const string key   = lua_tostring(L, -2);
-			const float  value = lua_tonumber(L, -1);
-			data[key] = value;
-		}
+		if (!lua_israwstring(L, -2) || !lua_isnumber(L, -1))
+			continue;
+
+		data[lua_tostring(L, -2)] = lua_tonumber(L, -1);
 	}
+
 	return true;
 }
 
-
-bool LuaTable::GetMap(spring::unordered_map<string, string>& data) const
+bool LuaTable::GetMap(spring::unordered_map<std::string, std::string>& data) const
 {
 	if (!PushTable())
 		return false;
 
 	const int table = lua_gettop(L);
+
 	for (lua_pushnil(L); lua_next(L, table) != 0; lua_pop(L, 1)) {
-		if (lua_israwstring(L, -2)) {
-			if (lua_isstring(L, -1)) { // includes numbers
-				const string key   = lua_tostring(L, -2);
-				const string value = lua_tostring(L, -1);
-				data[key] = value;
-			} else if (lua_isboolean(L, -1)) {
-				const string key   = lua_tostring(L, -2);
-				const string value = lua_toboolean(L, -1) ? "1" : "0";
-				data[key] = value;
-			}
+		if (!lua_israwstring(L, -2))
+			continue;
+
+		if (lua_isstring(L, -1)) { // includes numbers
+			data[lua_tostring(L, -2)] = lua_tostring(L, -1);
+			continue;
+		}
+		if (lua_isboolean(L, -1)) {
+			data[lua_tostring(L, -2)] = lua_toboolean(L, -1) ? "1" : "0";
+			continue;
 		}
 	}
+
 	return true;
 }
 
@@ -1243,12 +1371,10 @@ static bool ParseFloat3(lua_State* L, int index, float3& value)
 		}
 	}
 	else if (lua_isstring(L, index)) {
-		const int count = sscanf(lua_tostring(L, index), "%f %f %f",
-		                         &value.x, &value.y, &value.z);
-		if (count == 3) {
+		if (sscanf(lua_tostring(L, index), "%f %f %f", &value.x, &value.y, &value.z) == 3)
 			return true;
-		}
 	}
+
 	return false;
 }
 
@@ -1264,11 +1390,8 @@ static bool ParseFloat4(lua_State* L, int index, float4& value)
 		}
 	}
 	else if (lua_isstring(L, index)) {
-		const int count = sscanf(lua_tostring(L, index), "%f %f %f %f",
-		                         &value.x, &value.y, &value.z, &value.w);
-		if (count == 4) {
+		if (sscanf(lua_tostring(L, index), "%f %f %f %f", &value.x, &value.y, &value.z, &value.w) == 4)
 			return true;
-		}
 	}
 	return false;
 }
@@ -1286,7 +1409,7 @@ static bool ParseBoolean(lua_State* L, int index, bool& value)
 		return true;
 	}
 	else if (lua_isstring(L, index)) {
-		const string str = StringToLower(lua_tostring(L, index));
+		const std::string str = StringToLower(lua_tostring(L, index));
 		if ((str == "1") || (str == "true")) {
 			value = true;
 			return true;
@@ -1306,11 +1429,11 @@ static bool ParseBoolean(lua_State* L, int index, bool& value)
 //  String key functions
 //
 
-int LuaTable::Get(const string& key, int def) const
+int LuaTable::Get(const std::string& key, int def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	const int value = lua_toint(L, -1);
 	if (unlikely(value == 0) && !lua_isnumber(L, -1) && !lua_isstring(L, -1)) {
 		lua_pop(L, 1);
@@ -1321,11 +1444,11 @@ int LuaTable::Get(const string& key, int def) const
 }
 
 
-bool LuaTable::Get(const string& key, bool def) const
+bool LuaTable::Get(const std::string& key, bool def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	bool value;
 	if (!ParseBoolean(L, -1, value)) {
 		lua_pop(L, 1);
@@ -1336,11 +1459,11 @@ bool LuaTable::Get(const string& key, bool def) const
 }
 
 
-float LuaTable::Get(const string& key, float def) const
+float LuaTable::Get(const std::string& key, float def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	const float value = lua_tonumber(L, -1);
 	if (unlikely(value == 0.f) && !lua_isnumber(L, -1) && !lua_isstring(L, -1)) {
 		lua_pop(L, 1);
@@ -1352,11 +1475,11 @@ float LuaTable::Get(const string& key, float def) const
 
 
 
-float3 LuaTable::Get(const string& key, const float3& def) const
+float3 LuaTable::Get(const std::string& key, const float3& def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	float3 value;
 	if (!ParseFloat3(L, -1, value)) {
 		lua_pop(L, 1);
@@ -1366,11 +1489,11 @@ float3 LuaTable::Get(const string& key, const float3& def) const
 	return value;
 }
 
-float4 LuaTable::Get(const string& key, const float4& def) const
+float4 LuaTable::Get(const std::string& key, const float4& def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	float4 value;
 	if (!ParseFloat4(L, -1, value)) {
 		lua_pop(L, 1);
@@ -1382,16 +1505,16 @@ float4 LuaTable::Get(const string& key, const float4& def) const
 
 
 
-string LuaTable::Get(const string& key, const string& def) const
+std::string LuaTable::Get(const std::string& key, const std::string& def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	if (!lua_isstring(L, -1)) {
 		lua_pop(L, 1);
 		return def;
 	}
-	const string value = lua_tostring(L, -1);
+	const std::string value = lua_tostring(L, -1);
 	lua_pop(L, 1);
 	return value;
 }
@@ -1405,9 +1528,9 @@ string LuaTable::Get(const string& key, const string& def) const
 
 int LuaTable::Get(int key, int def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	const int value = lua_toint(L, -1);
 	if (unlikely(value == 0) && !lua_isnumber(L, -1) && !lua_isstring(L, -1)) {
 		lua_pop(L, 1);
@@ -1420,9 +1543,9 @@ int LuaTable::Get(int key, int def) const
 
 bool LuaTable::Get(int key, bool def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	bool value;
 	if (!ParseBoolean(L, -1, value)) {
 		lua_pop(L, 1);
@@ -1435,9 +1558,9 @@ bool LuaTable::Get(int key, bool def) const
 
 float LuaTable::Get(int key, float def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	const float value = lua_tonumber(L, -1);
 	if (unlikely(value == 0) && !lua_isnumber(L, -1) && !lua_isstring(L, -1)) {
 		lua_pop(L, 1);
@@ -1451,9 +1574,9 @@ float LuaTable::Get(int key, float def) const
 
 float3 LuaTable::Get(int key, const float3& def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	float3 value;
 	if (!ParseFloat3(L, -1, value)) {
 		lua_pop(L, 1);
@@ -1479,16 +1602,16 @@ float4 LuaTable::Get(int key, const float4& def) const
 
 
 
-string LuaTable::Get(int key, const string& def) const
+std::string LuaTable::Get(int key, const std::string& def) const
 {
-	if (!PushValue(key)) {
+	if (!PushValue(key))
 		return def;
-	}
+
 	if (!lua_isstring(L, -1)) {
 		lua_pop(L, 1);
 		return def;
 	}
-	const string value = lua_tostring(L, -1);
+	const std::string value = lua_tostring(L, -1);
 	lua_pop(L, 1);
 	return value;
 }
@@ -1512,13 +1635,13 @@ float4 LuaTable::GetFloat4(int key, const float4& def) const
 }
 
 
-float3 LuaTable::GetFloat3(const string& key, const float3& def) const
+float3 LuaTable::GetFloat3(const std::string& key, const float3& def) const
 {
 	return Get(key, def);
 }
 
 
-float4 LuaTable::GetFloat4(const string& key, const float4& def) const
+float4 LuaTable::GetFloat4(const std::string& key, const float4& def) const
 {
 	return Get(key, def);
 }
