@@ -7,9 +7,15 @@ class CBaseGroundTextures {
 public:
 	virtual ~CBaseGroundTextures() {}
 	virtual void DrawUpdate() {}
+	virtual void DrawUpdateSquare(int x, int y) {}
+
 	virtual bool SetSquareLuaTexture(int texSquareX, int texSquareY, int texID) { return false; }
 	virtual bool GetSquareLuaTexture(int texSquareX, int texSquareY, int texID, int texSizeX, int texSizeY, int texMipLevel) { return false; }
-	virtual void BindSquareTexture(int x, int y) {}
+
+	virtual void BindSquareTextureArray() const {}
+	virtual void UnBindSquareTextureArray() const {}
+
+	virtual unsigned int GetSquareMipLevel(unsigned int i) const { return 0; }
 };
 
 #endif
