@@ -15,10 +15,9 @@
 #include "System/TimeProfiler.h"
 
 
-CBasicMapDamage::CBasicMapDamage()
+void CBasicMapDamage::Init()
 {
 	mapHardness = mapInfo->map.hardness;
-	disabled = false;
 
 	for (int a = 0; a <= CRATER_TABLE_SIZE; ++a) {
 		const float r  = a / float(CRATER_TABLE_SIZE);
@@ -30,10 +29,10 @@ CBasicMapDamage::CBasicMapDamage()
 	}
 
 	for (int a = 0; a < CMapInfo::NUM_TERRAIN_TYPES; ++a) {
-		rawHardness[a] = mapInfo->terrainTypes[a].hardness * mapHardness;
-		invHardness[a] = 1.0f / rawHardness[a];
+		TerrainTypeHardnessChanged(a);
 	}
 
+	// 3x3 smoothing kernel
 	weightTable[0] = 1.0f / 16.0f;
 	weightTable[1] = 2.0f / 16.0f;
 	weightTable[2] = 1.0f / 16.0f;
@@ -43,8 +42,43 @@ CBasicMapDamage::CBasicMapDamage()
 	weightTable[6] = 1.0f / 16.0f;
 	weightTable[7] = 2.0f / 16.0f;
 	weightTable[8] = 1.0f / 16.0f;
+
+
+	explSquaresPoolIdx = 0;
+	explUpdateQueueIdx = 0;
+
+	explosionSquaresPool.clear();
+	explosionSquaresPool.resize(4 * 1024 * 1024);
+	explosionUpdateQueue.clear();
+	explosionUpdateQueue.reserve(64);
+
+	std::fill(explosionSquaresPool.begin(), explosionSquaresPool.end(), 0.0f);
 }
 
+
+void CBasicMapDamage::TerrainTypeHardnessChanged(int ttIndex)
+{
+	// table should contain only positive or only negative values, never both
+	rawHardness[ttIndex] = mapHardness * std::max(0.001f, mapInfo->terrainTypes[ttIndex].hardness);
+	invHardness[ttIndex] = 1.0f / rawHardness[ttIndex];
+}
+
+void CBasicMapDamage::TerrainTypeSpeedModChanged(int ttIndex)
+{
+	const unsigned char* typeMap = readMap->GetTypeMapSynced();
+
+	// update all map-squares that reference this terrain-type (slow)
+	for (int tz = 0; tz < mapDims.hmapy; tz++) {
+		for (int tx = 0; tx < mapDims.hmapx; tx++) {
+			if (typeMap[tz * mapDims.hmapx + tx] != ttIndex)
+				continue;
+
+			pathManager->TerrainChange((tx << 1), (tz << 1),  (tx << 1) + 1, (tz << 1) + 1,  TERRAINCHANGE_TYPEMAP_SPEED_VALUES);
+		}
+	}
+}
+
+
 void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 {
 	if (!pos.IsInMap())
@@ -53,9 +87,9 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 	if (strength < 10.0f || radius < 8.0f)
 		return;
 
-	explosions.emplace_back();
+	explosionUpdateQueue.emplace_back();
 
-	Explo& e = explosions.back();
+	Explo& e = explosionUpdateQueue.back();
 	e.pos = pos;
 	e.strength = strength;
 	e.radius = radius;
@@ -64,7 +98,7 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 	e.x2 = Clamp<int>((pos.x + radius) / SQUARE_SIZE, 1, mapDims.mapxm1);
 	e.y1 = Clamp<int>((pos.z - radius) / SQUARE_SIZE, 1, mapDims.mapym1);
 	e.y2 = Clamp<int>((pos.z + radius) / SQUARE_SIZE, 1, mapDims.mapym1);
-	e.squares.reserve((e.y2 - e.y1 + 1) * (e.x2 - e.x1 + 1));
+	e.idx = explSquaresPoolIdx;
 
 	const float* curHeightMap = readMap->GetCornerHeightMapSynced();
 	const float* orgHeightMap = readMap->GetOriginalHeightMapSynced();
@@ -76,11 +110,11 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 	// figure out how much height to add to each square
 	for (int y = e.y1; y <= e.y2; ++y) {
 		for (int x = e.x1; x <= e.x2; ++x) {
-			const CSolidObject* so = groundBlockingObjectMap->GroundBlockedUnsafe(y * mapDims.mapx + x);
+			const CSolidObject* so = groundBlockingObjectMap.GroundBlockedUnsafe(y * mapDims.mapx + x);
 
 			// do not change squares with buildings on them here
 			if (so != nullptr && so->blockHeightChanges) {
-				e.squares.push_back(0.0f);
+				SetExplosionSquare(0.0f);
 				continue;
 			}
 
@@ -103,6 +137,7 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 					const int tmz = Clamp((y >> 1) + j, 0, mapDims.hmapy - 1);
 					const int tmx = Clamp((x >> 1) + i, 0, mapDims.hmapx - 1);
 					const int tti = typeMap[tmz * mapDims.hmapx + tmx];
+
 					sumRawHardness += (rawHardness[tti] * weightTable[(j + 1) * 3 + (i + 1)]);
 				}
 			}
@@ -124,12 +159,12 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 			if (explDif < -0.3f && strength > 200.0f)
 				grassDrawer->RemoveGrass(float3(x * SQUARE_SIZE, 0.0f, y * SQUARE_SIZE));
 
-			e.squares.push_back(explDif);
+			SetExplosionSquare(explDif);
 		}
 	}
 
 	QuadFieldQuery qfQuery;
-	quadField->GetUnitsExact(qfQuery, pos, radius);
+	quadField.GetUnitsExact(qfQuery, pos, radius);
 
 	// calculate how much to raise or lower buildings in the explosion radius
 	// (while still keeping the ground below them flat)
@@ -180,7 +215,7 @@ void CBasicMapDamage::Explosion(const float3& pos, float strength, float radius)
 void CBasicMapDamage::RecalcArea(int x1, int x2, int y1, int y2)
 {
 	readMap->UpdateHeightMapSynced(SRectangle(x1, y1, x2, y2));
-	featureHandler->TerrainChanged(x1, y1, x2, y2);
+	featureHandler.TerrainChanged(x1, y1, x2, y2);
 	{
 		SCOPED_TIMER("Sim::BasicMapDamage::Los");
 		losHandler->UpdateHeightMapSynced(SRectangle(x1, y1, x2, y2));
@@ -196,22 +231,24 @@ void CBasicMapDamage::Update()
 {
 	SCOPED_TIMER("Sim::BasicMapDamage");
 
-	for (Explo& e: explosions) {
-		if (e.ttl <= 0)
+	for (unsigned int i = explUpdateQueueIdx, n = explosionUpdateQueue.size(); i < n; i++) {
+		Explo& e = explosionUpdateQueue[i];
+
+		if ((e.ttl--) <= 0)
 			continue;
 
-		--e.ttl;
 
-		auto si = e.squares.cbegin();
+		unsigned int expSquarePoolIdx = e.idx;
 
 		for (int y = e.y1; y <= e.y2; ++y) {
 			for (int x = e.x1; x <= e.x2; ++x) {
-				readMap->AddHeight(y * mapDims.mapxp1 + x, *(si++));
+				readMap->AddHeight(y * mapDims.mapxp1 + x, explosionSquaresPool[ (expSquarePoolIdx++) % explosionSquaresPool.size() ]);
 			}
 		}
 
+
 		for (const ExploBuilding& b: e.buildings) {
-			CUnit* unit = unitHandler->GetUnit(b.id);
+			CUnit* unit = unitHandler.GetUnit(b.id);
 
 			if (unit == nullptr)
 				continue;
@@ -226,18 +263,23 @@ void CBasicMapDamage::Update()
 			unit->Move(UpVector * b.dif, true);
 		}
 
-		if (e.ttl == 0) {
-			RecalcArea(e.x1 - 1, e.x2 + 1, e.y1 - 1, e.y2 + 1);
-		}
+		if (e.ttl != 0)
+			continue;
+
+		RecalcArea(e.x1 - 1, e.x2 + 1, e.y1 - 1, e.y2 + 1);
 	}
 
-	while (!explosions.empty()) {
-		const Explo& explosion = explosions.front();
 
-		if (explosion.ttl > 0)
-			break;
+	// pop explosions that are no longer being processed
+	while (explUpdateQueueIdx < explosionUpdateQueue.size()) {
+		if (explosionUpdateQueue[explUpdateQueueIdx].ttl > 0)
+			return;
 
-		explosions.pop_front();
+		explUpdateQueueIdx += 1;
 	}
+
+	// no more explosions left to process, clear queue for real
+	explosionUpdateQueue.clear();
+	explUpdateQueueIdx = 0;
 }
 
