@@ -8,18 +8,23 @@
 class IMapDamage
 {
 public:
-	static IMapDamage* GetMapDamage();
+	static IMapDamage* InitMapDamage();
+	static void FreeMapDamage(IMapDamage*);
 
 public:
-	IMapDamage();
 	virtual ~IMapDamage() {}
 
 	virtual void Explosion(const float3& pos, float strength, float radius) = 0;
 	virtual void RecalcArea(int x1, int x2, int y1, int y2) = 0;
-	virtual void Update() {}
+	virtual void TerrainTypeHardnessChanged(int ttIndex) {}
+	virtual void TerrainTypeSpeedModChanged(int ttIndex) {}
 
-	bool disabled;
-	float mapHardness;
+	virtual void Init() = 0;
+	virtual void Update() = 0;
+
+	virtual bool Disabled() const = 0;
+
+	float mapHardness = 0.0f;
 };
 
 extern IMapDamage* mapDamage;
