@@ -6,12 +6,11 @@
 #include "Sim/Misc/GlobalConstants.h"
 #include "Rendering/GlobalRendering.h"
 
-#include "MapParser.h"
 #include "Lua/LuaParser.h"
 #include "System/Log/ILog.h"
 #include "System/Exceptions.h"
-#include "System/myMath.h"
-#include "System/Util.h"
+#include "System/SpringMath.h"
+#include "System/StringUtil.h"
 #include "System/FileSystem/FileHandler.h"
 
 #if !defined(HEADLESS) && !defined(NO_SOUND)
@@ -19,15 +18,15 @@
 #include "System/Sound/OpenAL/EFXPresets.h"
 #endif
 
+#include <array>
 #include <cassert>
-#include <cfloat>
 
 
 // Before delete, the const is const_cast'ed away. There are
 // no (other) situations where mapInfo may be modified, except
 //   LuaUnsyncedCtrl may change water
 //   LuaSyncedCtrl may change terrainTypes
-const CMapInfo* mapInfo = NULL;
+const CMapInfo* mapInfo = nullptr;
 
 
 static void FIND_MAP_TEXTURE(std::string* filePath, const std::string& defaultDir = "maps/")
@@ -42,24 +41,21 @@ static void FIND_MAP_TEXTURE(std::string* filePath, const std::string& defaultDi
 
 
 
-CMapInfo::CMapInfo(const std::string& mapInfoFile, const string& mapName)
+CMapInfo::CMapInfo(const std::string& mapFileName, const string& mapHumanName): mapInfoParser(mapFileName)
 {
-	map.name = mapName;
+	map.name = mapHumanName;
 
-	parser = new MapParser(mapInfoFile);
-	if (!parser->IsValid()) {
-		throw content_error("MapInfo: " + parser->GetErrorLog());
-	}
+	if (!mapInfoParser.IsValid())
+		throw content_error("[MapInfo] info-parser for map \"" + map.name + "\" (\"" + mapFileName + "\") invalid: \"" + mapInfoParser.GetErrorLog() + "\"");
 
 	LuaParser resParser("gamedata/resources.lua", SPRING_VFS_MOD_BASE, SPRING_VFS_ZIP);
 	LuaTable resTable;
 
-	if (!resParser.Execute()) {
-		LOG_L(L_ERROR, "%s", resParser.GetErrorLog().c_str());
-	}
+	if (!resParser.Execute())
+		LOG_L(L_ERROR, "[%s] error executing resource-parser: \"%s\"", __func__, resParser.GetErrorLog().c_str());
 
 	resTable = resParser.GetRoot();
-	resRoot = &resTable;
+	resTableRoot = &resTable;
 
 	ReadGlobal();
 	ReadAtmosphere();
@@ -69,34 +65,25 @@ CMapInfo::CMapInfo(const std::string& mapInfoFile, const string& mapName)
 	ReadLight();
 	ReadWater();
 	ReadSMF();
-	ReadSM3();
 	ReadTerrainTypes();
 	ReadPFSConstants();
 	ReadSound();
 
-	//FIXME save all data in an array, so we can destroy the lua context (to save mem)?
-	//delete parser;
+	//FIXME save all mapParser data in an array, so we can destroy the lua context (to save mem)?
 }
 
 CMapInfo::~CMapInfo()
 {
 #if !defined(HEADLESS) && !defined(NO_SOUND)
 	delete efxprops;
-	efxprops = NULL;
+	efxprops = nullptr;
 #endif
-	delete parser;
-	parser = NULL;
 }
 
-std::string CMapInfo::GetStringValue(const std::string& key) const
-{
-	assert(parser->GetRoot().IsValid());
-	return parser->GetRoot().GetString(key, "");
-}
 
 void CMapInfo::ReadGlobal()
 {
-	const LuaTable& topTable = parser->GetRoot();
+	const LuaTable& topTable = mapInfoParser.GetRoot();
 
 	map.description  = topTable.GetString("description", map.name);
 	map.author       = topTable.GetString("author", "");
@@ -115,9 +102,8 @@ void CMapInfo::ReadGlobal()
 	map.voidWater       = topTable.GetBool("voidWater", false);
 	map.voidGround      = topTable.GetBool("voidGround", false);
 
-	// clamps
-	if (std::abs(map.hardness) < 0.001f)
-		map.hardness = std::copysignf(0.001f, map.hardness);
+	// clamps; global hardness can be either positive or negative but never zero
+	map.hardness        = std::max(0.001f, std::abs(map.hardness)) * std::copysign(1.0f, map.hardness);
 	map.tidalStrength   = std::max(0.000f, map.tidalStrength);
 	map.maxMetal        = std::max(0.000f, map.maxMetal);
 	map.extractorRadius = std::max(0.000f, map.extractorRadius);
@@ -127,14 +113,14 @@ void CMapInfo::ReadGlobal()
 void CMapInfo::ReadGui()
 {
 	// GUI
-	gui.autoShowMetal = parser->GetRoot().GetBool("autoShowMetal", true);
+	gui.autoShowMetal = mapInfoParser.GetRoot().GetBool("autoShowMetal", true);
 }
 
 
 void CMapInfo::ReadAtmosphere()
 {
 	// MAP\ATMOSPHERE
-	const LuaTable& atmoTable = parser->GetRoot().SubTable("atmosphere");
+	const LuaTable& atmoTable = mapInfoParser.GetRoot().SubTable("atmosphere");
 	atmosphere_t& atmo = atmosphere;
 
 	atmo.minWind = atmoTable.GetFloat("minWind", 5.0f);
@@ -163,7 +149,7 @@ void CMapInfo::ReadAtmosphere()
 
 void CMapInfo::ReadSplats()
 {
-	const LuaTable& splatsTable = parser->GetRoot().SubTable("splats");
+	const LuaTable& splatsTable = mapInfoParser.GetRoot().SubTable("splats");
 
 	splats.texScales = splatsTable.GetFloat4("texScales", float4(0.02f, 0.02f, 0.02f, 0.02f));
 	splats.texMults  = splatsTable.GetFloat4("texMults",  float4(1.0f, 1.0f, 1.0f, 1.0f));
@@ -171,8 +157,8 @@ void CMapInfo::ReadSplats()
 
 void CMapInfo::ReadGrass()
 {
-	const LuaTable& grassTable = parser->GetRoot().SubTable("grass");
-	const LuaTable& mapResTable = parser->GetRoot().SubTable("resources");
+	const LuaTable& grassTable = mapInfoParser.GetRoot().SubTable("grass");
+	const LuaTable& mapResTable = mapInfoParser.GetRoot().SubTable("resources");
 
 	grass.bladeWaveScale   = grassTable.GetFloat("bladeWaveScale", 1.0f);
 	grass.bladeWidth       = grassTable.GetFloat("bladeWidth", 0.7f);
@@ -187,18 +173,11 @@ void CMapInfo::ReadGrass()
 
 void CMapInfo::ReadLight()
 {
-	const LuaTable& lightTable = parser->GetRoot().SubTable("lighting");
-
-	light.sunStartAngle = lightTable.GetFloat("sunStartAngle", 0.0f);
-	light.sunOrbitTime = lightTable.GetFloat("sunOrbitTime", 10 * GAME_SPEED);
-	light.sunDir = lightTable.GetFloat4("sunDir", float4(0.0f, 1.0f, 2.0f, FLT_MAX));
-
-	if (light.sunDir.w == FLT_MAX) {
-		// if four params are not specified for sundir, fallback to the old three param format
-		light.sunDir = lightTable.GetFloat3("sunDir", float3(0.0f, 1.0f, 2.0f));
-		light.sunDir.w = FLT_MAX;
-	}
+	const LuaTable& lightTable = mapInfoParser.GetRoot().SubTable("lighting");
 
+	// read the float4 direction first; keep it if the float3 value does not exist
+	light.sunDir =  lightTable.GetFloat4("sunDir", float4(0.0f, 1.0f, 2.0f, 1.0f));
+	light.sunDir = {lightTable.GetFloat3("sunDir", light.sunDir), light.sunDir.w};
 	light.sunDir.ANormalize();
 
 	light.groundAmbientColor  = lightTable.GetFloat3("groundAmbientColor", float3(0.5f, 0.5f, 0.5f));
@@ -206,21 +185,21 @@ void CMapInfo::ReadLight()
 	light.groundSpecularColor = lightTable.GetFloat3("groundSpecularColor", float3(0.1f, 0.1f, 0.1f));
 	light.groundShadowDensity = lightTable.GetFloat("groundShadowDensity", 0.8f);
 
-	light.unitAmbientColor  = lightTable.GetFloat3("unitAmbientColor", float3(0.4f, 0.4f, 0.4f));
-	light.unitDiffuseColor  = lightTable.GetFloat3("unitDiffuseColor", float3(0.7f, 0.7f, 0.7f));
-	light.unitSpecularColor = lightTable.GetFloat3("unitSpecularColor", light.unitDiffuseColor);
-	light.unitShadowDensity = lightTable.GetFloat("unitShadowDensity", 0.8f);
+	light.modelAmbientColor  = lightTable.GetFloat3("unitAmbientColor", float3(0.4f, 0.4f, 0.4f));
+	light.modelDiffuseColor  = lightTable.GetFloat3("unitDiffuseColor", float3(0.7f, 0.7f, 0.7f));
+	light.modelSpecularColor = lightTable.GetFloat3("unitSpecularColor", light.modelDiffuseColor);
+	light.modelShadowDensity = lightTable.GetFloat("unitShadowDensity", 0.8f);
 
 	light.specularExponent = lightTable.GetFloat("specularExponent", 100.0f);
 
 	light.groundShadowDensity = Clamp(light.groundShadowDensity, 0.0f, 1.0f);
-	light.unitShadowDensity   = Clamp(light.unitShadowDensity,   0.0f, 1.0f);
+	light.modelShadowDensity   = Clamp(light.modelShadowDensity,   0.0f, 1.0f);
 }
 
 
 void CMapInfo::ReadWater()
 {
-	const LuaTable& wt = parser->GetRoot().SubTable("water");
+	const LuaTable& wt = mapInfoParser.GetRoot().SubTable("water");
 
 	water.fluidDensity = wt.GetFloat("fluidDensity", 960.0f * 0.25f);
 	water.repeatX = wt.GetFloat("repeatX", 0.0f);
@@ -267,7 +246,7 @@ void CMapInfo::ReadWater()
 	water.forceRendering = wt.GetBool("forceRendering", false);
 
 	// use 'resources.lua' for missing fields  (our the engine defaults)
-	const LuaTable& resGfxMaps = resRoot->SubTable("graphics").SubTable("maps");
+	const LuaTable& resGfxMaps = resTableRoot->SubTable("graphics").SubTable("maps");
 
 	FIND_MAP_TEXTURE(&water.texture);
 	if (water.texture.empty()) {
@@ -300,15 +279,14 @@ void CMapInfo::ReadWater()
 	LuaTable caustics = wt.SubTable("caustics");
 	std::string causticPrefix = "maps/";
 	if (!caustics.IsValid()) {
-		caustics = resRoot->SubTable("graphics").SubTable("caustics");
+		caustics = resTableRoot->SubTable("graphics").SubTable("caustics");
 		causticPrefix = "bitmaps/";
 	}
 	if (caustics.IsValid()) {
 		for (int i = 1; true; i++) {
 			std::string texName = caustics.GetString(i, "");
-			if (texName.empty()) {
+			if (texName.empty())
 				break;
-			}
 
 			FIND_MAP_TEXTURE(&texName, causticPrefix);
 			water.causticTextures.push_back(texName);
@@ -338,10 +316,10 @@ static bool ParseSplatDetailNormalTexture(const LuaTable& table, const T key, st
 void CMapInfo::ReadSMF()
 {
 	// SMF specific settings
-	const LuaTable& mapResTable = parser->GetRoot().SubTable("resources");
+	const LuaTable& mapResTable = mapInfoParser.GetRoot().SubTable("resources");
 	const LuaTable& sdnTexTable = mapResTable.SubTable("splatDetailNormalTex");
 
-	const std::vector<std::pair<std::string*, std::string>> texNames = {
+	const std::array<std::pair<std::string*, std::string>, 9> texNames = {{
 		{&smf.detailTexName,         "detailTex"},
 		{&smf.specularTexName,       "specularTex"},
 		{&smf.splatDetailTexName,    "splatDetailTex"},
@@ -351,7 +329,7 @@ void CMapInfo::ReadSMF()
 		{&smf.blendNormalsTexName,   "detailNormalTex"},
 		{&smf.lightEmissionTexName,  "lightEmissionTex"},
 		{&smf.parallaxHeightTexName, "parallaxHeightTex"},
-	};
+	}};
 
 	for (auto& pair: texNames) {
 		*pair.first = mapResTable.GetString(pair.second, "");
@@ -359,7 +337,7 @@ void CMapInfo::ReadSMF()
 	}
 
 	if (smf.detailTexName.empty()) {
-		const LuaTable& resGfxMaps = resRoot->SubTable("graphics").SubTable("maps");
+		const LuaTable& resGfxMaps = resTableRoot->SubTable("graphics").SubTable("maps");
 		smf.detailTexName = resGfxMaps.GetString("detailtex", "detailtex2.bmp");
 		FIND_MAP_TEXTURE(&smf.detailTexName, "bitmaps/");
 	}
@@ -384,7 +362,7 @@ void CMapInfo::ReadSMF()
 
 
 	// overrides for compiled parameters
-	const LuaTable& smfTable = parser->GetRoot().SubTable("smf");
+	const LuaTable& smfTable = mapInfoParser.GetRoot().SubTable("smf");
 
 	smf.minHeightOverride = smfTable.KeyExists("minHeight");
 	smf.maxHeightOverride = smfTable.KeyExists("maxHeight");
@@ -412,16 +390,9 @@ void CMapInfo::ReadSMF()
 }
 
 
-void CMapInfo::ReadSM3()
-{
-	// SM3 specific settings
-	sm3.minimap = parser->GetRoot().GetString("minimap", "");
-}
-
-
 void CMapInfo::ReadTerrainTypes()
 {
-	const LuaTable& terrTypeTable = parser->GetRoot().SubTable("terrainTypes");
+	const LuaTable& terrTypeTable = mapInfoParser.GetRoot().SubTable("terrainTypes");
 
 	for (int tt = 0; tt < NUM_TERRAIN_TYPES; tt++) {
 		TerrainType& terrType = terrainTypes[tt];
@@ -448,7 +419,7 @@ void CMapInfo::ReadTerrainTypes()
 
 void CMapInfo::ReadPFSConstants()
 {
-	const LuaTable& pfsTable = (parser->GetRoot()).SubTable("pfs");
+	const LuaTable& pfsTable = (mapInfoParser.GetRoot()).SubTable("pfs");
 //	const LuaTable& legacyTable = pfsTable.SubTable("legacyConstants");
 	const LuaTable& qtpfsTable = pfsTable.SubTable("qtpfsConstants");
 
@@ -468,58 +439,59 @@ void CMapInfo::ReadPFSConstants()
 void CMapInfo::ReadSound()
 {
 #if !defined(HEADLESS) && !defined(NO_SOUND)
-	const LuaTable& soundTable = parser->GetRoot().SubTable("sound");
+	const LuaTable& soundTable = mapInfoParser.GetRoot().SubTable("sound");
 
 	efxprops = new EAXSfxProps();
 
-	const std::string presetname = soundTable.GetString("preset", "default");
-	std::map<std::string, EAXSfxProps>::const_iterator et = eaxPresets.find(presetname);
-	if (et != eaxPresets.end()) {
-		*efxprops = et->second;
-	}
+	const auto presetIt = eaxPresets.find(soundTable.GetString("preset", "default"));
 
-	std::map<std::string, ALuint>::const_iterator it;
+	if (presetIt != eaxPresets.end())
+		*efxprops = presetIt->second;
 
 	const LuaTable& filterTable = soundTable.SubTable("passfilter");
-	for (it = nameToALFilterParam.begin(); it != nameToALFilterParam.end(); ++it) {
-		const std::string& name = it->first;
+
+	for (const auto& item: nameToALFilterParam) {
+		const std::string& name = item.first;
 		const int luaType = filterTable.GetType(name);
 
 		if (luaType == LuaTable::NIL)
 			continue;
-		
-		const ALuint param = it->second;
-		const unsigned& type = alParamType[param];
+
+		const ALuint param = item.second;
+		const unsigned type = alParamType[param];
+
 		switch (type) {
 			case EFXParamTypes::FLOAT:
 				if (luaType == LuaTable::NUMBER)
-					efxprops->filter_properties_f[param] = filterTable.GetFloat(name, 0.f);
+					efxprops->filter_props_f[param] = filterTable.GetFloat(name, 0.0f);
 				break;
 		}
 	}
 
 	soundTable.SubTable("reverb");
-	for (it = nameToALParam.begin(); it != nameToALParam.end(); ++it) {
-		const std::string& name = it->first;
+
+	for (const auto& item: nameToALFilterParam) {
+		const std::string& name = item.first;
 		const int luaType = filterTable.GetType(name);
 
 		if (luaType == LuaTable::NIL)
 			continue;
 
-		const ALuint param = it->second;
-		const unsigned& type = alParamType[param];
+		const ALuint param = item.second;
+		const unsigned type = alParamType[param];
+
 		switch (type) {
 			case EFXParamTypes::VECTOR:
 				if (luaType == LuaTable::TABLE)
-					efxprops->properties_v[param] = filterTable.GetFloat3(name, ZeroVector);
+					efxprops->reverb_props_v[param] = filterTable.GetFloat3(name, ZeroVector);
 				break;
 			case EFXParamTypes::FLOAT:
 				if (luaType == LuaTable::NUMBER)
-					efxprops->properties_f[param] = filterTable.GetFloat(name, 0.f);
+					efxprops->reverb_props_f[param] = filterTable.GetFloat(name, 0.0f);
 				break;
 			case EFXParamTypes::BOOL:
 				if (luaType == LuaTable::BOOLEAN)
-					efxprops->properties_i[param] = filterTable.GetBool(name, false);
+					efxprops->reverb_props_i[param] = filterTable.GetBool(name, false);
 				break;
 		}
 	}
