@@ -15,8 +15,12 @@ struct MapBitmapInfo;
 class CSMFMapFile
 {
 public:
+	CSMFMapFile(                              ): ifs("", "") {                    } // defer Open
+	CSMFMapFile(const std::string& mapFileName): ifs("", "") { Open(mapFileName); } // unitsync
+	~CSMFMapFile() { Close(); }
 
-	CSMFMapFile(const std::string& mapFileName);
+	void Open(const std::string& mapFileName);
+	void Close();
 
 	void ReadMinimap(void* data);
 	/// @return mip size
@@ -25,8 +29,8 @@ public:
 	void ReadHeightmap(float* sHeightMap, float* uHeightMap, float base, float mod);
 	void ReadFeatureInfo();
 	void ReadFeatureInfo(MapFeatureInfo* f);
-	void GetInfoMapSize(const std::string& name, MapBitmapInfo*) const;
-	bool ReadInfoMap(const std::string& name, void* data);
+	void GetInfoMapSize(const char* name, MapBitmapInfo*) const;
+	bool ReadInfoMap(const char* name, void* data);
 
 	int GetNumFeatures()     const { return featureHeader.numFeatures; }
 	int GetNumFeatureTypes() const { return featureHeader.numFeatureType; }
@@ -40,6 +44,7 @@ public:
 	 *   with SMFGroundTextures.cpp
 	 */
 	CFileHandler* GetFileHandler() { return &ifs; }
+
 	static void ReadMapTileHeader(MapTileHeader& head, CFileHandler& file);
 	static void ReadMapTileFileHeader(TileFileHeader& head, CFileHandler& file);
 
@@ -49,12 +54,14 @@ private:
 	void ReadMapFeatureHeader(MapFeatureHeader& head, CFileHandler& file);
 	void ReadMapFeatureStruct(MapFeatureStruct& head, CFileHandler& file);
 
-	SMFHeader header;
 	CFileHandler ifs;
 
+	SMFHeader header;
 	MapFeatureHeader featureHeader;
-	std::vector<std::string> featureTypes;
-	int featureFileOffset;
+
+	char featureTypes[16384][32];
+
+	int featureFileOffset = 0;
 };
 
 #endif // _SMF_MAP_FILE_H
