@@ -6,6 +6,9 @@
 #include "System/float3.h"
 #include "System/float4.h"
 
+#include <string>
+#include <vector>
+
 struct CWaterRendering {
 public:
 	void Init();
