@@ -16,10 +16,13 @@ struct GlobalRenderingInfo {
 	const char* glslVersion;
 	const char* glewVersion;
 
+	const char* sdlDriverName;
+
 	char glVersionShort[256];
 	char glslVersionShort[256];
 
 	int2 glContextVersion;
+	int2 gpuMemorySize;
 
 	SDL_version sdlVersionCompiled;
 	SDL_version sdlVersionLinked;
