@@ -3,7 +3,6 @@
 #include "IModelParser.h"
 #include "3DOParser.h"
 #include "S3OParser.h"
-#include "OBJParser.h"
 #include "AssParser.h"
 #include "Game/GlobalUnsynced.h"
 #include "Rendering/Textures/S3OTextureHandler.h"
@@ -20,56 +19,75 @@
 #include "lib/assimp/include/assimp/Importer.hpp"
 
 
-static void RegisterAssimpModelFormats(CModelLoader::FormatMap& formats) {
-	spring::unordered_set<std::string> whitelist;
+CModelLoader modelLoader;
+
+static C3DOParser g3DOParser;
+static CS3OParser gS3OParser;
+static CAssParser gAssParser;
+
+
+static bool CheckAssimpWhitelist(const char* aiExt) {
+	constexpr std::array<const char*, 5> whitelist = {
+		"3ds"  , // 3DSMax
+		"dae"  , // Collada
+		"lwo"  , // LightWave
+		"obj"  ,
+		"blend", // Blender
+	};
+
+	const auto pred = [&aiExt](const char* wlExt) { return (strcmp(aiExt, wlExt) == 0); };
+	const auto iter = std::find_if(whitelist.begin(), whitelist.end(), pred);
+
+	return (iter != whitelist.end());
+}
+
+static void RegisterModelFormats(CModelLoader::FormatMap& formats) {
+	// file-extension should be lowercase
+	formats.insert("3do", MODELTYPE_3DO);
+	formats.insert("s3o", MODELTYPE_S3O);
 
 	std::string extension;
 	std::string extensions;
 	std::string enabledExtensions;
 
-	whitelist.insert("3ds"  ); // 3DSMax
-	whitelist.insert("dae"  ); // Collada
-	whitelist.insert("lwo"  ); // LightWave
-	whitelist.insert("blend"); // Blender
-
 	Assimp::Importer importer;
 	// get a ";" separated list of format extensions ("*.3ds;*.lwo;*.mesh.xml;...")
 	importer.GetExtensionList(extensions);
-
 	// do not ignore the last extension
-	extensions += ";";
+	extensions.append(";");
 
 	size_t curIdx = 0;
 	size_t nxtIdx = 0;
 
 	// split the list, strip off the "*." extension prefixes
-	while ((nxtIdx = extensions.find(";", curIdx)) != std::string::npos) {
+	while ((nxtIdx = extensions.find(';', curIdx)) != std::string::npos) {
 		extension = extensions.substr(curIdx, nxtIdx - curIdx);
 		extension = extension.substr(extension.find("*.") + 2);
 		extension = StringToLower(extension);
 
 		curIdx = nxtIdx + 1;
 
-		if (whitelist.find(extension) == whitelist.end())
+		if (!CheckAssimpWhitelist(extension.c_str()))
 			continue;
 		if (formats.find(extension) != formats.end())
 			continue;
 
-		formats[extension] = MODELTYPE_ASS;
-		enabledExtensions += "*." + extension + ";";
+		formats.insert(extension, MODELTYPE_ASS);
+		enabledExtensions.append("*." + extension + ";");
 	}
 
-	LOG("[%s] supported Assimp model formats: %s", __FUNCTION__, enabledExtensions.c_str());
+	LOG("[%s] supported (Assimp) model formats: %s", __func__, enabledExtensions.c_str());
 }
 
-static S3DModel CreateDummyModel()
+static S3DModel CreateDummyModel(unsigned int id)
 {
 	// create a crash-dummy
 	S3DModel model;
+	model.id = id;
 	model.type = MODELTYPE_3DO;
 	model.numPieces = 1;
 	// give it one empty piece
-	model.AddPiece(new S3DOPiece());
+	model.AddPiece(g3DOParser.AllocPiece());
 	model.GetRootPiece()->SetCollisionVolume(CollisionVolume('b', 'z', -UpVector, ZeroVector));
 	return model;
 }
@@ -92,7 +110,7 @@ static void CheckPieceNormals(const S3DModel* model, const S3DModelPiece* modelP
 			const char* modelName = model->name.c_str();
 			const char* pieceName = modelPiece->name.c_str();
 
-			LOG_L(L_DEBUG, formatStr, __FUNCTION__, pieceName, modelName, numNullNormals, modelPiece->GetVertexCount());
+			LOG_L(L_DEBUG, formatStr, __func__, pieceName, modelName, numNullNormals, modelPiece->GetVertexCount());
 		}
 	}
 
@@ -102,27 +120,33 @@ static void CheckPieceNormals(const S3DModel* model, const S3DModelPiece* modelP
 }
 
 
+
 void CModelLoader::Init()
 {
-	// file-extension should be lowercase
-	formats["3do"] = MODELTYPE_3DO;
-	formats["s3o"] = MODELTYPE_S3O;
-	formats["obj"] = MODELTYPE_OBJ;
+	RegisterModelFormats(formats);
+	InitParsers();
 
-	parsers[MODELTYPE_3DO] = new C3DOParser();
-	parsers[MODELTYPE_S3O] = new CS3OParser();
-	parsers[MODELTYPE_OBJ] = new COBJParser();
-	parsers[MODELTYPE_ASS] = new CAssParser();
+	models.clear();
+	models.resize(MAX_MODEL_OBJECTS);
 
-	// FIXME: unify the metadata formats of CAssParser and COBJParser
-	RegisterAssimpModelFormats(formats);
+	// dummy first model, legitimate model IDs start at 1
+	models[0] = std::move(CreateDummyModel(numModels = 0));
+}
 
-	// dummy first model, model IDs start at 1
-	models.emplace_back();
+void CModelLoader::InitParsers()
+{
+	parsers[MODELTYPE_3DO] = &g3DOParser;
+	parsers[MODELTYPE_S3O] = &gS3OParser;
+	parsers[MODELTYPE_ASS] = &gAssParser;
+
+	for (IModelParser* p: parsers) {
+		p->Init();
+	}
 }
 
 void CModelLoader::Kill()
 {
+	LogErrors();
 	KillModels();
 	KillParsers();
 
@@ -132,26 +156,20 @@ void CModelLoader::Kill()
 
 void CModelLoader::KillModels()
 {
-	for (unsigned int n = 1; n < models.size(); n++) {
-		models[n].DeletePieces();
+	for (unsigned int i = 0; i < numModels; i++) {
+		models[i].DeletePieces();
 	}
-
-	models.clear();
 }
 
 void CModelLoader::KillParsers()
 {
-	for (auto& p: parsers) {
-		spring::SafeDelete(p.second);
+	for (IModelParser* p: parsers) {
+		p->Kill();
 	}
 
-	parsers.clear();
-}
-
-CModelLoader& CModelLoader::GetInstance()
-{
-	static CModelLoader instance;
-	return instance;
+	parsers[MODELTYPE_3DO] = nullptr;
+	parsers[MODELTYPE_S3O] = nullptr;
+	parsers[MODELTYPE_ASS] = nullptr;
 }
 
 
@@ -160,44 +178,74 @@ std::string CModelLoader::FindModelPath(std::string name) const
 {
 	// check for empty string because we can be called
 	// from Lua*Defs and certain features have no models
-	if (!name.empty()) {
-		const std::string vfsPath = "objects3d/";
-		const std::string& fileExt = FileSystem::GetExtension(name);
+	if (name.empty())
+		return name;
 
-		if (fileExt.empty()) {
-			for (auto it = formats.cbegin(); it != formats.cend(); ++it) {
-				const std::string& formatExt = it->first;
+	const std::string vfsPath = "objects3d/";
+	const std::string& fileExt = FileSystem::GetExtension(name);
 
-				if (CFileHandler::FileExists(name + "." + formatExt, SPRING_VFS_ZIP)) {
-					name += ("." + formatExt); break;
-				}
-			}
-		}
+	if (fileExt.empty()) {
+		for (const auto& format: formats) {
+			const std::string& formatExt = format.first;
 
-		if (!CFileHandler::FileExists(name, SPRING_VFS_ZIP)) {
-			if (name.find(vfsPath) == std::string::npos) {
-				return (FindModelPath(vfsPath + name));
+			if (CFileHandler::FileExists(name + "." + formatExt, SPRING_VFS_ZIP)) {
+				name.append("." + formatExt);
+				break;
 			}
 		}
 	}
 
-	return name;
+	if (CFileHandler::FileExists(name, SPRING_VFS_ZIP))
+		return name;
+
+	if (name.find(vfsPath) != std::string::npos)
+		return name;
+
+	return (FindModelPath(vfsPath + name));
 }
 
 
 void CModelLoader::PreloadModel(const std::string& modelName)
 {
+	assert(Threading::IsMainThread());
+
+	if (modelName.empty())
+		return;
 	if (!ThreadPool::HasThreads())
 		return;
 
-	if (cache.find(StringToLower(modelName)) != cache.end())
-		return;
-
+	// if already in cache, thread just returns early
+	// not spawning the thread at all would be better but still
+	// requires locking around cache.find(...) since some other
+	// preload worker might be down in CreateModel modifying it
+	// at the same time
 	ThreadPool::Enqueue([modelName]() {
 		modelLoader.LoadModel(modelName, true);
 	});
 }
 
+void CModelLoader::LogErrors()
+{
+	assert(Threading::IsMainThread());
+
+	if (errors.empty())
+		return;
+
+	// block any preload threads from modifying <errors>
+	// doing the empty-check outside lock should be fine
+	std::lock_guard<spring::mutex> lock(mutex);
+
+	for (const auto& pair: errors) {
+		char buf[1024];
+
+		SNPRINTF(buf, sizeof(buf), "could not load model \"%s\" (reason: %s)", pair.first.c_str(), pair.second.c_str());
+		LOG_L(L_ERROR, "%s", buf);
+		CLIENT_NETLOG(gu->myPlayerNum, LOG_LEVEL_INFO, buf);
+	}
+
+	errors.clear();
+}
+
 
 S3DModel* CModelLoader::LoadModel(std::string name, bool preload)
 {
@@ -205,22 +253,24 @@ S3DModel* CModelLoader::LoadModel(std::string name, bool preload)
 	if (name.empty())
 		return nullptr;
 
-	std::lock_guard<spring::mutex> lock(mutex);
-
-	StringToLowerInPlace(name);
-
 	std::string  path;
 	std::string* refs[2] = {&name, &path};
 
-	// search in cache first
-	for (unsigned int n = 0; n < 2; n++) {
-		S3DModel* cachedModel = LoadCachedModel(*refs[n], preload);
+	StringToLowerInPlace(name);
 
-		if (cachedModel != nullptr)
-			return cachedModel;
+	{
+		std::lock_guard<spring::mutex> lock(mutex);
 
-		// expensive, delay until needed
-		path = FindModelPath(name);
+		// search in cache first
+		for (const auto& ref: refs) {
+			S3DModel* cachedModel = LoadCachedModel(*ref, preload);
+
+			if (cachedModel != nullptr)
+				return cachedModel;
+
+			// expensive, delay until needed
+			path = FindModelPath(name);
+		}
 	}
 
 	// not found in cache, create the model and cache it
@@ -229,6 +279,7 @@ S3DModel* CModelLoader::LoadModel(std::string name, bool preload)
 
 S3DModel* CModelLoader::LoadCachedModel(const std::string& name, bool preload)
 {
+	// caller has lock
 	const auto ci = cache.find(name);
 
 	if (ci == cache.end())
@@ -237,7 +288,7 @@ S3DModel* CModelLoader::LoadCachedModel(const std::string& name, bool preload)
 	S3DModel* cachedModel = &models[ci->second];
 
 	if (!preload)
-		CreateLists(cachedModel);
+		UploadRenderData(cachedModel);
 
 	return cachedModel;
 }
@@ -250,25 +301,37 @@ S3DModel* CModelLoader::CreateModel(
 	bool preload
 ) {
 	S3DModel model = std::move(ParseModel(name, path));
+	S3DModel* pmodel = &models[0];
 
-	if (model.numPieces == 0)
-		model = std::move(CreateDummyModel());
+	{
+		assert(model.numPieces != 0);
+		assert(model.GetRootPiece() != nullptr);
 
-	assert(model.GetRootPiece() != nullptr);
-	model.SetPieceMatrices();
+		model.SetPieceMatrices();
 
-	if (!preload)
-		CreateLists(&model);
+		if (!preload)
+			UploadRenderData(&model);
+	}
+	{
+		std::lock_guard<spring::mutex> lock(mutex);
 
-	// add (parsed or dummy) model to cache
-	model.id = models.size();
+		// discard loaded model and return dummy if at limit
+		if (numModels >= MAX_MODEL_OBJECTS) {
+			errors.emplace_back(name, "numModels >= MAX_MODEL_OBJECTS");
+			return pmodel;
+		}
 
-	cache[name] = model.id;
-	cache[path] = model.id;
+		// NB: id depends on thread order, can not be used in synced code
+		pmodel = &models[model.id = ++numModels];
 
-	models.emplace_back();
-	models.back() = std::move(model);
-	return &models[model.id];
+		// add (parsed or dummy) model to cache
+		cache[name] = model.id;
+		cache[path] = model.id;
+
+		*pmodel = std::move(model);
+	}
+
+	return pmodel;
 }
 
 
@@ -288,20 +351,20 @@ S3DModel CModelLoader::ParseModel(const std::string& name, const std::string& pa
 	S3DModel model;
 	IModelParser* parser = GetFormatParser(FileSystem::GetExtension(path));
 
-	if (parser != nullptr) {
-		try {
-			model = std::move(parser->Load(path));
-		} catch (const content_error& ex) {
-			char buf[1024];
-
-			SNPRINTF(buf, sizeof(buf), "could not load model \"%s\" (reason: %s)", name.c_str(), ex.what());
-			LOG_L(L_ERROR, "%s", buf);
-			CLIENT_NETLOG(gu->myPlayerNum, LOG_LEVEL_INFO, buf);
-
-			model.numPieces = 0;
-		}
-	} else {
+	if (parser == nullptr) {
 		LOG_L(L_ERROR, "could not find a parser for model \"%s\" (unknown format?)", name.c_str());
+		return (CreateDummyModel(0));
+	}
+
+	try {
+		model = std::move(parser->Load(path));
+	} catch (const content_error& ex) {
+		{
+			std::lock_guard<spring::mutex> lock(mutex);
+			errors.emplace_back(name, ex.what());
+		}
+
+		model = std::move(CreateDummyModel(0));
 	}
 
 	return model;
@@ -309,28 +372,20 @@ S3DModel CModelLoader::ParseModel(const std::string& name, const std::string& pa
 
 
 
-void CModelLoader::CreateLists(S3DModel* model) {
-	const S3DModelPiece* rootPiece = model->GetRootPiece();
-
-	if (rootPiece->GetDisplayListID() != 0)
+void CModelLoader::UploadRenderData(S3DModel* model) {
+	if (model->UploadedBuffers())
 		return;
 
-	for (S3DModelPiece* p: model->pieces) {
-		p->UploadGeometryVBOs();
-		p->CreateShatterPieces();
-		p->CreateDispList();
-	}
+	model->UploadBuffers();
 
 	if (model->type == MODELTYPE_3DO)
 		return;
 
 	// make sure textures (already preloaded) are fully loaded
-	texturehandlerS3O->LoadTexture(model);
+	textureHandlerS3O.LoadTexture(model);
 
-	// warn about models with bad normals (they break lighting)
+	// warn about models with bad normals, they break lighting
 	// skip for 3DO's since those have auto-calculated normals
-	CheckPieceNormals(model, rootPiece);
+	CheckPieceNormals(model, model->GetRootPiece());
 }
 
-/******************************************************************************/
-/******************************************************************************/
