@@ -1,15 +1,15 @@
 #include "BuildingMaskMap.h"
-#include "GlobalConstants.h"
+#include "Map/ReadMap.h"
 
-BuildingMaskMap* buildingMaskMap = nullptr;
+BuildingMaskMap buildingMaskMap;
 
 CR_BIND(BuildingMaskMap, ())
 CR_REG_METADATA(BuildingMaskMap, (
 	CR_MEMBER(maskMap)
-	))
+))
 
 
-bool BuildingMaskMap::CheckBounds(unsigned int x, unsigned int z)
+bool BuildingMaskMap::CheckBounds(unsigned int x, unsigned int z) const
 {
 	return ((x < mapDims.hmapx) && (z < mapDims.hmapy));
 }
@@ -26,8 +26,8 @@ bool BuildingMaskMap::SetTileMask(unsigned int x, unsigned int z, std::uint16_t
 
 // tests previously set mask for tile[x,z] in 2*SQUARE_SIZE coordinates against supplied value
 // true - construction is allowed, false - it's not
-bool BuildingMaskMap::TestTileMaskUnsafe(unsigned int x, unsigned int z, std::uint16_t value)
+bool BuildingMaskMap::TestTileMaskUnsafe(unsigned int x, unsigned int z, std::uint16_t value) const
 {
 	assert(CheckBounds(x, z));
 	return (maskMap[x + z * mapDims.hmapx] & value) == value;
-}
\ No newline at end of file
+}
