@@ -18,25 +18,14 @@ CR_REG_METADATA(CCategoryHandler, (
 	CR_MEMBER(firstUnused)
 ))
 
-CCategoryHandler* CCategoryHandler::instance = NULL;
 
-CCategoryHandler* CCategoryHandler::Instance() {
-	assert(instance != NULL);
-	return instance;
-}
+static CCategoryHandler instance;
 
 
-void CCategoryHandler::CreateInstance() {
-	if (instance == NULL) {
-		instance = new CCategoryHandler();
-	}
-}
+CCategoryHandler* CCategoryHandler::Instance() { return &instance; }
 
-void CCategoryHandler::RemoveInstance()
-{
-	delete instance;
-	instance = NULL;
-}
+void CCategoryHandler::CreateInstance() { instance.Init(); }
+void CCategoryHandler::RemoveInstance() { instance.Kill(); }
 
 
 unsigned int CCategoryHandler::GetCategory(std::string name)
@@ -44,23 +33,23 @@ unsigned int CCategoryHandler::GetCategory(std::string name)
 	StringTrimInPlace(name);
 	StringToLowerInPlace(name);
 
-	if (name.empty())
-		return 0; // the empty category
-
 	unsigned int cat = 0;
 
+	// the empty category
+	if (name.empty())
+		return cat;
+
 	if (categories.find(name) == categories.end()) {
 		// this category is yet unknown
 		if (firstUnused >= CCategoryHandler::GetMaxCategories()) {
 			// skip this category
-			LOG_L(L_WARNING, "too many unit categories (%i), skipping %s",
-					firstUnused, name.c_str());
+			LOG_L(L_WARNING, "[%s] too many unit categories (%i), skipping %s", __func__, firstUnused, name.c_str());
 			cat = 0;
 		} else {
 			// create the category (bit field value)
 			cat = (1 << firstUnused);
-			//LOG_L(L_DEBUG, "New unit-category %s #%i", name.c_str(), firstUnused);
 		}
+
 		// if (cat == 0), this will prevent further warnings for this category
 		categories[name] = cat;
 		firstUnused++;
@@ -84,9 +73,10 @@ unsigned int CCategoryHandler::GetCategories(std::string names)
 	std::string name;
 
 	while (std::getline(namesStream, name, ' ')) {
-		if (!name.empty()) {
-			ret |= GetCategory(name);
-		}
+		if (name.empty())
+			continue;
+
+		ret |= GetCategory(name);
 	}
 
 	return ret;
@@ -97,13 +87,17 @@ std::vector<std::string> CCategoryHandler::GetCategoryNames(unsigned int bits) c
 {
 	std::vector<std::string> names;
 
+	names.reserve(categories.size());
+
 	for (unsigned int bit = 1; bit != 0; bit = (bit << 1)) {
-		if ((bit & bits) != 0) {
-			for (auto it = categories.cbegin(); it != categories.cend(); ++it) {
-				if (it->second == bit) {
-					names.push_back(it->first);
-				}
-			}
+		if ((bit & bits) == 0)
+			continue;
+
+		for (auto it = categories.cbegin(); it != categories.cend(); ++it) {
+			if (it->second != bit)
+				continue;
+
+			names.push_back(it->first);
 		}
 	}
 
