@@ -15,57 +15,57 @@ using std::cout;
  * @brief Log an error about a DefTagMetaData
  */
 #define LOG_VAR(data, fmt, ...) \
-	LOG_L(L_ERROR, "%s:%d: " fmt, data->GetDeclarationFile().Get().c_str(), data->GetDeclarationLine().Get(), ## __VA_ARGS__) \
+	LOG_L(L_ERROR, "%s:%d: " fmt, (data)->GetDeclarationFile().Get().c_str(), (data)->GetDeclarationLine().Get(), ## __VA_ARGS__) \
 
 
-DefType::~DefType() {
-	for (const DefTagMetaData* md: tags)
-		delete md;
-}
-
-std::vector<const DefType*>& DefType::GetTypes()
-{
-	static std::vector<const DefType*> tagtypes;
-	return tagtypes;
-}
-
-
-DefType::DefType(const std::string& n) : name(n), luaTable(NULL) {
+DefType::DefType(const char* n): name(n) {
+	metaDataMem.fill(0);
+	defInitFuncs.fill(nullptr);
+	tagMetaData.fill(nullptr);
 	GetTypes().push_back(this);
 }
 
 
-void DefType::AddMetaData(const DefTagMetaData* data)
+void DefType::AddTagMetaData(const DefTagMetaData* data)
 {
 	const auto key = data->GetInternalName();
-	for (const DefTagMetaData* md: tags) {
-		if (key == md->GetInternalName()) {
-			LOG_VAR(data, "Duplicate config variable declaration \"%s\"", key.c_str());
-			LOG_VAR(md,   "  Previously declared here");
-			assert(false);
-			return;
-		}
+
+	const auto tend = tagMetaData.begin() + tagMetaDataCnt;
+	const auto pred = [&](const DefTagMetaData* md) { return (key == md->GetInternalName()); };
+	const auto iter = std::find_if(tagMetaData.begin(), tend, pred);
+
+	if (iter != tend) {
+		LOG_VAR(data, "Duplicate config variable declaration \"%s\"", key.c_str());
+		LOG_VAR(*iter, "  Previously declared here");
+		assert(false);
+		return;
+	}
+	if (tagMetaDataCnt >= tagMetaData.size()) {
+		LOG_VAR(data, "Too many config-variable metadata instances");
+		return;
 	}
 
-	tags.push_back(data);
+	tagMetaData[tagMetaDataCnt++] = data;
 }
 
 
 const DefTagMetaData* DefType::GetMetaDataByInternalKey(const string& key)
 {
-	for (const DefTagMetaData* md: tags) {
-		if (key == md->GetInternalName()) {
-			return md;
-		}
-	}
-	return nullptr;
+	const auto tend = tagMetaData.begin() + tagMetaDataCnt;
+	const auto pred = [&](const DefTagMetaData* md) { return (key == md->GetInternalName()); };
+	const auto iter = std::find_if(tagMetaData.begin(), tend, pred);
+
+	return ((iter == tend)? nullptr: *iter);
 }
 
 
 const DefTagMetaData* DefType::GetMetaDataByExternalKey(const string& key)
 {
 	const std::string lkey = StringToLower(key);
-	for (const DefTagMetaData* md: tags) {
+
+	for (unsigned int i = 0; i < tagMetaDataCnt; i++) {
+		const DefTagMetaData* md = tagMetaData[i];
+
 		if (md->GetExternalName().IsSet()) {
 			if (lkey == StringToLower(md->GetExternalName().Get()))
 				return md;
@@ -77,6 +77,7 @@ const DefTagMetaData* DefType::GetMetaDataByExternalKey(const string& key)
 		if (lkey == StringToLower(md->GetFallbackName().Get()))
 			return md;
 	}
+
 	return nullptr;
 }
 
@@ -99,11 +100,11 @@ std::string DefTagMetaData::GetTypeName(const std::type_info& typeInfo)
 /**
  * @brief Call Quote if type is not bool, float or int.
  */
-static inline string Quote(const string& type, const string& value)
+static inline std::string Quote(const std::string& type, const std::string& value)
 {
-	if (type == "std::string") {
+	if (type == "std::string")
 		return Quote(value);
-	}
+
 	return value;
 }
 
@@ -122,39 +123,39 @@ static std::ostream& operator<< (std::ostream& out, const DefTagMetaData* d)
 
 #define KV(key, value) out << INDENT << Quote(#key) << ": " << (value) << ",\n"
 
-	if (d->GetDeclarationFile().IsSet()) {
+	if (d->GetDeclarationFile().IsSet())
 		KV(declarationFile, Quote(d->GetDeclarationFile().Get()));
-	}
-	if (d->GetDeclarationLine().IsSet()) {
+
+	if (d->GetDeclarationLine().IsSet())
 		KV(declarationLine, d->GetDeclarationLine().Get());
-	}
-	if (d->GetExternalName().IsSet()) {
+
+	if (d->GetExternalName().IsSet())
 		KV(internalName, Quote(d->GetInternalName()));
-	}
-	if (d->GetFallbackName().IsSet()) {
+
+	if (d->GetFallbackName().IsSet())
 		KV(fallbackName, Quote(d->GetFallbackName().Get()));
-	}
-	if (d->GetDescription().IsSet()) {
+
+	if (d->GetDescription().IsSet())
 		KV(description, Quote(d->GetDescription().Get()));
-	}
-	if (d->GetDefaultValue().IsSet()) {
+
+	if (d->GetDefaultValue().IsSet())
 		KV(defaultValue, Quote(tname, d->GetDefaultValue().ToString()));
-	}
-	if (d->GetMinimumValue().IsSet()) {
+
+	if (d->GetMinimumValue().IsSet())
 		KV(minimumValue, Quote(tname, d->GetMinimumValue().ToString()));
-	}
-	if (d->GetMaximumValue().IsSet()) {
+
+	if (d->GetMaximumValue().IsSet())
 		KV(maximumValue, Quote(tname, d->GetMaximumValue().ToString()));
-	}
-	if (d->GetScaleValue().IsSet()) {
+
+	if (d->GetScaleValue().IsSet())
 		KV(scaleValue, Quote(tname, d->GetScaleValue().ToString()));
-	}
-	if (d->GetScaleValueStr().IsSet()) {
+
+	if (d->GetScaleValueStr().IsSet())
 		KV(scaleValueString, Quote(d->GetScaleValueStr().ToString()));
-	}
-	if (d->GetTagFunctionStr().IsSet()) {
+
+	if (d->GetTagFunctionStr().IsSet())
 		KV(tagFunction, Quote(d->GetTagFunctionStr().ToString()));
-	}
+
 	// Type is required.
 	// Easiest to do this last because of the trailing comma that isn't there.
 	out << INDENT << Quote("type") << ": " << Quote(tname) << "\n";
@@ -179,11 +180,14 @@ void DefType::OutputMetaDataMap() const
 	cout << "{\n";
 
 	bool first = true;
-	for (auto metaData: tags) {
-		if (!first) {
+
+	for (unsigned int i = 0; i < tagMetaDataCnt; i++) {
+		const DefTagMetaData* md = tagMetaData[i];
+
+		if (!first)
 			cout << ",\n";
-		}
-		cout << metaData;
+
+		cout << md;
 		first = false;
 	}
 
@@ -196,9 +200,9 @@ void DefType::OutputTagMap()
 
 	bool first = true;
 	for (const DefType* defType: GetTypes()) {
-		if (!first) {
+		if (!first)
 			cout << ",\n";
-		}
+
 		cout << "  " << Quote(defType->GetName()) << ": ";
 		defType->OutputMetaDataMap();
 		first = false;
@@ -210,7 +214,7 @@ void DefType::OutputTagMap()
 
 void DefType::CheckType(const DefTagMetaData* meta, const std::type_info& want)
 {
-	assert(meta);
+	assert(meta != nullptr);
 	if (meta->GetTypeInfo() != want)
 		LOG_L(L_ERROR, "DEFTAG \"%s\" defined with wrong typevalue \"%s\" should be \"%s\"", meta->GetKey().c_str(), DefTagMetaData::GetTypeName(meta->GetTypeInfo()).c_str(), DefTagMetaData::GetTypeName(want).c_str());
 	assert(meta->GetTypeInfo() == want);
@@ -221,8 +225,10 @@ void DefType::ReportUnknownTags(const std::string& instanceName, const LuaTable&
 {
 	std::vector<std::string> keys;
 	luaTable.GetKeys(keys);
+
 	for (const std::string& tag: keys) {
 		const DefTagMetaData* meta = GetMetaDataByExternalKey(pre + tag);
+
 		if (meta != nullptr)
 			continue;
 
@@ -231,7 +237,7 @@ void DefType::ReportUnknownTags(const std::string& instanceName, const LuaTable&
 			continue;
 		}
 
-		LOG_L(L_WARNING, "%s: Unknown tag \"%s%s\" in \"%s\"", name.c_str(), pre.c_str(), tag.c_str(), instanceName.c_str());
+		LOG_L(L_WARNING, "%s: Unknown tag \"%s%s\" in \"%s\"", name, pre.c_str(), tag.c_str(), instanceName.c_str());
 	}
 }
 
@@ -240,8 +246,8 @@ void DefType::Load(void* instance, const LuaTable& luaTable)
 {
 	this->luaTable = &luaTable;
 
-	for (const DefInitializer& init: inits) {
-		init(instance);
+	for (unsigned int i = 0; i < defInitFuncCnt; i++) {
+		defInitFuncs[i](instance);
 	}
 
 	this->luaTable = nullptr;
