@@ -5,7 +5,7 @@
 
 #include <vector>
 #include <deque>
-#include <boost/noncopyable.hpp>
+
 #include "Map/Ground.h"
 #include "Sim/Misc/LosMap.h"
 #include "Sim/Objects/WorldObject.h"
@@ -13,7 +13,7 @@
 #include "System/type2.h"
 #include "System/Rectangle.h"
 #include "System/EventClient.h"
-#include <boost/unordered_map.hpp>
+#include "System/UnorderedMap.hpp"
 
 
 /**
@@ -45,7 +45,7 @@ struct SLosInstance
 		, refCount(0)
 		, hashNum(-1)
 		, status(NONE)
-		, isCache(false)
+		, isCached(false)
 		, isQueuedForUpdate(false)
 		, isQueuedForTerraform(false)
 	{}
@@ -68,15 +68,15 @@ public:
 	// helpers
 	int hashNum;
 	enum TLosStatus {
-		NONE = 0,
-		NEW  = 1,
-		REACTIVATE = 2,
-		RECALC = 4,
-		REMOVE = 8,
+		NONE       =  0,
+		NEW        =  1,
+		REACTIVATE =  2,
+		RECALC     =  4,
+		REMOVE     =  8,
 	};
 	int status;
 
-	bool isCache;
+	bool isCached;
 	bool isQueuedForUpdate;
 	bool isQueuedForTerraform;
 };
@@ -92,7 +92,7 @@ public:
  * LosInstance.
  *
  * To quickly find LosInstances that can be shared CLosHandler implements a
- * hash table (instanceHash). Additionally, LosInstances that reach a refCount
+ * hash table (instanceHashes). Additionally, LosInstances that reach a refCount
  * of 0 are not immediately deleted, but up to 500 of those are stored, in case
  * they can be reused for a future unit.
  *
@@ -124,13 +124,14 @@ public:
 		LOS_TYPE_COUNT
 	};
 
-	ILosType(const int mipLevel, LosType type);
+	void Init(const int mipLevel, LosType type);
+	void Kill();
 
 public:
 	void Update();
 	void UpdateHeightMapSynced(SRectangle rect);
 	void RemoveUnit(CUnit* unit, bool delayed = false);
-	void UpdateUnit(CUnit* unit);
+	void UpdateUnit(CUnit* unit, bool ignore = false);
 
 private:
 	//void PostLoad();
@@ -156,21 +157,24 @@ private:
 	float GetHeight(const CUnit* unit) const;
 
 public:
-	const int   mipLevel;
-	const int   divisor;
-	const float invDiv;
-	const int2  size;
-	const LosType type;
-	const LosAlgoType algoType;
-	std::vector<CLosMap> losMaps;
+	int   mipLevel = 0;
+	int   mipDiv = 0;
+
+	float invDiv = 0.0f;
+	int2  size;
+
+	LosType type = LOS_TYPE_LOS;
+	LosAlgoType algoType = LOS_ALGO_RAYCAST;
 
 	static size_t cacheFails;
 	static size_t cacheHits;
-	static size_t cacheReactivated;
+	static size_t cacheRefs;
 
-	boost::unordered_multimap<int, SLosInstance*> instanceHash; // we intentionally use boost version here, gcc's one uses a linked listed and so is much slower
+	spring::unordered_map<int, std::vector<SLosInstance*> > instanceHashes;
+
+	std::vector<CLosMap> losMaps;
 	std::deque<SLosInstance> instances;
-	std::deque<int> freeIDs;
+	std::vector<int> freeIDs;
 
 private:
 	struct DelayedInstance {
@@ -182,6 +186,12 @@ private:
 	std::deque<DelayedInstance> delayedTerraQue;
 	std::deque<SLosInstance*> losUpdate;
 	std::deque<SLosInstance*> losCache;
+
+	std::vector<SLosInstance*> losRemove;
+	std::vector<SLosInstance*> losAdd;
+	std::vector<SLosInstance*> losDeleted;
+	std::vector<SLosInstance*> losRecalc;
+
 	static constexpr int CACHE_SIZE = 4096;
 };
 
@@ -197,8 +207,13 @@ class CLosHandler : public CEventClient
 {
 	CR_DECLARE_STRUCT(CLosHandler)
 public:
-	CLosHandler();
-	~CLosHandler();
+	CLosHandler(): CEventClient("[CLosHandler]", 271993, true) {}
+
+	static void InitStatic();
+	static void KillStatic(bool reload);
+
+	void Init();
+	void Kill();
 
 	// the Interface
 	bool InLos(const CUnit* unit, int allyTeam) const;
@@ -286,7 +301,7 @@ public:
 	* Whether everything on the map is visible at all times to a given ALLYteam
 	* There can never be more allyteams than teams, hence the size is MAX_TEAMS
 	*/
-	bool globalLOS[MAX_TEAMS];
+	std::array<bool, MAX_TEAMS> globalLOS;
 
 	ILosType los;
 	ILosType airLos;
@@ -298,12 +313,13 @@ public:
 
 private:
 	static constexpr float defBaseRadarErrorSize = 96.0f;
-	static constexpr float defBaseRadarErrorMult = 20.f;
+	static constexpr float defBaseRadarErrorMult =  2.0f;
+
+	float baseRadarErrorSize = 0.0f;
+	float baseRadarErrorMult = 0.0f;
 
-	float baseRadarErrorSize;
-	float baseRadarErrorMult;
 	std::vector<float> radarErrorSizes;
-	std::vector<ILosType*> losTypes;
+	std::array<ILosType*, 7> losTypes;
 };
 
 
