@@ -10,7 +10,10 @@ class CStaticMoveType : public AMoveType
 	CR_DECLARE_DERIVED(CStaticMoveType)
 
 public:
-	CStaticMoveType(CUnit* unit) : AMoveType(unit) {}
+	CStaticMoveType(CUnit* unit) : AMoveType(unit) {
+		useWantedSpeed[false] = false;
+		useWantedSpeed[ true] = false;
+	}
 
 	void StartMoving(float3 pos, float goalRadius) override {}
 	void StartMoving(float3 pos, float goalRadius, float speed) override {}
