@@ -3,35 +3,37 @@
 #ifndef FLARE_PROJECTILE_H
 #define FLARE_PROJECTILE_H
 
+#include <array>
 #include "Projectile.h"
-#include <vector>
 
 class CFlareProjectile : public CProjectile
 {
 	CR_DECLARE_DERIVED(CFlareProjectile)
 
+private:
+	CFlareProjectile() {}
+
 public:
 	CFlareProjectile(const float3& pos, const float3& speed, CUnit* owner, int activateFrame);
-	~CFlareProjectile();
 
 	void Update() override;
-	void Draw(CVertexArray* va) override;
+	void Draw(GL::RenderDataBufferTC* va) const override;
 
-	int GetProjectilesCount() const override;
+	int GetProjectilesCount() const override { return (subProjPos.size()); }
 
 public:
 	int activateFrame;
 	int deathFrame;
 
-	int numSub;
-	int lastSub;
-	std::vector<float3> subPos;
-	std::vector<float3> subSpeed;
+	unsigned int numSubProjs;
+	unsigned int maxSubProjs;
+	unsigned int lastSubProj;
+
+	// position and velocity for each sub-salvo particle
+	std::array<float3, 16> subProjPos;
+	std::array<float3, 16> subProjVel;
+
 	float alphaFalloff;
-
-private:
-	CFlareProjectile() { }
-
 };
 
 #endif // FLARE_PROJECTILE_H
