@@ -17,37 +17,40 @@ struct Command;
 class CMobileCAI : public CCommandAI
 {
 public:
-	CR_DECLARE(CMobileCAI)
+	CR_DECLARE_DERIVED(CMobileCAI)
 	CMobileCAI(CUnit* owner);
 	CMobileCAI();
 	virtual ~CMobileCAI();
 
 	virtual void SetGoal(const float3& pos, const float3& curPos, float goalRadius = SQUARE_SIZE);
 	virtual void SetGoal(const float3& pos, const float3& curPos, float goalRadius, float speed);
-	virtual void BuggerOff(const float3& pos, float radius);
-	bool SetFrontMoveCommandPos(const float3& pos);
-	void StopMove();
-	void StopMoveAndFinishCommand();
-	void StopMoveAndKeepPointing(const float3& p, const float r, bool b);
+	virtual void BuggerOff(const float3& pos, float radius) override;
 
-	bool AllowedCommand(const Command& c, bool fromSynced);
-	int GetDefaultCmd(const CUnit* pointed, const CFeature* feature);
-	void SlowUpdate();
-	void GiveCommandReal(const Command& c, bool fromSynced = true);
+	bool SetFrontMoveCommandPos(const float3& pos);
+
+	void StopMove() override;
+	void StopMoveAndKeepPointing(const float3& p, const float r, bool b);
+	void StopMoveAndFinishCommand() {
+		StopMove();
+		FinishCommand();
+	}
+
+	bool AllowedCommand(const Command& c, bool fromSynced) override;
+	int GetDefaultCmd(const CUnit* pointed, const CFeature* feature) override;
+	void SlowUpdate() override;
+	void GiveCommandReal(const Command& c, bool fromSynced = true) override;
 	void NonMoving();
-	void FinishCommand();
-	bool CanSetMaxSpeed() const { return true; }
+	void FinishCommand() override;
 	void StopSlowGuard();
 	void StartSlowGuard(float speed);
-	void ExecuteAttack(Command& c);
-	void ExecuteStop(Command& c);
+	void ExecuteAttack(Command& c) override;
+	void ExecuteStop(Command& c) override;
 
 	virtual void Execute();
 	virtual void ExecuteGuard(Command& c);
 	virtual void ExecuteFight(Command& c);
 	virtual void ExecutePatrol(Command& c);
 	virtual void ExecuteMove(Command& c);
-	virtual void ExecuteSetWantedMaxSpeed(Command& c);
 	virtual void ExecuteLoadOnto(Command& c);
 
 	virtual void ExecuteUnloadUnit(Command& c);
@@ -56,8 +59,8 @@ public:
 
 	int GetCancelDistance() { return cancelDistance; }
 
-	virtual bool IsValidTarget(const CUnit* enemy) const;
-	virtual bool CanWeaponAutoTarget(const CWeapon* weapon) const;
+	virtual bool IsValidTarget(const CUnit* enemy, CWeapon* weapon) const;
+	virtual bool CanWeaponAutoTarget(const CWeapon* weapon) const override;
 
 	void SetTransportee(CUnit* unit);
 	bool FindEmptySpot(const CUnit* unloadee, const float3& center, float radius, float spread, float3& found, bool fromSynced = true);
@@ -76,30 +79,45 @@ public:
 
 	float3 lastBuggerGoalPos;
 	float3 lastUserGoal;
-
-	int lastIdleCheck;
-	bool tempOrder;
-
-	/// helps avoid infinate loops
-	int lastPC;
-
-	int lastBuggerOffTime;
-	float3 buggerOffPos;
-	float buggerOffRadius;
-
-	float repairBelowHealth;
 	/**
-	 * Used to avoid stuff in maneuvre mode moving too far away from patrol path
+	 * Used to avoid stuff in maneuver-mode moving too far away from patrol path
 	 */
 	float3 commandPos1;
 	float3 commandPos2;
 
+	float3 buggerOffPos;
+
+	float buggerOffRadius;
+	float repairBelowHealth;
+
+	bool tempOrder;
+
 protected:
-	int cancelDistance;
-	int lastCloseInTry;
 	bool slowGuard;
 	bool moveDir;
 
+	int cancelDistance = 1024;
+
+	/// last frame certain types of area-commands were handled, helps avoid infinite loops
+	int lastCommandFrame = -1;
+	int lastCloseInTry = -1;
+	int lastBuggerOffTime = -BUGGER_OFF_TTL;
+	int lastIdleCheck = 0;
+	int numNonMovingCalls = 0;
+
+	static constexpr int MAX_CLOSE_IN_RETRY_TICKS = 30;
+	static constexpr int BUGGER_OFF_TTL = 200;
+
+	static constexpr float MAX_USERGOAL_TOLERANCE_DIST = 100.0f;
+	static constexpr float AIRTRANSPORT_DOCKING_RADIUS = 16.0f;
+	static constexpr float AIRTRANSPORT_DOCKING_ANGLE = 50.0f;
+
+	enum {
+		UNLOAD_LAND = 0,
+		UNLOAD_DROP = 1,
+		UNLOAD_LANDFLOOD = 2,
+	};
+
 	void PushOrUpdateReturnFight() {
 		CCommandAI::PushOrUpdateReturnFight(commandPos1, commandPos2);
 	}
@@ -107,6 +125,9 @@ protected:
 	void CalculateCancelDistance();
 
 private:
+	void ExecuteObjectAttack(Command& c);
+	void ExecuteGroundAttack(Command& c);
+
 	bool MobileAutoGenerateTarget();
 	bool GenerateAttackCmd();
 };
