@@ -3,6 +3,7 @@
 #ifndef COB_FILE_H
 #define COB_FILE_H
 
+#include <array>
 #include <vector>
 #include <string>
 
@@ -16,11 +17,30 @@ class CCobFile
 {
 public:
 	CCobFile(CFileHandler& in, const std::string& scriptName);
+	CCobFile(CCobFile&& f) { *this = std::move(f); }
+
+	CCobFile& operator = (CCobFile&& f) {
+		numStaticVars = f.numStaticVars;
+
+		code = std::move(f.code);
+		scriptNames = std::move(f.scriptNames);
+		scriptOffsets = std::move(f.scriptOffsets);
+
+		scriptLengths = std::move(f.scriptLengths);
+		pieceNames = std::move(f.pieceNames);
+		scriptIndex = std::move(f.scriptIndex);
+		sounds = std::move(f.sounds);
+		luaScripts = std::move(f.luaScripts);
+		scriptMap = std::move(f.scriptMap);
+
+		name = std::move(f.name);
+		return *this;
+	}
 
 	int GetFunctionId(const std::string& name);
 
 public:
-	int numStaticVars;
+	int numStaticVars = 0;
 
 	std::vector<int> code;
 	std::vector<std::string> scriptNames;
@@ -28,7 +48,7 @@ public:
 	/// Assumes that the scripts are sorted by offset in the file
 	std::vector<int> scriptLengths;
 	std::vector<std::string> pieceNames;
-	std::vector<int> scriptIndex;
+	std::array<int, COBFN_NumUnitFuncs> scriptIndex;
 	std::vector<int> sounds;
 	std::vector<LuaHashString> luaScripts;
 	spring::unordered_map<std::string, int> scriptMap;
@@ -37,3 +57,4 @@ public:
 };
 
 #endif // COB_FILE_H
+
