@@ -3,6 +3,7 @@
 
 #include "CobEngine.h"
 #include "CobFile.h"
+#include "CobFileHandler.h"
 #include "CobInstance.h"
 #include "CobThread.h"
 
@@ -30,9 +31,8 @@
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
 #include "System/StringUtil.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 #include "System/Sound/ISoundChannels.h"
-#include "System/Sync/SyncTracer.h"
 
 
 /******************************************************************************/
@@ -41,11 +41,11 @@
 CR_BIND_DERIVED(CCobInstance, CUnitScript, )
 
 CR_REG_METADATA(CCobInstance, (
-	CR_MEMBER(staticVars),
-	CR_MEMBER(threads),
+	// loaded from cobFileHandler
+	CR_IGNORED(cobFile),
 
-	//loaded from cobFileHandler
-	CR_IGNORED(script),
+	CR_MEMBER(staticVars),
+	CR_MEMBER(threadIDs),
 
 	CR_POSTLOAD(PostLoad)
 ))
@@ -53,30 +53,17 @@ CR_REG_METADATA(CCobInstance, (
 
 inline bool CCobInstance::HasFunction(int id) const
 {
-	return (script->scriptIndex.size() > id && script->scriptIndex[id] >= 0);
+	return (cobFile->scriptIndex.size() > id && cobFile->scriptIndex[id] >= 0);
 }
 
-//Used by creg
-CCobInstance::CCobInstance()
-	: CUnitScript(nullptr)
-	, script(nullptr)
-{ }
 
-CCobInstance::CCobInstance(CCobFile* _script, CUnit* _unit)
-	: CUnitScript(_unit)
-	, script(_script)
-{
-	Init();
-}
 
 void CCobInstance::Init()
 {
-	assert(script != nullptr);
+	assert(cobFile != nullptr);
 
-	staticVars.reserve(script->numStaticVars);
-	for (int i = 0; i < script->numStaticVars; ++i) {
-		staticVars.push_back(0);
-	}
+	staticVars.clear();
+	staticVars.resize(cobFile->numStaticVars, 0);
 
 	MapScriptToModelPieces(&unit->localModel);
 
@@ -89,32 +76,43 @@ void CCobInstance::Init()
 void CCobInstance::PostLoad()
 {
 	assert(unit != nullptr);
-	script = cobFileHandler->GetCobFile(unit->unitDef->scriptName);
+	assert(cobFile == nullptr);
+
+	cobFile = cobFileHandler->GetCobFile(unit->unitDef->scriptName);
+
+	for (int threadID: threadIDs) {
+		CCobThread* t = cobEngine->GetThread(threadID);
+
+		t->cobInst = this;
+		t->cobFile = cobFile;
+	}
+
 	Init();
 }
 
 
 CCobInstance::~CCobInstance()
 {
-	//this may be dangerous, is it really desired?
-	//Destroy();
-	// Deleting waiting threads and unregistering all callbacks
-	while (!threads.empty()) {
-		CCobThread* t = threads.back();
-		t->owner = nullptr;
-		if (t->IsWaiting()) {
-			delete t;
-		} else {
-			t->state = CCobThread::Dead;
-		}
-		threads.pop_back();
+	// this may be dangerous, is it really desired?
+	// Destroy();
+
+	// delete our threads, make sure callbacks do not run
+	while (!threadIDs.empty()) {
+		CCobThread* t = cobEngine->GetThread(threadIDs.back());
+
+		t->MakeGarbage();
+		cobEngine->RemoveThread(t->GetID());
+
+		threadIDs.pop_back();
 	}
+
+	cobEngine->SanityCheckThreads(this);
 }
 
 
 void CCobInstance::MapScriptToModelPieces(LocalModel* lmodel)
 {
-	std::vector<std::string>& pieceNames = script->pieceNames; // already in lowercase!
+	std::vector<std::string>& pieceNames = cobFile->pieceNames; // already in lowercase!
 	std::vector<LocalModelPiece>& lmodelPieces = lmodel->pieces;
 
 	pieces.clear();
@@ -148,11 +146,11 @@ void CCobInstance::MapScriptToModelPieces(LocalModel* lmodel)
 			lmodelPieces[lmodelPieceNum].SetScriptPieceIndex(scriptPieceNum);
 			pieces.push_back(&lmodelPieces[lmodelPieceNum]);
 		} else {
-			pieces.push_back(NULL);
+			pieces.push_back(nullptr);
 
 			const char* fmtString = "[%s] could not find piece named \"%s\" (referenced by COB script \"%s\")";
 			const char* pieceName = pieceNames[scriptPieceNum].c_str();
-			const char* scriptName = script->name.c_str();
+			const char* scriptName = cobFile->name.c_str();
 
 			LOG_L(L_WARNING, fmtString, __FUNCTION__, pieceName, scriptName);
 		}
@@ -162,7 +160,7 @@ void CCobInstance::MapScriptToModelPieces(LocalModel* lmodel)
 
 int CCobInstance::GetFunctionId(const std::string& fname) const
 {
-	return script->GetFunctionId(fname);
+	return cobFile->GetFunctionId(fname);
 }
 
 
@@ -189,22 +187,11 @@ void CCobInstance::Create()
 
 	for (const CWeapon* w: unit->weapons) {
 		maxReloadTime = std::max(maxReloadTime, w->reloadTime);
-
-		#if 0
-		if (dynamic_cast<CBeamLaser*>(w))
-			maxReloadTime = 150; // ???
-		#endif
 	}
 
 	// convert ticks to milliseconds
 	maxReloadTime *= GAME_SPEED;
 
-	#if 0
-	// TA does some special handling depending on weapon count, Spring != TA
-	if (unit->weapons.size() > 1)
-		maxReloadTime = std::max(maxReloadTime, 3000);
-	#endif
-
 	Call(COBFN_Create);
 	Call(COBFN_SetMaxReloadTime, maxReloadTime);
 }
@@ -213,62 +200,68 @@ void CCobInstance::Create()
 
 void CCobInstance::Killed()
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back((int) (unit->recentDamage / unit->maxHealth * 100));
-	args.push_back(0);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
 
-	Call(COBFN_Killed, args, CBKilled, 0, nullptr);
+	callinArgs[0] = 2;
+	callinArgs[1] = int(unit->recentDamage / unit->maxHealth * 100);
+	callinArgs[2] = 0;
+
+	Call(COBFN_Killed, callinArgs, CBKilled, 0, nullptr);
 }
 
 
 void CCobInstance::WindChanged(float heading, float speed)
 {
-	Call(COBFN_SetSpeed, (int)(speed * 3000.0f));
+	Call(COBFN_SetSpeed, int(speed * 3000.0f));
 	Call(COBFN_SetDirection, short(heading * RAD2TAANG));
 }
 
 
 void CCobInstance::ExtractionRateChanged(float speed)
 {
-	Call(COBFN_SetSpeed, (int)(speed * 500.0f));
-	if (unit->activated) {
-		Call(COBFN_Go);
-	}
+	Call(COBFN_SetSpeed, int(speed * 500.0f));
+
+	if (!unit->activated)
+		return;
+
+	Call(COBFN_Go);
 }
 
 
 void CCobInstance::RockUnit(const float3& rockDir)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(rockDir.z);
-	args.push_back(rockDir.x);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
 
-	Call(COBFN_RockUnit, args);
+	callinArgs[0] = 2;
+	callinArgs[1] = int(rockDir.z);
+	callinArgs[2] = int(rockDir.x);
+
+	Call(COBFN_RockUnit, callinArgs);
 }
 
 
 void CCobInstance::HitByWeapon(const float3& hitDir, int weaponDefId, float& inoutDamage)
 {
-	vector<int> args;
-	args.reserve(4);
-	args.push_back(hitDir.z);
-	args.push_back(hitDir.x);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = int(hitDir.z);
+	callinArgs[2] = int(hitDir.x);
 
 	if (HasFunction(COBFN_HitByWeaponId)) {
 		const WeaponDef* wd = weaponDefHandler->GetWeaponDefByID(weaponDefId);
 
-		args.push_back((wd != nullptr)? wd->tdfId : -1);
-		args.push_back((int)(100 * inoutDamage));
+		callinArgs[0] = 4;
+		callinArgs[3] = ((wd != nullptr)? wd->tdfId : -1);
+		callinArgs[4] = int(100 * inoutDamage);
+		// weaponHitMod, not an actual arg
+		callinArgs[MAX_COB_ARGS] = 1;
 
-		int weaponHitMod = 1;
+		Call(COBFN_HitByWeaponId, callinArgs, CBNone, 0, &callinArgs[MAX_COB_ARGS]);
 
-		Call(COBFN_HitByWeaponId, args, CBNone, 0, &weaponHitMod);
-
-		inoutDamage *= weaponHitMod * 0.01f;
+		inoutDamage *= (callinArgs[MAX_COB_ARGS] * 0.01f);
 	} else {
-		Call(COBFN_HitByWeapon, args);
+		Call(COBFN_HitByWeapon, callinArgs);
 	}
 }
 
@@ -279,119 +272,152 @@ void CCobInstance::SetSFXOccupy(int curTerrainType)
 }
 
 
-void CCobInstance::QueryLandingPads(std::vector<int>& out_pieces)
+void CCobInstance::QueryLandingPads(std::vector<int>& outPieces)
 {
-	int maxPadCount = 16; // default max pad count
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 1;
+	callinArgs[1] = MAX_COB_ARGS; // default max pad count
 
 	if (HasFunction(COBFN_QueryLandingPadCount)) {
-		vector<int> args;
-		args.push_back(maxPadCount);
-		Call(COBFN_QueryLandingPadCount, args);
-		maxPadCount = args[0];
+		// query the count
+		Call(COBFN_QueryLandingPadCount, callinArgs);
+
+		// setup count as first arg for next query
+		callinArgs[1] = callinArgs[0];
+		callinArgs[0] = 1;
 	}
 
-	for (int i = 0; i < maxPadCount; i++) {
-		out_pieces.push_back(-1);
-	}
+	const int maxPadCount = std::min(callinArgs[1], int(MAX_COB_ARGS));
 
-	Call(COBFN_QueryLandingPad, out_pieces);
+	// query the pieces
+	Call(COBFN_QueryLandingPad, callinArgs);
+
+	outPieces.clear();
+	outPieces.resize(maxPadCount);
+	outPieces.assign(callinArgs.begin(), callinArgs.begin() + maxPadCount);
 }
 
 
 void CCobInstance::BeginTransport(const CUnit* unit)
 {
-	// yes, COB is silly, while it handles integers fine it uses model height to identify units
-	Call(COBFN_BeginTransport, (int)(unit->model->height*65536));
+	// COB uses model height to identify units
+	Call(COBFN_BeginTransport, int(unit->model->height * 65536));
 }
 
 
 int CCobInstance::QueryTransport(const CUnit* unit)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(0);
-	args.push_back((int)(unit->model->height*65536));
-	Call(COBFN_QueryTransport, args);
-	return args[0];
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = 0;
+	callinArgs[2] = int(unit->model->height * 65536);
+
+	Call(COBFN_QueryTransport, callinArgs);
+	return callinArgs[0];
 }
 
 
 void CCobInstance::TransportPickup(const CUnit* unit)
 {
-	// funny, now it uses unitIDs instead of model height
+	// here COB uses unitIDs instead of model height
 	Call(COBFN_TransportPickup, unit->id);
 }
 
 
 void CCobInstance::TransportDrop(const CUnit* unit, const float3& pos)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(unit->id);
-	args.push_back(PACKXZ(pos.x, pos.z));
-	Call(COBFN_TransportDrop, args);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = unit->id;
+	callinArgs[2] = PACKXZ(pos.x, pos.z);
+
+	Call(COBFN_TransportDrop, callinArgs);
 }
 
 
 void CCobInstance::StartBuilding(float heading, float pitch)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(short(heading * RAD2TAANG));
-	args.push_back(short(pitch * RAD2TAANG));
-	Call(COBFN_StartBuilding, args);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = short(heading * RAD2TAANG);
+	callinArgs[2] = short(  pitch * RAD2TAANG);
+
+	Call(COBFN_StartBuilding, callinArgs);
 }
 
 
 int CCobInstance::QueryNanoPiece()
 {
-	vector<int> args(1, -1);
-	Call(COBFN_QueryNanoPiece, args);
-	return args[0];
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] =  1;
+	callinArgs[1] = -1;
+
+	Call(COBFN_QueryNanoPiece, callinArgs);
+	return callinArgs[0];
 }
 
 
 int CCobInstance::QueryBuildInfo()
 {
-	vector<int> args(1, -1);
-	Call(COBFN_QueryBuildInfo, args);
-	return args[0];
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] =  1;
+	callinArgs[1] = -1;
+
+	Call(COBFN_QueryBuildInfo, callinArgs);
+	return callinArgs[0];
 }
 
 
 int CCobInstance::QueryWeapon(int weaponNum)
 {
-	vector<int> args(1, -1);
-	Call(COBFN_QueryPrimary + COBFN_Weapon_Funcs * weaponNum, args);
-	return args[0];
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] =  1;
+	callinArgs[1] = -1;
+
+	Call(COBFN_QueryPrimary + COBFN_Weapon_Funcs * weaponNum, callinArgs);
+	return callinArgs[0];
 }
 
 
 void CCobInstance::AimWeapon(int weaponNum, float heading, float pitch)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(short(heading * RAD2TAANG));
-	args.push_back(short(pitch * RAD2TAANG));
-	Call(COBFN_AimPrimary + COBFN_Weapon_Funcs * weaponNum, args, CBAimWeapon, weaponNum, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = short(heading * RAD2TAANG);
+	callinArgs[2] = short(  pitch * RAD2TAANG);
+
+	Call(COBFN_AimPrimary + COBFN_Weapon_Funcs * weaponNum, callinArgs, CBAimWeapon, weaponNum, nullptr);
 }
 
 
 void CCobInstance::AimShieldWeapon(CPlasmaRepulser* weapon)
 {
-	vector<int> args;
-	args.reserve(2);
-	args.push_back(0); // compat with AimWeapon (same script is called)
-	args.push_back(0);
-	Call(COBFN_AimPrimary + COBFN_Weapon_Funcs * weapon->weaponNum, args, CBAimShield, weapon->weaponNum, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 2;
+	callinArgs[1] = 0; // compat with AimWeapon (same script is called)
+
+	Call(COBFN_AimPrimary + COBFN_Weapon_Funcs * weapon->weaponNum, callinArgs, CBAimShield, weapon->weaponNum, nullptr);
 }
 
 
 int CCobInstance::AimFromWeapon(int weaponNum)
 {
-	vector<int> args(1, -1);
-	Call(COBFN_AimFromPrimary + COBFN_Weapon_Funcs * weaponNum, args);
-	return args[0];
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] =  1;
+	callinArgs[1] = -1;
+
+	Call(COBFN_AimFromPrimary + COBFN_Weapon_Funcs * weaponNum, callinArgs);
+	return callinArgs[0];
 }
 
 
@@ -403,41 +429,36 @@ void CCobInstance::Shot(int weaponNum)
 
 bool CCobInstance::BlockShot(int weaponNum, const CUnit* targetUnit, bool userTarget)
 {
-	const int unitID = targetUnit ? targetUnit->id : 0;
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
 
-	vector<int> args;
-	args.reserve(3);
+	callinArgs[0] = 3;
+	callinArgs[1] = (targetUnit != nullptr)? targetUnit->id : 0;
+	callinArgs[2] = 0; // return value; default is to not block the shot
+	callinArgs[3] = userTarget;
 
-	args.push_back(unitID);
-	args.push_back(0); // arg[1], for the return value
-	                   // the default is to not block the shot
-	args.push_back(userTarget);
+	Call(COBFN_BlockShot + COBFN_Weapon_Funcs * weaponNum, callinArgs);
 
-	Call(COBFN_BlockShot + COBFN_Weapon_Funcs * weaponNum, args);
-
-	return !!args[1];
+	return (callinArgs[1] != 0);
 }
 
 
 float CCobInstance::TargetWeight(int weaponNum, const CUnit* targetUnit)
 {
-	const int unitID = targetUnit ? targetUnit->id : 0;
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
 
-	vector<int> args;
-	args.reserve(2);
+	callinArgs[0] = 2;
+	callinArgs[1] = (targetUnit != nullptr)? targetUnit->id : 0;
+	callinArgs[2] = COBSCALE; // return value; default is 1.0
 
-	args.push_back(unitID);
-	args.push_back(COBSCALE); // arg[1], for the return value
-	                          // the default is 1.0
+	Call(COBFN_TargetWeight + COBFN_Weapon_Funcs * weaponNum, callinArgs);
 
-	Call(COBFN_TargetWeight + COBFN_Weapon_Funcs * weaponNum, args);
-
-	return (float)args[1] / (float)COBSCALE;
+	return (callinArgs[1] * 1.0f / COBSCALE);
 }
 
 void CCobInstance::AnimFinished(AnimType type, int piece, int axis)
 {
-	for (CCobThread* t: threads) {
+	for (int threadID: threadIDs) {
+		CCobThread* t = cobEngine->GetThread(threadID);
 		t->AnimFinished(type, piece, axis);
 	}
 }
@@ -471,60 +492,68 @@ void CCobInstance::EndBurst(int weaponNum) { Call(COBFN_EndBurst + COBFN_Weapon_
  * @return 0 if the call terminated. If the caller provides a callback and the thread does not terminate,
  *  it will continue to run. Otherwise it will be killed. Returns 1 in this case.
  */
-int CCobInstance::RealCall(int functionId, vector<int>& args, ThreadCallbackType cb, int cbParam, int* retCode)
+int CCobInstance::RealCall(int functionId, std::array<int, 1 + MAX_COB_ARGS>& args, ThreadCallbackType cb, int cbParam, int* retCode)
 {
+	int ret = -1;
 
-	if (functionId < 0 || size_t(functionId) >= script->scriptNames.size()) {
+	if (size_t(functionId) >= cobFile->scriptNames.size()) {
 		if (retCode != nullptr)
-			*retCode = -1;
+			*retCode = ret;
 
-		if (cb != CBNone) {
-			// in case the function does not exist the callback should
-			// still be called; -1 is the default CobThread return code
+		// in case the function does not exist the callback should
+		// still be called; -1 is the default CobThread return code
+		if (cb != CBNone)
 			ThreadCallback(cb, -1, cbParam);
-		}
-		return -1;
+
+		return ret;
 	}
 
+	// LOG_L(L_DEBUG, "Calling %s:%s", cobFile->name.c_str(), cobFile->scriptNames[functionId].c_str());
 
-	CCobThread* thread = new CCobThread(this);
-	thread->Start(functionId, args, false);
+	// tick the thread locally in case we're recursively running this function and then the threads may reallocate
+	CCobThread newThread(this);
+	newThread.SetID(cobEngine->GenThreadID());
 
-	//LOG_L(L_DEBUG, "Calling %s:%s", script->name.c_str(), script->scriptNames[functionId].c_str());
-
-	const bool res = thread->Tick();
-
-	// Make sure this is run even if the call terminates instantly
+	// make sure this is run even if the call terminates instantly
 	if (cb != CBNone)
-		thread->SetCallback(cb, cbParam);
+		newThread.SetCallback(cb, cbParam);
 
-	if (!res) {
+	newThread.Start(functionId, 0, args, false);
+
+	if ((ret = newThread.Tick()) == 0) {
 		// thread died already after one tick
 		// NOTE:
+		//   ticking can trigger recursion, for example FireWeapon ->
+		//   Call(COBFN_FirePrimary + N*) -> EmitSFX -> FireWeapon ->
+		//   ...
+		//
 		//   the StartMoving callin now takes an argument which means
 		//   there will be a mismatch between the number of arguments
-		//   passed in (1) and the number returned (0) as of 95.0 -->
-		//   prevent error-spam
-		unsigned int i = 0, argc = thread->CheckStack(args.size(), functionId != script->scriptIndex[COBFN_StartMoving]);
+		//   passed in (1) and the number returned (0) as of 95.0, so
+		//   prevent error-spam by checking functionId
+		//
+		//   args[0] holds the number of input args
+		const unsigned int numArgs = args[0];
+		const unsigned int retArgs = newThread.CheckStack(numArgs, functionId != cobFile->scriptIndex[COBFN_StartMoving]);
 
-		// Retrieve parameter values from stack
-		for (; i < argc; ++i)
-			args[i] = thread->GetStackVal(i);
+		// retrieve output parameter values from stack
+		for (unsigned int i = 0, n = std::min(retArgs, MAX_COB_ARGS); i < n; ++i)
+			args[i] = newThread.GetStackVal(i);
 
-		// Set erroneous parameters to 0
-		for (; i < args.size(); ++i)
+		// set erroneous parameters to 0
+		for (unsigned int i = std::min(retArgs, MAX_COB_ARGS); i < numArgs; ++i)
 			args[i] = 0;
 
 		// dtor runs the callback
 		if (retCode != nullptr)
-			*retCode = thread->GetRetCode();
-		delete thread;
-		return 0;
+			*retCode = newThread.GetRetCode();
+	} else {
+		cobEngine->AddThread(std::move(newThread));
 	}
 
-	// thread has already added itself to the correct
-	// scheduler (global for sleep, or local for anim)
-	return 1;
+	// handle any spawned threads
+	cobEngine->AddQueuedThreads();
+	return ret;
 }
 
 
@@ -533,24 +562,29 @@ int CCobInstance::RealCall(int functionId, vector<int>& args, ThreadCallbackType
 
 int CCobInstance::Call(const std::string& fname)
 {
-	vector<int> x;
-	return Call(fname, x, CBNone, 0, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs = {{0}};
+
+	return Call(fname, callinArgs, CBNone, 0, nullptr);
 }
 
-int CCobInstance::Call(const std::string& fname, std::vector<int>& args)
+int CCobInstance::Call(const std::string& fname, std::array<int, 1 + MAX_COB_ARGS>& args)
 {
 	return Call(fname, args, CBNone, 0, nullptr);
 }
 
 int CCobInstance::Call(const std::string& fname, int arg1)
 {
-	vector<int> x;
-	x.reserve(1);
-	x.push_back(arg1);
-	return Call(fname, x, CBNone, 0, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 1;
+	callinArgs[1] = arg1;
+
+	return Call(fname, callinArgs, CBNone, 0, nullptr);
 }
 
-int CCobInstance::Call(const std::string& fname, std::vector<int>& args, ThreadCallbackType cb, int cbParam, int* retCode)
+
+
+int CCobInstance::Call(const std::string& fname, std::array<int, 1 + MAX_COB_ARGS>& args, ThreadCallbackType cb, int cbParam, int* retCode)
 {
 	//TODO: Check that new behaviour of actually calling cb when the function is not defined is right?
 	//      (the callback has always been called [when the function is not defined]
@@ -559,61 +593,68 @@ int CCobInstance::Call(const std::string& fname, std::vector<int>& args, ThreadC
 }
 
 
+
 int CCobInstance::Call(int id)
 {
-	vector<int> x;
-	return Call(id, x, CBNone, 0, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs = {{0}};
+
+	return Call(id, callinArgs, CBNone, 0, nullptr);
 }
 
 int CCobInstance::Call(int id, int arg1)
 {
-	vector<int> x;
-	x.reserve(1);
-	x.push_back(arg1);
-	return Call(id, x, CBNone, 0, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs;
+
+	callinArgs[0] = 1;
+	callinArgs[1] = arg1;
+
+	return Call(id, callinArgs, CBNone, 0, nullptr);
 }
 
-int CCobInstance::Call(int id, std::vector<int>& args)
+int CCobInstance::Call(int id, std::array<int, 1 + MAX_COB_ARGS>& args)
 {
 	return Call(id, args, CBNone, 0, nullptr);
 }
 
-int CCobInstance::Call(int id, std::vector<int>& args, ThreadCallbackType cb, int cbParam, int* retCode)
+int CCobInstance::Call(int id, std::array<int, 1 + MAX_COB_ARGS>& args, ThreadCallbackType cb, int cbParam, int* retCode)
 {
-	return RealCall(script->scriptIndex[id], args, cb, cbParam, retCode);
+	return RealCall(cobFile->scriptIndex[id], args, cb, cbParam, retCode);
 }
 
 
 void CCobInstance::RawCall(int fn)
 {
-	vector<int> x;
-	RealCall(fn, x, CBNone, 0, nullptr);
+	std::array<int, 1 + MAX_COB_ARGS> callinArgs = {{0}};
+
+	RealCall(fn, callinArgs, CBNone, 0, nullptr);
 }
 
-int CCobInstance::RawCall(int fn, std::vector<int> &args)
+int CCobInstance::RawCall(int fn, std::array<int, 1 + MAX_COB_ARGS>& args)
 {
 	return RealCall(fn, args, CBNone, 0, nullptr);
 }
 
+
 void CCobInstance::ThreadCallback(ThreadCallbackType type, int retCode, int cbParam)
 {
-	switch(type) {
+	switch (type) {
 		// note: this callback is always called, even if Killed does not exist
 		// however, retCode is only set if the function has a return statement
 		// (otherwise its value is -1 regardless of Killed being present which
 		// means *no* wreck will be spawned)
-		case CBKilled:
+		case CBKilled: {
 			unit->KilledScriptFinished(retCode);
-			break;
-		case CBAimWeapon:
+		} break;
+		case CBAimWeapon: {
 			unit->weapons[cbParam]->AimScriptFinished(retCode == 1);
-			break;
-		case CBAimShield:
+		} break;
+		case CBAimShield: {
 			static_cast<CPlasmaRepulser*>(unit->weapons[cbParam])->SetEnabled(retCode != 0);
-			break;
-		default:
+		} break;
+		default: {
 			assert(false);
-		}
+		} break;
+	}
 }
 
 /******************************************************************************/
@@ -622,18 +663,20 @@ void CCobInstance::ThreadCallback(ThreadCallbackType type, int retCode, int cbPa
 
 void CCobInstance::Signal(int signal)
 {
-	for (CCobThread* t: threads) {
-		if ((signal & t->signalMask) != 0) {
-			t->state = CCobThread::Dead;
-			//LOG_L(L_DEBUG, "Killing a thread %d %d", signal, (*i)->signalMask);
-		}
+	for (int threadID: threadIDs) {
+		CCobThread* t = cobEngine->GetThread(threadID);
+
+		if ((signal & t->GetSignalMask()) == 0)
+			continue;
+
+		t->SetState(CCobThread::Dead);
 	}
 }
 
 
 void CCobInstance::PlayUnitSound(int snr, int attr)
 {
-	Channels::UnitReply->PlaySample(script->sounds[snr], unit->pos, unit->speed, attr);
+	Channels::UnitReply->PlaySample(cobFile->sounds[snr], unit->pos, unit->speed, attr);
 }
 
 
