@@ -4,7 +4,7 @@
 #define LUASCRIPTNAMES_H
 
 #include <string>
-#include <vector>
+#include <array>
 
 #include "System/UnorderedMap.hpp"
 
@@ -62,11 +62,13 @@ enum {
 class CLuaUnitScriptNames
 {
 public:
-	static const std::vector<std::string>& GetScriptNames(); // LUAFN_* -> string
+	static void InitScriptNames();
+
+	static const std::array<std::string, LUAFN_Last>& GetScriptNames(); // LUAFN_* -> string
 	static const spring::unordered_map<std::string, int>& GetScriptMap(); // string -> LUAFN_*
 
 	static int GetScriptNumber(const std::string& fname);
-	static const std::string& GetScriptName(int num);
+	static const std::string& GetScriptName(unsigned int num);
 };
 
 #endif
