@@ -5,9 +5,10 @@
 #ifndef UNIT_SCRIPT_ENGINE_H
 #define UNIT_SCRIPT_ENGINE_H
 
-#include "System/creg/creg_cond.h"
 #include <vector>
 
+#include "System/creg/creg_cond.h"
+
 struct UnitDef;
 class CUnit;
 class CUnitScript;
@@ -17,22 +18,23 @@ class CUnitScriptEngine
 {
 	CR_DECLARE_STRUCT(CUnitScriptEngine)
 
-protected:
-	std::vector<CUnitScript*> animating; // hash would be optimal, but not crucial
-	void CheckForDuplicates(const char* name, const CUnitScript* instance);
-
 public:
-	CUnitScriptEngine(): currentScript(nullptr) {}
-
 	void AddInstance(CUnitScript* instance);
 	void RemoveInstance(CUnitScript* instance);
 	void ReloadScripts(const UnitDef* udef);
+
 	void Tick(int deltaTime);
 
+	void Init() { animating.reserve(256); }
+	void Kill() { animating.clear(); }
+
 	static void InitStatic();
 	static void KillStatic();
+
 private:
-	CUnitScript* currentScript;
+	CUnitScript* currentScript = nullptr;
+
+	std::vector<CUnitScript*> animating;
 };
 
 extern CUnitScriptEngine* unitScriptEngine;
