@@ -2,7 +2,6 @@
 
 #include "UnitDef.h"
 #include "UnitDefHandler.h"
-#include "UnitDefImage.h"
 #include "Game/GameSetup.h"
 #include "Lua/LuaParser.h"
 #include "Map/MapInfo.h"
@@ -17,22 +16,12 @@
 #include "System/EventHandler.h"
 #include "System/Exceptions.h"
 #include "System/Log/ILog.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 #include "System/SafeUtil.h"
 #include "System/StringUtil.h"
 
 /******************************************************************************/
 
-UnitDefWeapon::UnitDefWeapon()
-: def(NULL)
-, slavedTo(0)
-, maxMainDirAngleDif(-1.0f)
-, badTargetCat(0)
-, onlyTargetCat(0)
-, mainDir(FwdVector)
-{
-}
-
 UnitDefWeapon::UnitDefWeapon(const WeaponDef* weaponDef) {
 	*this = UnitDefWeapon();
 	this->def = weaponDef;
@@ -67,8 +56,7 @@ UnitDefWeapon::UnitDefWeapon(const WeaponDef* weaponDef, const LuaTable& weaponT
 UnitDef::UnitDef()
 	: SolidObjectDef()
 	, cobID(-1)
-	, decoyDef(NULL)
-	, techLevel(-1)
+	, decoyDef(nullptr)
 	, metalUpkeep(0.0f)
 	, energyUpkeep(0.0f)
 	, metalMake(0.0f)
@@ -122,6 +110,7 @@ UnitDef::UnitDef()
 	, floatOnWater(false)
 	, pushResistant(false)
 	, strafeToAttack(false)
+	, stopToAttack(false)
 	, minCollisionSpeed(0.0f)
 	, slideTolerance(0.0f)
 	, maxHeightDif(0.0f)
@@ -136,13 +125,13 @@ UnitDef::UnitDef()
 	, flankingBonusMax(0.0f)
 	, flankingBonusMin(0.0f)
 	, flankingBonusMobilityAdd(0.0f)
-	, shieldWeaponDef(NULL)
-	, stockpileWeaponDef(NULL)
+	, shieldWeaponDef(nullptr)
+	, stockpileWeaponDef(nullptr)
 	, maxWeaponRange(0.0f)
 	, maxCoverage(0.0f)
-	, deathExpWeaponDef(NULL)
-	, selfdExpWeaponDef(NULL)
-	, buildPic(NULL)
+	, deathExpWeaponDef(nullptr)
+	, selfdExpWeaponDef(nullptr)
+	, buildPic(nullptr)
 	, selfDCountdown(0)
 	, builder(false)
 	, activateWhenBuilt(false)
@@ -203,7 +192,6 @@ UnitDef::UnitDef()
 	, transportCapacity(0)
 	, transportSize(0)
 	, minTransportSize(0)
-	, isAirBase(false)
 	, isFirePlatform(false)
 	, transportMass(0.0f)
 	, minTransportMass(0.0f)
@@ -220,7 +208,7 @@ UnitDef::UnitDef()
 	, decloakDistance(0.0f)
 	, decloakSpherical(false)
 	, decloakOnFire(false)
-	, cloakTimeout(0)
+
 	, kamikazeDist(0.0f)
 	, kamikazeUseLOS(false)
 	, targfac(false)
@@ -250,6 +238,14 @@ UnitDef::UnitDef()
 	, realEnergyUpkeep(0.0f)
 	, realBuildTime(0.0f)
 {
+	memset(&modelCEGTags[0], 0, sizeof(modelCEGTags));
+	memset(&pieceCEGTags[0], 0, sizeof(pieceCEGTags));
+	memset(&crashCEGTags[0], 0, sizeof(crashCEGTags));
+
+	// filled in later by UnitDrawer
+	modelExplGenIDs.fill(-1u); modelExplGenIDs[0] = 0;
+	pieceExplGenIDs.fill(-1u); pieceExplGenIDs[0] = 0;
+	crashExplGenIDs.fill(-1u); crashExplGenIDs[0] = 0;
 }
 
 
@@ -281,7 +277,11 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	makesMetal   = udTable.GetFloat("makesMetal", 0.0f);
 	energyMake   = udTable.GetFloat("energyMake", 0.0f);
 
-	health       = std::max(0.1f, udTable.GetFloat("maxDamage",  0.0f)); //avoid some nasty divide by 0
+	/* maxDamage is the legacy Total Annihilation spelling,
+	 * and what most games use, so not really deprecatable */
+	health       = udTable.GetFloat("health", udTable.GetFloat("maxDamage", 0.0f));
+	health       = std::max(0.1f, health); // avoid some nasty divide by 0
+
 	autoHeal     = udTable.GetFloat("autoHeal",      0.0f) * (UNIT_SLOWUPDATE_RATE / float(GAME_SPEED));
 	idleAutoHeal = udTable.GetFloat("idleAutoHeal", 10.0f) * (UNIT_SLOWUPDATE_RATE / float(GAME_SPEED));
 	idleTime     = udTable.GetInt("idleTime", 600);
@@ -383,7 +383,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	flankingBonusMobilityAdd = udTable.GetFloat("flankingBonusMobilityAdd", 0.01f);
 
 	armoredMultiple = udTable.GetFloat("damageModifier", 1.0f);
-	armorType = damageArrayHandler->GetTypeFromName(name);
+	armorType = damageArrayHandler.GetTypeFromName(name);
 
 	losHeight = udTable.GetFloat("losEmitHeight", 20.0f);
 	radarHeight = udTable.GetFloat("radarEmitHeight", losHeight);
@@ -412,7 +412,6 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	decloakDistance  = udTable.GetFloat("minCloakDistance", 0.0f);
 	decloakSpherical = udTable.GetBool("decloakSpherical", true);
 	decloakOnFire    = udTable.GetBool("decloakOnFire",    true);
-	cloakTimeout     = udTable.GetInt("cloakTimeout", 128);
 
 	highTrajectoryType = udTable.GetInt("highTrajectory", 0);
 
@@ -451,7 +450,6 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	minTransportSize  = udTable.GetInt("minTransportSize",   0);
 	transportCapacity = udTable.GetInt("transportCapacity",  0);
 	isFirePlatform    = udTable.GetBool("isFirePlatform",    false);
-	isAirBase         = udTable.GetBool("isAirBase",         false);
 	loadingRadius     = udTable.GetFloat("loadingRadius",    220.0f);
 	unloadSpread      = udTable.GetFloat("unloadSpread",     5.0f);
 	transportMass     = udTable.GetFloat("transportMass",    100000.0f);
@@ -490,10 +488,10 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	category = CCategoryHandler::Instance()->GetCategories(udTable.GetString("category", ""));
 	noChaseCategory = CCategoryHandler::Instance()->GetCategories(udTable.GetString("noChaseCategory", ""));
 
-	iconType = icon::iconHandler->GetIcon(udTable.GetString("iconType", "default"));
+	iconType = icon::iconHandler.GetIcon(udTable.GetString("iconType", "default"));
 
-	shieldWeaponDef    = NULL;
-	stockpileWeaponDef = NULL;
+	shieldWeaponDef    = nullptr;
+	stockpileWeaponDef = nullptr;
 
 	maxWeaponRange = 0.0f;
 	maxCoverage = 0.0f;
@@ -515,10 +513,9 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 			const std::string errMsg = "WARNING: Couldn't find a MoveClass named " + moveClass + " (used in UnitDef: " + unitName + ")";
 
 			// invalidate this unitDef; caught in ParseUnitDef
-			if ((moveDef = moveDefHandler->GetMoveDefByName(moveClass)) == nullptr)
+			if ((moveDef = moveDefHandler.GetMoveDefByName(moveClass)) == nullptr)
 				throw content_error(errMsg);
 
-			moveDef->udRefCount += 1;
 			this->pathType = moveDef->pathType;
 		}
 
@@ -540,8 +537,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 		}
 
 		if (seismicSignature == -1.0f) {
-			const bool isTank = (moveDef != NULL && moveDef->speedModClass == MoveDef::Tank);
-			const bool isKBot = (moveDef != NULL && moveDef->speedModClass == MoveDef::KBot);
+			const bool isTank = (moveDef != nullptr && moveDef->speedModClass == MoveDef::Tank);
+			const bool isKBot = (moveDef != nullptr && moveDef->speedModClass == MoveDef::KBot);
 
 			// seismic signatures only make sense for certain mobile ground units
 			if (isTank || isKBot) {
@@ -555,10 +552,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	if (IsAirUnit()) {
 		if (IsFighterAirUnit() || IsBomberAirUnit()) {
 			// double turn-radius for bombers if not set explicitly
-			if (IsBomberAirUnit() && turnRadius == 500.0f) {
-				turnRadius *= 2.0f;
-			}
-
+			turnRadius *= (1.0f + (IsBomberAirUnit() && turnRadius == 500.0f));
 			maxAcc = udTable.GetFloat("maxAcc", 0.065f); // engine power
 		}
 	}
@@ -569,17 +563,12 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 
 	deathExpWeaponDef = weaponDefHandler->GetWeaponDef(udTable.GetString("explodeAs", ""));
 	selfdExpWeaponDef = weaponDefHandler->GetWeaponDef(udTable.GetString("selfDestructAs", udTable.GetString("explodeAs", "")));
-	if (deathExpWeaponDef == NULL) {
-		deathExpWeaponDef = weaponDefHandler->GetWeaponDef("NOWEAPON");
-		if (deathExpWeaponDef == NULL) {
-			LOG_L(L_ERROR, "Couldn't find WeaponDef NOWEAPON and explodeAs for %s is missing!", unitName.c_str());
-		}
+
+	if (deathExpWeaponDef == nullptr && (deathExpWeaponDef = weaponDefHandler->GetWeaponDef("NOWEAPON")) == nullptr) {
+		LOG_L(L_ERROR, "Couldn't find WeaponDef NOWEAPON and explodeAs for %s is missing!", unitName.c_str());
 	}
-	if (selfdExpWeaponDef == NULL) {
-		selfdExpWeaponDef = weaponDefHandler->GetWeaponDef("NOWEAPON");
-		if (selfdExpWeaponDef == NULL) {
-			LOG_L(L_ERROR, "Couldn't find WeaponDef NOWEAPON and selfDestructAs for %s is missing!", unitName.c_str());
-		}
+	if (selfdExpWeaponDef == nullptr && (selfdExpWeaponDef = weaponDefHandler->GetWeaponDef("NOWEAPON")) == nullptr) {
+		LOG_L(L_ERROR, "Couldn't find WeaponDef NOWEAPON and selfDestructAs for %s is missing!", unitName.c_str());
 	}
 
 	power = udTable.GetFloat("power", (metal + (energy / 60.0f)));
@@ -598,9 +587,8 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	zsize = std::max(1 * SPRING_FOOTPRINT_SCALE, (udTable.GetInt("footprintZ", 1) * SPRING_FOOTPRINT_SCALE));
 
 	buildingMask = (std::uint16_t)udTable.GetInt("buildingMask", 1); //1st bit set to 1 constitutes for "normal building"
-	if (IsImmobileUnit()) {
+	if (IsImmobileUnit())
 		CreateYardMap(udTable.GetString("yardMap", ""));
-	}
 
 	decalDef.Parse(udTable);
 
@@ -616,6 +604,7 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	canLoopbackAttack = udTable.GetBool("canLoopbackAttack", false);
 	levelGround = udTable.GetBool("levelGround", true);
 	strafeToAttack = udTable.GetBool("strafeToAttack", false);
+	stopToAttack = udTable.GetBool("stopToAttack", false);
 
 
 	// initialize the (per-unitdef) collision-volume
@@ -623,83 +612,87 @@ UnitDef::UnitDef(const LuaTable& udTable, const std::string& unitName, int id)
 	ParseCollisionVolume(udTable);
 	ParseSelectionVolume(udTable);
 
+	{
+		const LuaTable& buildsTable = udTable.SubTable("buildOptions");
+		const LuaTable& paramsTable = udTable.SubTable("customParams");
 
-	const LuaTable& buildsTable = udTable.SubTable("buildOptions");
+		if (buildsTable.IsValid())
+			buildsTable.GetMap(buildOptions);
 
-	if (buildsTable.IsValid())
-		buildsTable.GetMap(buildOptions);
-
-
-	const LuaTable& sfxTable = udTable.SubTable("SFXTypes");
-	const LuaTable& modelCEGTable = sfxTable.SubTable(     "explosionGenerators");
-	const LuaTable& pieceCEGTable = sfxTable.SubTable("pieceExplosionGenerators");
-
-	std::vector<int> modelCEGKeys; modelCEGTable.GetKeys(modelCEGKeys);
-	std::vector<int> pieceCEGKeys; pieceCEGTable.GetKeys(pieceCEGKeys);
-
-	for (unsigned int n = 0; n < modelCEGKeys.size(); n++) {
-		modelCEGTags.push_back(modelCEGTable.GetString(modelCEGKeys[n], ""));
-	}
-	for (unsigned int n = 0; n < pieceCEGKeys.size(); n++) {
-		pieceCEGTags.push_back(pieceCEGTable.GetString(pieceCEGKeys[n], ""));
+		// custom parameters table
+		paramsTable.GetMap(customParams);
 	}
+	{
+		const LuaTable&      sfxTable =  udTable.SubTable("SFXTypes");
+		const LuaTable& modelCEGTable = sfxTable.SubTable(     "explosionGenerators");
+		const LuaTable& pieceCEGTable = sfxTable.SubTable("pieceExplosionGenerators");
+		const LuaTable& crashCEGTable = sfxTable.SubTable("crashExplosionGenerators");
 
-	// filled in later by UnitDrawer
-	modelExplGenIDs.resize(modelCEGTags.size(), -1u);
-	pieceExplGenIDs.resize(pieceCEGTags.size(), -1u);
+		std::vector<int> cegKeys;
+		std::array<const LuaTable*, 3> cegTbls = {&modelCEGTable, &pieceCEGTable, &crashCEGTable};
+		std::array<char[64], MAX_UNITDEF_EXPGEN_IDS>* cegTags[3] = {&modelCEGTags, &pieceCEGTags, &crashCEGTags};
 
-	// custom parameters table
-	udTable.SubTable("customParams").GetMap(customParams);
-}
+		for (int i = 0; i < 3; i++) {
+			auto& tagStrs = *cegTags[i];
 
+			cegKeys.clear();
+			cegKeys.reserve(tagStrs.size());
 
-UnitDef::~UnitDef()
-{
-	if (buildPic != nullptr) {
-		buildPic->Free();
-		spring::SafeDelete(buildPic);
+			cegTbls[i]->GetKeys(cegKeys);
+
+			// get at most N tags, discard the rest
+			for (unsigned int j = 0, k = 0; j < cegKeys.size() && k < tagStrs.size(); j++) {
+				const std::string& tag = cegTbls[i]->GetString(cegKeys[j], "");
+
+				if (tag.empty())
+					continue;
+
+				strncpy(tagStrs[k++], tag.c_str(), sizeof(tagStrs[0]));
+			}
+		}
 	}
 }
 
 
-
 void UnitDef::ParseWeaponsTable(const LuaTable& weaponsTable)
 {
 	const WeaponDef* noWeaponDef = weaponDefHandler->GetWeaponDef("NOWEAPON");
 
-	for (int w = 0; w < MAX_WEAPONS_PER_UNIT; w++) {
+	for (int k = 0, w = 0; w < MAX_WEAPONS_PER_UNIT; w++) {
 		LuaTable wTable;
-		string name = weaponsTable.GetString(w + 1, "");
+		std::string wdName = weaponsTable.GetString(w + 1, "");
 
-		if (name.empty()) {
+		if (wdName.empty()) {
 			wTable = weaponsTable.SubTable(w + 1);
-			name = wTable.GetString("name", "");
+			wdName = wTable.GetString("name", "");
 		}
 
-		const WeaponDef* wd = NULL;
-		if (!name.empty()) {
-			wd = weaponDefHandler->GetWeaponDef(name);
+		const WeaponDef* wd = nullptr;
+
+		if (!wdName.empty())
+			wd = weaponDefHandler->GetWeaponDef(wdName);
+
+		if (wd == nullptr) {
+			// allow any of the first three weapons to be null; these will be
+			// replaced by NoWeapon's if there is a valid WeaponDef among this
+			// set
+			if (w <= 3)
+				continue;
+
+			// otherwise stop trying
+			break;
 		}
 
-		if (wd == NULL) {
-			if (w <= 3) {
-				continue; // allow empty weapons among the first 3
-			} else {
+		while (k < w) {
+			if (noWeaponDef == nullptr) {
+				LOG_L(L_ERROR, "[%s] missing NOWEAPON for WeaponDef %s (#%d) of UnitDef %s", __func__, wdName.c_str(), w, humanName.c_str());
 				break;
 			}
+
+			weapons[k++] = {noWeaponDef};
 		}
 
-		while (weapons.size() < w) {
-			if (!noWeaponDef) {
-				LOG_L(L_ERROR, "Spring requires a NOWEAPON weapon type "
-						"to be present as a placeholder for missing weapons");
-				break;
-			} else {
-				weapons.push_back(UnitDefWeapon(noWeaponDef));
-			}
-		}
-
-		weapons.emplace_back(wd, wTable);
+		weapons[k++] = {wd, wTable};
 
 		maxWeaponRange = std::max(maxWeaponRange, wd->range);
 
@@ -707,24 +700,22 @@ void UnitDef::ParseWeaponsTable(const LuaTable& weaponsTable)
 			maxCoverage = wd->coverageRange;
 
 		if (wd->isShield) {
-			if (!shieldWeaponDef || // use the biggest shield
-			    (shieldWeaponDef->shieldRadius < wd->shieldRadius)) {
+			// use the biggest shield
+			if (shieldWeaponDef == nullptr || (shieldWeaponDef->shieldRadius < wd->shieldRadius))
 				shieldWeaponDef = wd;
-			}
 		}
 
 		if (wd->stockpile) {
 			// interceptors have priority
-			if (wd->interceptor || !stockpileWeaponDef || !stockpileWeaponDef->interceptor) {
+			if (wd->interceptor || stockpileWeaponDef == nullptr || !stockpileWeaponDef->interceptor)
 				stockpileWeaponDef = wd;
-			}
 		}
 	}
 }
 
 
 
-void UnitDef::CreateYardMap(std::string yardMapStr)
+void UnitDef::CreateYardMap(std::string&& yardMapStr)
 {
 	// if a unit is immobile but does *not* have a yardmap
 	// defined, assume it is not supposed to be a building
@@ -732,69 +723,67 @@ void UnitDef::CreateYardMap(std::string yardMapStr)
 	if (yardMapStr.empty())
 		return;
 
-	yardmap.resize(xsize * zsize);
+	const bool highResMap = (tolower(yardMapStr[0]) == 'h');
 
-	StringToLowerInPlace(yardMapStr);
+	// determine number of characters to parse from str
+	const unsigned int hxSize = xsize >> (1 - highResMap);
+	const unsigned int hzSize = zsize >> (1 - highResMap);
+	const unsigned int ymSize = hxSize * hzSize;
 
-	const bool highResMap = (yardMapStr[0] == 'h');
-
-	const unsigned int hxsize = xsize >> (1 - highResMap);
-	const unsigned int hzsize = zsize >> (1 - highResMap);
-
-	// if high-res yardmap, start at second character
+	// if high-res yardmap, start reading at second character
 	unsigned int ymReadIdx = highResMap;
 	unsigned int ymCopyIdx = 0;
 
-	std::vector<YardMapStatus> defYardMap(hxsize * hzsize, YARDMAP_BLOCKED);
-	std::string unknownChars;
+	std::array<YardMapStatus, 256 * 256> defYardMap;
+
+	if (ymSize > defYardMap.size()) {
+		LOG_L(L_WARNING, "[%s] %s: footprint{x=%u,z=%u} too large to create %s-res yardmap", __func__, name.c_str(), xsize, zsize, highResMap? "high": "low");
+		return;
+	}
+
+	yardmap.resize(xsize * zsize);
+	defYardMap.fill(YARDMAP_BLOCKED);
 
 	// read the yardmap from the LuaDef string
 	while (ymReadIdx < yardMapStr.size()) {
-		const unsigned char c = yardMapStr[ymReadIdx++];
+		const char c = tolower(yardMapStr[ymReadIdx++]);
 
 		if (isspace(c))
 			continue;
-
-		YardMapStatus ys = YARDMAP_BLOCKED;
+		// continue rather than break s.t. the excess-count can be shown
+		if ((ymCopyIdx++) >= ymSize)
+			continue;
 
 		switch (c) {
-			case 'g': ys = YARDMAP_GEO; needGeo = true; break;
-			case 'y': ys = YARDMAP_OPEN;    break;
-			case 'c': ys = YARDMAP_YARD;    break;
-			case 'i': ys = YARDMAP_YARDINV; break;
-			//case 'w': { ys = YARDMAP_WALKABLE; } break; //FIXME
+			case 'g': { defYardMap[ymCopyIdx - 1] = YARDMAP_GEO; needGeo = true; } break;
+			case 'y': { defYardMap[ymCopyIdx - 1] = YARDMAP_OPEN;                } break;
+			case 'c': { defYardMap[ymCopyIdx - 1] = YARDMAP_YARD;                } break;
+			case 'i': { defYardMap[ymCopyIdx - 1] = YARDMAP_YARDINV;             } break;
+		//	case 'w': { defYardMap[ymCopyIdx - 1] = YARDMAP_WALKABLE;            } break; // TODO?
 			case 'w':
 			case 'x':
 			case 'f':
-			case 'o': ys = YARDMAP_BLOCKED; break;
-			default:
-				unknownChars += (unknownChars.find_first_of(c) == std::string::npos);
-		}
-
-		if (ymCopyIdx < defYardMap.size()) {
-			defYardMap[ymCopyIdx++] = ys;
+			case 'o': { defYardMap[ymCopyIdx - 1] = YARDMAP_BLOCKED;             } break;
+			default : {                                                          } break;
 		}
 	}
 
 	// print warnings
-	if (ymCopyIdx > defYardMap.size())
-		LOG_L(L_WARNING, "%s: Given yardmap contains " _STPF_ " excess char(s)!", name.c_str(), ymCopyIdx - defYardMap.size());
+	if (ymCopyIdx > ymSize)
+		LOG_L(L_WARNING, "[%s] %s: given yardmap contains %u excess char(s)!", __func__, name.c_str(), ymCopyIdx - ymSize);
 
-	if (ymCopyIdx > 0 && ymCopyIdx < defYardMap.size())
-		LOG_L(L_WARNING, "%s: Given yardmap requires " _STPF_ " extra char(s)!", name.c_str(), defYardMap.size() - ymCopyIdx);
-
-	if (!unknownChars.empty())
-		LOG_L(L_WARNING, "%s: Given yardmap contains unknown char(s) \"%s\"!", name.c_str(), unknownChars.c_str());
+	if (ymCopyIdx > 0 && ymCopyIdx < ymSize)
+		LOG_L(L_WARNING, "[%s] %s: given yardmap requires %u extra char(s)!", __func__, name.c_str(), ymSize - ymCopyIdx);
 
 	// write the final yardmap at blocking-map resolution
 	// (in case of a high-res map this becomes a 1:1 copy,
-	// otherwise the given yardmap will be upsampled)
+	// otherwise the given yardmap will be upsampled 2:1)
 	for (unsigned int bmz = 0; bmz < zsize; bmz++) {
 		for (unsigned int bmx = 0; bmx < xsize; bmx++) {
-			const unsigned int yardMapIdx = (bmx >> (1 - highResMap)) + ((bmz >> (1 - highResMap)) * hxsize);
-			const YardMapStatus yardMapChar = defYardMap[yardMapIdx];
+			const unsigned int ymx = bmx >> (1 - highResMap);
+			const unsigned int ymz = bmz >> (1 - highResMap);
 
-			yardmap[bmx + bmz * xsize] = yardMapChar;
+			yardmap[bmx + bmz * xsize] = defYardMap[ymx + ymz * hxSize];
 		}
 	}
 }
@@ -825,12 +814,9 @@ void UnitDef::SetNoCost(bool noCost)
 	}
 }
 
-bool UnitDef::HasBomberWeapon() const {
-	if (weapons.empty())
-		return false;
-	if (weapons[0].def == nullptr)
-		return false;
-
-	return (weapons[0].def->IsAircraftWeapon());
+bool UnitDef::HasBomberWeapon(unsigned int idx) const {
+	// checked by Is*AirUnit
+	assert(HasWeapon(idx));
+	return (weapons[idx].def->IsAircraftWeapon());
 }
 
