@@ -2,15 +2,15 @@
 
 #include "LaserCannon.h"
 #include "WeaponDef.h"
-#include "WeaponMemPool.h"
 #include "Map/Ground.h"
+#include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Projectiles/WeaponProjectiles/WeaponProjectileFactory.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 
-CR_BIND_DERIVED_POOL(CLaserCannon, CWeapon, , weaponMemPool.alloc, weaponMemPool.free)
-CR_REG_METADATA(CLaserCannon,(
+CR_BIND_DERIVED(CLaserCannon, CWeapon, )
+CR_REG_METADATA(CLaserCannon, (
 	CR_MEMBER(color)
 ))
 
@@ -22,6 +22,11 @@ CLaserCannon::CLaserCannon(CUnit* owner, const WeaponDef* def): CWeapon(owner, d
 }
 
 
+void CLaserCannon::UpdateProjectileSpeed(const float val)
+{
+	projectileSpeed = std::max(0.001f, val); // sanitize
+	range = std::max(1.0f, std::floor(val / projectileSpeed)) * projectileSpeed;
+}
 void CLaserCannon::UpdateRange(const float val)
 {
 	// round range *DOWN* to integer multiple of projectile speed
@@ -29,7 +34,7 @@ void CLaserCannon::UpdateRange(const float val)
 	// (val / speed) is the total number of frames the projectile
 	// is allowed to do damage to objects, ttl decreases from N-1
 	// to 0 and collisions are checked at 0 inclusive
-	range = std::max(1.0f, std::floor(val / weaponDef->projectilespeed)) * weaponDef->projectilespeed;
+	range = std::max(1.0f, std::floor(val / projectileSpeed)) * projectileSpeed;
 }
 
 
@@ -38,13 +43,12 @@ void CLaserCannon::FireImpl(const bool scriptCall)
 	float3 dir = currentTargetPos - weaponMuzzlePos;
 
 	const float dist = dir.LengthNormalize();
-	const int ttlreq = std::ceil(dist / weaponDef->projectilespeed);
-	const int ttlmax = std::floor(range / weaponDef->projectilespeed) - 1;
+	const int ttlreq = std::ceil(dist / projectileSpeed);
+	const int ttlmax = std::floor(range / projectileSpeed) - 1;
 
-	if (onlyForward && owner->unitDef->IsStrafingAirUnit()) {
-		// [?] StrafeAirMovetype cannot align itself properly, change back when that is fixed
+	// [?] StrafeAirMovetype cannot align itself properly, change back when that is fixed
+	if (onlyForward && owner->unitDef->IsStrafingAirUnit())
 		dir = owner->frontdir;
-	}
 
 	dir += (gsRNG.NextVector() * SprayAngleExperience() + SalvoErrorExperience());
 	dir.Normalize();
