@@ -9,12 +9,14 @@ class CMissileLauncher: public CWeapon
 {
 	CR_DECLARE_DERIVED(CMissileLauncher)
 public:
-	CMissileLauncher(CUnit* owner, const WeaponDef* def);
+	CMissileLauncher(CUnit* owner = nullptr, const WeaponDef* def = nullptr): CWeapon(owner, def) {}
 
 	void UpdateWantedDir() override final;
 
 private:
-	bool HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg, bool useMuzzle = false) const override final;
+	const float3& GetAimFromPos(bool useMuzzle = false) const override { return weaponMuzzlePos; }
+
+	bool HaveFreeLineOfFire(const float3 srcPos, const float3 tgtPos, const SWeaponTarget& trg) const override final;
 	void FireImpl(const bool scriptCall) override final;
 };
 
