@@ -2,19 +2,15 @@
 
 #include "Rifle.h"
 #include "WeaponDef.h"
-#include "WeaponMemPool.h"
 #include "Game/TraceRay.h"
 #include "Game/GameHelper.h"
 #include "Map/Ground.h"
-#include "Rendering/Env/Particles/Classes/HeatCloudProjectile.h"
-#include "Rendering/Env/Particles/Classes/SmokeProjectile.h"
-#include "Rendering/Env/Particles/Classes/TracerProjectile.h"
+#include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Features/Feature.h"
-#include "System/Sync/SyncTracer.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 
-CR_BIND_DERIVED_POOL(CRifle, CWeapon, , weaponMemPool.alloc, weaponMemPool.free)
+CR_BIND_DERIVED(CRifle, CWeapon, )
 CR_REG_METADATA(CRifle, )
 
 //////////////////////////////////////////////////////////////////////
@@ -32,17 +28,11 @@ void CRifle::FireImpl(const bool scriptCall)
 	CUnit* hitUnit;
 	CFeature* hitFeature;
 
-	const float length = TraceRay::TraceRay(weaponMuzzlePos, dir, range, 0, owner, hitUnit, hitFeature);
+	TraceRay::TraceRay(weaponMuzzlePos, dir, range, collisionFlags, owner, hitUnit, hitFeature);
 	const float impulse = CGameHelper::CalcImpulseScale(*damages, 1.0f);
 
-	if (hitUnit != nullptr) {
+	if (hitUnit != nullptr)
 		hitUnit->DoDamage(*damages, dir * impulse, owner, weaponDef->id, -1);
-		projMemPool.alloc<CHeatCloudProjectile>(owner, weaponMuzzlePos + dir * length, hitUnit->speed * 0.9f, 30, 1);
-	} else if (hitFeature != nullptr) {
+	else if (hitFeature != nullptr)
 		hitFeature->DoDamage(*damages, dir * impulse, owner, weaponDef->id, -1);
-		projMemPool.alloc<CHeatCloudProjectile>(owner, weaponMuzzlePos + dir * length, hitFeature->speed * 0.9f, 30, 1);
-	}
-
-	projMemPool.alloc<CTracerProjectile>(owner, weaponMuzzlePos, dir * projectileSpeed, length);
-	projMemPool.alloc<CSmokeProjectile>(owner, weaponMuzzlePos, ZeroVector, 70, 0.1f, 0.02f, 0.6f);
 }
