@@ -9,11 +9,11 @@ class CRifle: public CWeapon
 {
 	CR_DECLARE_DERIVED(CRifle)
 public:
-	CRifle(CUnit* owner, const WeaponDef* def);
+	CRifle(CUnit* owner = nullptr, const WeaponDef* def = nullptr): CWeapon(owner, def) {}
 
 private:
 	void FireImpl(const bool scriptCall) override final;
-	float GetPredictedImpactTime(float3 p) const override final;
+	float GetPredictedImpactTime(float3 p) const override final { return 0.0f; }
 };
 
 #endif /* RIFLE_H */
