@@ -24,12 +24,12 @@ struct SWeaponTarget {
 	SWeaponTarget();
 
 	// explicit ctors
-	SWeaponTarget(const CUnit* unit, bool isUserTarget = false);
-	SWeaponTarget(float3 groundPos, bool isUserTarget = false);
-	SWeaponTarget(CWeaponProjectile* intercept, bool isUserTarget = false);
+	SWeaponTarget(const CUnit* unit, bool userTarget = false, bool autoTarget = false);
+	SWeaponTarget(float3 groundPos, bool userTarget = false, bool autoTarget = false);
+	SWeaponTarget(CWeaponProjectile* intercept, bool userTarget = false, bool autoTarget = false);
 
 	// conditional ctors
-	SWeaponTarget(const CUnit* unit, float3 groundPos, bool isUserTarget = false);
+	SWeaponTarget(const CUnit* unit, float3 groundPos, bool userTarget = false, bool autoTarget = false);
 
 	// operators
 	bool operator!=(const SWeaponTarget& other) const;
@@ -37,11 +37,14 @@ struct SWeaponTarget {
 
 public:
 	TargetType type;    // indicates if we have a target and what type
-	bool isUserTarget;
-	bool isManualFire;
+
+	bool isUserTarget = false;
+	bool isAutoTarget = false;
+	bool isManualFire = false;
 
 	CUnit* unit;                  // if targettype=unit: the targeted unit
 	CWeaponProjectile* intercept; // if targettype=intercept: projectile that we currently target for interception
+
 	float3 groundPos;             // if targettype=ground: the ground position
 };
 
