@@ -5,9 +5,12 @@
 
 #include <string>
 #include <vector>
-#include <map>
 #include <cinttypes>
 
+#include "ArchiveTypes.h"
+#include "System/Sync/SHA512.hpp"
+#include "System/UnorderedMap.hpp"
+
 /**
  * @brief Abstraction of different archive types
  *
@@ -19,13 +22,16 @@
 class IArchive
 {
 protected:
-	IArchive(const std::string& archiveName);
+	IArchive(const std::string& archiveFile): archiveFile(archiveFile) {
+	}
 
 public:
 	virtual ~IArchive() {}
 
+	virtual int GetType() const = 0;
+
 	virtual bool IsOpen() = 0;
-	const std::string& GetArchiveName() const { return archiveFile; }
+	const std::string& GetArchiveFile() const { return archiveFile; }
 
 	/**
 	 * @return The amount of files in the archive, does not change during
@@ -116,16 +122,18 @@ public:
 	 */
 	virtual bool CheckForSolid() const { return false; }
 	/**
-	 * Fetches the CRC32 hash of a file by its ID.
+	 * Fetches the (SHA512) hash of a file by its ID.
 	 */
-	virtual unsigned int GetCrc32(unsigned int fid);
+	virtual bool CalcHash(uint32_t fid, uint8_t hash[sha512::SHA_LEN], std::vector<std::uint8_t>& fb);
 
 
 protected:
-	/// must be populated by the subclass
-	std::map<std::string, unsigned int> lcNameIndex;
+	// Spring expects the contents of archives to be case-independent
+	// this map (which must be populated by subclass archives) is kept
+	// to allow converting back from lowercase to original case
+	spring::unordered_map<std::string, unsigned int> lcNameIndex;
 
-private:
+protected:
 	/// "ExampleArchive.sdd"
 	const std::string archiveFile;
 };
