@@ -3,7 +3,7 @@
 #ifndef _ZIP_ARCHIVE_H
 #define _ZIP_ARCHIVE_H
 
-#include "ArchiveFactory.h"
+#include "IArchiveFactory.h"
 #include "BufferedArchive.h"
 #include "minizip/unzip.h"
 
@@ -17,9 +17,10 @@
  */
 class CZipArchiveFactory : public IArchiveFactory {
 public:
-	CZipArchiveFactory();
+	CZipArchiveFactory(): IArchiveFactory("sdz") {}
+
 private:
-	IArchive* DoCreateArchive(const std::string& filePath) const;
+	IArchive* DoCreateArchive(const std::string& filePath) const override;
 };
 
 
@@ -32,24 +33,34 @@ public:
 	CZipArchive(const std::string& archiveName);
 	virtual ~CZipArchive();
 
-	virtual bool IsOpen();
+	int GetType() const override { return ARCHIVE_TYPE_SDZ; }
 
-	virtual unsigned int NumFiles() const;
-	virtual void FileInfo(unsigned int fid, std::string& name, int& size) const;
-	virtual unsigned int GetCrc32(unsigned int fid);
+	bool IsOpen() override { return (zip != nullptr); }
+
+	unsigned int NumFiles() const override { return (fileEntries.size()); }
+	void FileInfo(unsigned int fid, std::string& name, int& size) const override;
+
+	#if 0
+	unsigned int GetCrc32(unsigned int fid) {
+		assert(IsFileId(fid));
+		return fileEntries[fid].crc;
+	}
+	#endif
 
 protected:
 	unzFile zip;
 
-	struct FileData {
+	// actual data is in BufferedArchive
+	struct FileEntry {
 		unz_file_pos fp;
 		int size;
 		std::string origName;
 		unsigned int crc;
 	};
-	std::vector<FileData> fileData;
 
-	virtual bool GetFileImpl(unsigned int fid, std::vector<std::uint8_t>& buffer);
+	std::vector<FileEntry> fileEntries;
+
+	int GetFileImpl(unsigned int fid, std::vector<std::uint8_t>& buffer) override;
 };
 
 #endif // _ZIP_ARCHIVE_H
