@@ -1,12 +1,12 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "LogUtil.h"
-
 #include "Level.h"
+
 #include <stddef.h> // for NULL
 
-const char* log_util_levelToString(int level) {
-
+const char* log_util_levelToString(int level)
+{
 	switch (level) {
 		case LOG_LEVEL_DEBUG:   return "Debug";
 		case LOG_LEVEL_INFO:    return "Info";
@@ -18,8 +18,8 @@ const char* log_util_levelToString(int level) {
 	}
 }
 
-char log_util_levelToChar(int level) {
-
+char log_util_levelToChar(int level)
+{
 	switch (level) {
 		case LOG_LEVEL_DEBUG:   return 'D';
 		case LOG_LEVEL_INFO:    return 'I';
@@ -31,8 +31,8 @@ char log_util_levelToChar(int level) {
 	}
 }
 
-int log_util_getNearestLevel(int level) {
-
+int log_util_getNearestLevel(int level)
+{
 	if (level >= LOG_LEVEL_FATAL)   return LOG_LEVEL_FATAL;
 	if (level >= LOG_LEVEL_ERROR)   return LOG_LEVEL_ERROR;
 	if (level >= LOG_LEVEL_WARNING) return LOG_LEVEL_WARNING;
@@ -45,9 +45,8 @@ int log_util_getNearestLevel(int level) {
 const char* log_util_prepareSection(const char* section) {
 
 	// make sure we always have a string for printing
-	if (section == NULL) {
+	if (section == NULL)
 		section = "<default>";
-	}
 
 	return section;
 }
