@@ -17,7 +17,6 @@ namespace netcode
 class CConnection
 {
 public:
-	CConnection();
 	virtual ~CConnection() {}
 
 	/**
@@ -63,6 +62,7 @@ public:
 	virtual bool NeedsReconnect() = 0;
 
 	unsigned int GetDataReceived() const { return dataRecv; }
+	unsigned int GetNumQueuedPings() const { return numPings; }
 	virtual unsigned int GetPacketQueueSize() const { return 0; }
 
 	virtual std::string Statistics() const = 0;
@@ -78,8 +78,9 @@ public:
 	virtual void Update() {}
 
 protected:
-	unsigned dataSent;
-	unsigned dataRecv;
+	unsigned int dataSent = 0;
+	unsigned int dataRecv = 0;
+	unsigned int numPings = 0;
 };
 
 } // namespace netcode
