@@ -2,25 +2,23 @@
 
 #include "Socket.h"
 
-#include <boost/system/error_code.hpp>
 #include "lib/streflop/streflop_cond.h"
 
 #include "System/Log/ILog.h"
-#include "System/Util.h"
+#include "System/StringUtil.h"
 
 
 namespace netcode
 {
-using namespace boost::system::errc;
 
-boost::asio::io_service netservice;
+asio::io_service netservice;
 
-bool CheckErrorCode(boost::system::error_code& err)
+bool CheckErrorCode(asio::error_code& err)
 {
 	// connection reset can happen when host did not start up
 	// before the client wants to connect
-	if (!err || err.value() == connection_reset || 
-		err.value() == resource_unavailable_try_again) { // this should only ever happen with async sockets, but testing indicates it happens anyway...
+	if (!err || err.value() == asio::error::connection_reset ||
+		err.value() == asio::error::try_again) { // this should only ever happen with async sockets, but testing indicates it happens anyway...
 		return false;
 	} else {
 		LOG_L(L_WARNING, "Network error %i: %s", err.value(),
@@ -29,16 +27,16 @@ bool CheckErrorCode(boost::system::error_code& err)
 	}
 }
 
-boost::asio::ip::udp::endpoint ResolveAddr(const std::string& host, int port, boost::system::error_code* err)
+asio::ip::udp::endpoint ResolveAddr(const std::string& host, int port, asio::error_code* err)
 {
 	assert(err);
-	using namespace boost::asio;
+	using namespace asio;
 	ip::address tempAddr = WrapIP(host, err);
 	if (!*err)
 		return ip::udp::endpoint(tempAddr, port);
 
 	auto errBuf = *err; // WrapResolve() might clear err
-	boost::asio::io_service io_service;
+	asio::io_service io_service;
 	ip::udp::resolver resolver(io_service);
 	ip::udp::resolver::query query(host, IntToString(port));
 	auto iter = WrapResolve(resolver, query, err);
@@ -52,54 +50,50 @@ boost::asio::ip::udp::endpoint ResolveAddr(const std::string& host, int port, bo
 }
 
 
-boost::asio::ip::address WrapIP(const std::string& ip,
-		boost::system::error_code* err)
+asio::ip::address WrapIP(const std::string& ip,
+		asio::error_code* err)
 {
-	boost::asio::ip::address addr;
+	asio::ip::address addr;
 
 	if (err == NULL) {
-		addr = boost::asio::ip::address::from_string(ip);
+		addr = asio::ip::address::from_string(ip);
 	} else {
-		addr = boost::asio::ip::address::from_string(ip, *err);
+		addr = asio::ip::address::from_string(ip, *err);
 	}
-//#ifdef STREFLOP_H
+
 	// (date of note: 08/05/10)
 	// something in from_string() is invalidating the FPU flags
 	// tested on win2k and linux (not happening there)
 	streflop::streflop_init<streflop::Simple>();
-//#endif
-
 	return addr;
 }
 
-boost::asio::ip::udp::resolver::iterator WrapResolve(
-		boost::asio::ip::udp::resolver& resolver,
-		boost::asio::ip::udp::resolver::query& query,
-		boost::system::error_code* err)
+asio::ip::udp::resolver::iterator WrapResolve(
+		asio::ip::udp::resolver& resolver,
+		asio::ip::udp::resolver::query& query,
+		asio::error_code* err)
 {
-	boost::asio::ip::udp::resolver::iterator resolveIt;
+	asio::ip::udp::resolver::iterator resolveIt;
 
 	if (err == NULL) {
 		resolveIt = resolver.resolve(query);
 	} else {
 		resolveIt = resolver.resolve(query, *err);
 	}
-//#ifdef STREFLOP_H
+
 	// (date of note: 08/22/10)
 	// something in resolve() is invalidating the FPU flags
 	streflop::streflop_init<streflop::Simple>();
-//#endif
-
 	return resolveIt;
 }
 
 
-boost::asio::ip::address GetAnyAddress(const bool IPv6)
+asio::ip::address GetAnyAddress(const bool IPv6)
 {
 	if (IPv6) {
-		return boost::asio::ip::address_v6::any();
+		return asio::ip::address_v6::any();
 	}
-	return boost::asio::ip::address_v4::any();
+	return asio::ip::address_v4::any();
 }
 
 
