@@ -1,63 +1,71 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#include "Misc.h"
+#if defined(__linux__)
+	#include <arpa/inet.h>
+	#include <ifaddrs.h>
+	#include <net/if.h>
+	#include <netinet/in.h>
+	#include <netpacket/packet.h>
 
-#ifdef __linux__
-#include <unistd.h>
-#include <dlfcn.h> // for dladdr(), dlopen()
-#include <sys/statvfs.h>
+	#include <sys/ioctl.h>
+	#include <sys/socket.h>
 
-#elif WIN32
-#include <io.h>
-#include <direct.h>
-#include <process.h>
-#include <shlobj.h>
-#include <shlwapi.h>
+#elif defined(_WIN32)
+	#include <io.h>
+	#include <direct.h>
+	#include <process.h>
+	#include <shlobj.h>
+	#include <shlwapi.h>
+	#include <iphlpapi.h>
 
-#ifndef SHGFP_TYPE_CURRENT
-	#define SHGFP_TYPE_CURRENT 0
-#endif
-#include "System/Platform/Win/WinVersion.h"
+	#ifndef SHGFP_TYPE_CURRENT
+		#define SHGFP_TYPE_CURRENT 0
+	#endif
 
-#elif __APPLE__
-#include <unistd.h>
-#include <mach-o/dyld.h>
-#include <stdlib.h>
-#include <dlfcn.h> // for dladdr(), dlopen()
-#include <climits> // for PATH_MAX
-#include <sys/statvfs.h>
+#elif defined(__APPLE__)
+	#include <cstdlib>
+	#include <climits> // for PATH_MAX
 
-#elif defined __FreeBSD__
-#include <unistd.h>
-#include <dlfcn.h> // for dladdr(), dlopen()
-#include <sys/types.h>
-#include <sys/sysctl.h>
+	#include <mach-o/dyld.h>
+
+#elif defined( __FreeBSD__)
+	#include <sys/sysctl.h>
 
 #else
 
 #endif
 
 
-#if !defined(WIN32)
-#include <sys/utsname.h> // for uname()
-#include <sys/types.h> // for getpw
-#include <pwd.h> // for getpw
 
-#include <fstream>
+#if !defined(_WIN32)
+#include <dlfcn.h> // for dladdr(), dlopen()
+#include <pwd.h> // for getpw*()
+#include <sys/statvfs.h>
+#include <sys/types.h>
+#include <sys/utsname.h> // for uname()
+#include <unistd.h>
 #endif
 
 
+
 #include <cstring>
 #include <cerrno>
-#include <deque>
+#include <fstream>
+#include <vector>
 
+#include "Misc.h"
+#include "System/CRC.h"
 #include "System/StringUtil.h"
-#include "System/SafeCStrings.h"
 #include "System/Log/ILog.h"
 #include "System/FileSystem/FileSystem.h"
+#if defined(_WIN32)
+#include "System/Platform/Win/WinVersion.h"
+#endif
+#include "System/Sync/SHA512.hpp"
 
 
-#if       defined WIN32
+
+#if       defined _WIN32
 /**
  * Returns a handle to the currently loaded module.
  * Note: requires at least Windows 2000
@@ -89,12 +97,10 @@ static HMODULE GetCurrentModule()
 static std::string GetUserDirFromEnvVar()
 {
 	#ifdef _WIN32
-	#define HOME_ENV_VAR "LOCALAPPDATA"
+	const char* home = getenv("LOCALAPPDATA");
 	#else
-	#define HOME_ENV_VAR "HOME"
+	const char* home = getenv("HOME");
 	#endif
-	const char* home = getenv(HOME_ENV_VAR);
-	#undef HOME_ENV_VAR
 
 	return (home == nullptr) ? "" : home;
 }
@@ -141,21 +147,14 @@ namespace Platform
 		return userDir;
 	}
 
-	#ifndef WIN32
-	static std::string GetRealPath(const std::string& path) {
+	#ifndef _WIN32
+	static std::string GetRealPath(const std::string& path)
+	{
+		char realPath[65536] = {0};
+		char* realPathPtr = realpath(path.c_str(), &realPath[0]);
 
-		std::string realPath = path;
-
-		// using NULL here is not supported in very old systems,
-		// but should be no problem for spring
-		// see for older systems:
-		// http://stackoverflow.com/questions/4109638/what-is-the-safe-alternative-to-realpath
-		char* realPathC = realpath(path.c_str(), nullptr);
-
-		if (realPathC != nullptr) {
-			realPath.assign(realPathC);
-			free(realPathC);
-		}
+		if (realPathPtr == nullptr)
+			memcpy(realPath, path.c_str(), std::min(path.size(), sizeof(realPath)));
 
 		if (FileSystem::GetDirectory(realPath).empty())
 			return (GetProcessExecutablePath() + realPath);
@@ -172,47 +171,45 @@ namespace Platform
 	// Windows:         GetModuleFileName() with hModule = NULL
 	std::string GetProcessExecutableFile()
 	{
-		std::string procExeFilePath;
-
+		const char* procExeFilePath = "";
 		// error will only be used if procExeFilePath stays empty
 		const char* error = nullptr;
 
-	#ifdef __linux__
+		#if defined(__linux__)
 		char file[512];
 		const int ret = readlink("/proc/self/exe", file, sizeof(file) - 1);
 
 		if (ret >= 0) {
 			file[ret] = '\0';
-			procExeFilePath = std::string(file);
+			procExeFilePath = file;
 		} else {
 			error = "[linux] failed to read /proc/self/exe";
 		}
 
 
-	#elif WIN32
+		#elif defined(_WIN32)
 		TCHAR procExeFile[MAX_PATH + 1];
 
-		// with NULL, it will return the handle
-		// for the main executable of the process
+		// main process executable handle
 		const HMODULE hModule = GetModuleHandle(nullptr);
 		const int ret = ::GetModuleFileName(hModule, procExeFile, sizeof(procExeFile));
 
 		if ((ret != 0) && (ret != sizeof(procExeFile))) {
-			procExeFilePath = std::string(procExeFile);
+			procExeFilePath = procExeFile;
 		} else {
 			error = "[win32] unknown";
 		}
 
 
-	#elif __APPLE__
+		#elif defined(__APPLE__)
 		uint32_t pathlen = PATH_MAX;
 		char path[PATH_MAX];
 
 		if (_NSGetExecutablePath(path, &pathlen) == 0)
-			procExeFilePath = GetRealPath(path);
+			procExeFilePath = path;
 
 
-	#elif defined(__FreeBSD__)
+		#elif defined(__FreeBSD__)
 		const int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
 		const long maxpath = pathconf("/", _PC_PATH_MAX);
 
@@ -223,14 +220,18 @@ namespace Platform
 			procExeFilePath = buf;
 
 
-	#else
+		#else
 		#error implement this
-	#endif
+		#endif
 
-		if (procExeFilePath.empty())
+		if (procExeFilePath[0] == 0)
 			LOG_L(L_WARNING, "[%s] could not get process executable file path, reason: %s", __func__, error);
 
+		#if defined(__APPLE__)
+		return GetRealPath(procExeFilePath);
+		#else
 		return procExeFilePath;
+		#endif
 	}
 
 	std::string GetProcessExecutablePath()
@@ -295,7 +296,7 @@ namespace Platform
 			error = "Not loaded";
 		}
 
-	#elif WIN32
+	#elif _WIN32
 		HMODULE hModule = nullptr;
 
 		if (moduleName.empty()) {
@@ -339,38 +340,49 @@ namespace Platform
 
 
 
-	std::string GetOS()
+	std::string GetOSNameStr()
 	{
-	#if defined(WIN32)
-		return ("Microsoft Windows\n" + GetOSDisplayString() + "\n" + GetHardwareInfoString());
-	#else
+		#if defined(_WIN32)
+		// "Windows Vista"
+		return (windows::GetDisplayString(true, false, false));
+		#else
+		// "Linux 3.16.0-45-generic"
+		// "Darwin 17.7.0"
 		struct utsname info;
 		if (uname(&info) == 0)
-			return (std::string(info.sysname) + " " + info.release + " " + info.version + " " + info.machine);
-
-		#if defined(__linux__)
-		return "Linux";
-		#elif defined(__FreeBSD__)
-		return "FreeBSD";
-		#elif defined(__APPLE__)
-		return "Mac OS X";
-		#else
-		#warning improve this
-		return "unknown OS";
+			return (std::string(info.sysname) + " " + info.release);
 		#endif
-	#endif
+
+		return {};
 	}
 
-	std::string GetOSFamily()
+	std::string GetOSVersionStr()
 	{
-		#if defined(WIN32)
+		#if defined(_WIN32)
+		// "Ultimate Edition, 32-bit Service Pack 1 (build 7601)"
+		return (windows::GetDisplayString(false, true, true));
+		#else
+		struct utsname info;
+		// "#60~14.04.1-Ubuntu SMP Fri Jul 24 21:16:23 UTC 2015 (x86_64)"
+		// "Darwin Kernel Version 17.7.0: Wed Sep 19 21:20:59 PDT 2018; root:xnu-4570.71.8~4/RELEASE_X86_64 (x86_64)"
+		if (uname(&info) == 0)
+			return (std::string(info.version) + " (" + info.machine + ")");
+
+		return {};
+		#endif
+	}
+
+	std::string GetOSDisplayStr() { return (GetOSNameStr() + " " + GetOSVersionStr()); }
+	std::string GetOSFamilyStr()
+	{
+		#if defined(_WIN32)
 		return "Windows";
 		#elif defined(__linux__)
 		return "Linux";
 		#elif defined(__FreeBSD__)
 		return "FreeBSD";
 		#elif defined(__APPLE__)
-		return "MacOSX";
+		return "MacOS";
 		#else
 		return "Unknown";
 		#endif
@@ -381,12 +393,105 @@ namespace Platform
 		if (Is64Bit())
 			return "64-bit (native)";
 
-		return (std::string("32-bit ") + (Is32BitEmulation()? "emulated": "native"));
+		return (std::string("32-bit ") + (Is32BitEmulation()? "(emulated)": "(native)"));
+	}
+
+	std::string GetPlatformStr()
+	{
+		return (Platform::GetOSFamilyStr() + " " + Platform::GetWordSizeStr());
+	}
+
+	#if (defined(_WIN32))
+	std::string GetHardwareStr() { return (windows::GetHardwareString()); }
+	#else
+	std::string GetHardwareStr() {
+		std::string ret;
+
+		FILE* cpuInfo = fopen("/proc/cpuinfo", "r");
+		FILE* memInfo = fopen("/proc/meminfo", "r");
+
+		char buf[1024];
+		char tmp[1024];
+
+		if (cpuInfo != nullptr) {
+			while (fgets(buf, sizeof(buf), cpuInfo) != nullptr) {
+				if (strstr(buf, "model name") != nullptr) {
+					const char* s = strstr(buf, ": ") + 2;
+					const char* e = strstr(s, "\n");
+
+					memset(tmp, 0, sizeof(tmp));
+					memcpy(tmp, s, e - s);
+
+					ret += (std::string(tmp) + "; ");
+					break;
+				}
+			}
+
+			fclose(cpuInfo);
+		}
+
+		if (memInfo != nullptr) {
+			while (fgets(buf, sizeof(buf), memInfo) != nullptr) {
+				if (strstr(buf, "MemTotal") != nullptr) {
+					const char* s = strstr(buf, ": ") + 2;
+					const char* e = s;
+
+					for (     ; !std::isdigit(*s); s++) {}
+					for (e = s;  std::isdigit(*e); e++) {}
+
+					memset(tmp, 0, sizeof(tmp));
+					memcpy(tmp, s, e - s);
+
+					// sufficient up to 4TB
+					uint32_t kb = 0;
+
+					sscanf(tmp, "%u", &kb);
+					sprintf(tmp, "%u", kb / 1024);
+
+					ret += (std::string(tmp) + "MB RAM");
+					break;
+				}
+			}
+
+			fclose(memInfo);
+		}
+
+		return ret;
+	}
+	#endif
+
+
+	std::string GetSysInfoHash() {
+		std::vector<uint8_t> sysInfo;
+
+		sha512::raw_digest rawHash;
+		sha512::hex_digest hexHash;
+
+		const std::string& oss = GetOSDisplayStr();
+		const std::string& hws = GetHardwareStr();
+		const std::string& wss = GetWordSizeStr();
+
+		sysInfo.clear();
+		sysInfo.resize((oss.size() + 1) + (hws.size() + 1) + (wss.size() + 1), 0);
+
+		std::snprintf(reinterpret_cast<char*>(sysInfo.data()), sysInfo.size(), "%s\n%s\n%s\n", oss.data(), hws.data(), wss.data());
+
+		sha512::calc_digest(sysInfo, rawHash);
+		sha512::dump_digest(rawHash, hexHash);
+
+		return {hexHash.data(), hexHash.data() + hexHash.size()};
+	}
+
+	std::string GetMacAddrHash() {
+		const std::array<uint8_t, 6>& rawAddr = GetRawMacAddr();
+		const std::string& hashStr = IntToString(CRC::CalcDigest(rawAddr.data(), rawAddr.size()), "%u");
+		return hashStr;
 	}
 
 
+
 	uint64_t FreeDiskSpace(const std::string& path) {
-		#ifdef WIN32
+		#ifdef _WIN32
 		ULARGE_INTEGER bytesFree;
 
 		if (!GetDiskFreeSpaceEx(path.c_str(), &bytesFree, nullptr, nullptr))
@@ -411,16 +516,11 @@ namespace Platform
 
 	uint32_t NativeWordSize() { return (sizeof(void*)); }
 	uint32_t SystemWordSize() { return ((Is32BitEmulation())? 8: NativeWordSize()); }
-	uint32_t DequeChunkSize() {
-		std::deque<int> q = {0, 1};
-		for (int i = 1; ((&q[i] - &q[i - 1]) == 1); q.push_back(++i));
-		return (q.size() - 1);
-	}
 
 
 	bool Is64Bit() { return (NativeWordSize() == 8); }
 
-	#ifdef WIN32
+	#ifdef _WIN32
 	/** @brief checks if the current process is running in 32bit emulation mode
 		@return FALSE, TRUE, -1 on error (usually no permissions) */
 	bool Is32BitEmulation()
@@ -465,7 +565,7 @@ namespace Platform
 
 
 	std::string GetShortFileName(const std::string& file) {
-		#ifdef WIN32
+		#ifdef _WIN32
 		std::vector<TCHAR> shortPathC(file.size() + 1, 0);
 
 		// FIXME: stackoverflow.com/questions/843843/getshortpathname-unpredictable-results
@@ -479,46 +579,62 @@ namespace Platform
 	}
 
 
-	std::string ExecuteProcess(const std::string& file, std::vector<std::string> args, bool asSubprocess)
+	const char* ExecuteProcess(std::array<std::string, 32>& args, bool asSubprocess)
 	{
+		static char execError[1024];
+
+		memset(execError, 0, sizeof(execError));
+		strcpy(execError, "ExecuteProcess failure");
+
+		// "The array of pointers must be terminated by a NULL pointer."
+		// --> always include one extra argument string and leave it NULL
+		std::array<char*, (sizeof(args) / sizeof(args[0])) + 1> argPointers;
+		std::array<char[4096], (sizeof(args) / sizeof(args[0]))> processArgs;
+
 		// "The first argument, by convention, should point to
 		// the filename associated with the file being executed."
 		// NOTE:
 		//   spaces in the first argument or quoted file paths
-		//   are not supported on Windows, so translate <file>
+		//   are not supported on Windows, so translate args[0]
 		//   to a short path there
-		args.insert(args.begin(), GetShortFileName(file));
-
-		std::string execError;
+		args[0] = GetShortFileName(args[0]);
 
 		if (asSubprocess) {
-			#ifdef WIN32
+			#ifdef _WIN32
 			STARTUPINFO si;
 			PROCESS_INFORMATION pi;
 
 			ZeroMemory(&si, sizeof(si)); si.cb = sizeof(si);
 			ZeroMemory(&pi, sizeof(pi));
 
-			std::vector<char> argsVec;
-			std::string argsStr;
+			char* flatArgsStr = &processArgs[0][0];
 
-			for (size_t a = 0; a < args.size(); ++a) {
-				argsStr.append(args[a] + ' ');
+			{
+				size_t i = 0;
+				size_t n = sizeof(processArgs);
+
+				// flatten args, i.e. from {"s0", "s1", "s2"} to "s0 s1 s2"
+				for (size_t a = 0; a < args.size(); ++a, ++i) {
+					if (args[a].empty())
+						break;
+					if ((i + args[a].size()) >= n)
+						break;
+
+					memcpy(&flatArgsStr[i                  ], args[a].data(), args[a].size());
+					memset(&flatArgsStr[i += args[a].size()], ' ', 1);
+				}
+
+				memset(&flatArgsStr[ std::min(i, n - 1) ], '\0', 1);
 			}
 
-			argsVec.assign(argsStr.begin(), argsStr.end());
-			argsVec.push_back('\0');
-
-			LOG("[%s] Windows start process arguments: %s", __func__, argsStr.c_str());
-
-			if (!CreateProcess(nullptr, argsVec.data(), nullptr, nullptr, TRUE, 0, nullptr, nullptr, &si, &pi))
-				LOG("[%s] Error creating subprocess (%lu)", __func__, GetLastError());
+			if (!CreateProcess(nullptr, flatArgsStr, nullptr, nullptr, TRUE, 0, nullptr, nullptr, &si, &pi))
+				LOG("[%s] error %lu creating subprocess with arguments \"%s\"", __func__, GetLastError(), flatArgsStr);
 
 			#else
 
 			int pid;
 			if ((pid = fork()) < 0) {
-				LOG("[%s] Error forking process", __func__);
+				LOG("[%s] error forking process", __func__);
 			} else if (pid != 0) {
 				// TODO: Maybe useful to return the subprocess ID (pid)?
 			}
@@ -527,33 +643,119 @@ namespace Platform
 			return execError;
 		}
 
-
-		// "The array of pointers must be terminated by a NULL pointer."
-		// --> include one extra argument string and leave it NULL
-		std::vector<char*> processArgs(args.size() + 1, nullptr);
 		for (size_t a = 0; a < args.size(); ++a) {
-			const std::string& arg = args[a];
-			const size_t argSize = arg.length() + 1;
+			if (args[a].empty())
+				break;
 
-			STRCPY_T(processArgs[a] = new char[argSize], argSize, arg.c_str());
+			memset(&processArgs[a][0], 0, sizeof(processArgs[a]));
+			memcpy(&processArgs[a][0], args[a].c_str(), std::min(args[a].size(), sizeof(processArgs[a]) - 1));
+
+			argPointers[a    ] = &processArgs[a][0];
+			argPointers[a + 1] = nullptr;
 		}
 
-		#ifdef WIN32
+		#ifdef _WIN32
 			#define EXECVP _execvp
 		#else
 			#define EXECVP execvp
 		#endif
 
-		if (EXECVP(args[0].c_str(), &processArgs[0]) == -1)
-			LOG("[%s] error: \"%s\" %s (%d)", __func__, args[0].c_str(), (execError = strerror(errno)).c_str(), errno);
+		if (EXECVP(args[0].c_str(), &argPointers[0]) == -1) {
+			STRNCPY(execError, strerror(errno), sizeof(execError) - 1);
+			LOG("[%s] error: \"%s\" %s (%d)", __func__, args[0].c_str(), execError, errno);
+		}
 
 		#undef EXECVP
 
-		for (size_t a = 0; a < args.size(); ++a) {
-			delete[] processArgs[a];
-		}
-
 		return execError;
 	}
 
+
+
+
+
+
+
+	#if defined(WIN32) || defined(_WIN32)
+	bool GetMacType(std::array<uint8_t, 6>& macAddr, uint32_t macType) {
+		IP_ADAPTER_INFO adapterInfo[16];
+
+		DWORD dwBufLen = sizeof(adapterInfo);
+		DWORD dwStatus = GetAdaptersInfo(adapterInfo, &dwBufLen);
+
+		if (dwStatus == ERROR_BUFFER_OVERFLOW)
+			return false;
+		if (dwStatus != NO_ERROR)
+			return false;
+
+		for (size_t i = 0, n = dwBufLen / sizeof(adapterInfo); i < n; i++) {
+			if ((macType != 0) && (adapterInfo[i].Type != macType))
+				continue;
+			if (adapterInfo[i].AddressLength != macAddr.size())
+				continue;
+
+			memcpy(macAddr.data(), adapterInfo[i].Address, adapterInfo[i].AddressLength);
+
+			if (std::find_if(macAddr.begin(), macAddr.end(), [](uint8_t byte) { return (byte != 0); }) != macAddr.end())
+				return true;
+		}
+
+		return false;
+	}
+
+	std::array<uint8_t, 6> GetRawMacAddr() {
+		std::array<uint8_t, 6> macAddr = {{0, 0, 0, 0, 0, 0}};
+
+		if (GetMacType(macAddr, MIB_IF_TYPE_ETHERNET))
+			return macAddr;
+		if (GetMacType(macAddr, IF_TYPE_IEEE80211))
+			return macAddr;
+
+		return (GetMacType(macAddr, 0), macAddr);
+	}
+
+	#elif defined(__APPLE__)
+
+	std::array<uint8_t, 6> GetRawMacAddr() {
+		// TODO: http://lists.freebsd.org/pipermail/freebsd-hackers/2004-June/007415.html
+		return {{0, 0, 0, 0, 0, 0}};
+	}
+
+	#else
+
+	std::array<uint8_t, 6> GetRawMacAddr() {
+		std::array<uint8_t, 6> macAddr = {{0, 0, 0, 0, 0, 0}};
+
+		ifaddrs* ifap = nullptr;
+
+		if (getifaddrs(&ifap) != 0)
+			return macAddr;
+
+		for (ifaddrs* iter = ifap; iter != nullptr; iter = iter->ifa_next) {
+			const sockaddr_ll* sal = reinterpret_cast<sockaddr_ll*>(iter->ifa_addr);
+
+			if (sal->sll_family != AF_PACKET)
+				continue;
+			if (sal->sll_halen != macAddr.size())
+				continue;
+
+			memcpy(macAddr.data(), sal->sll_addr, sal->sll_halen);
+
+			if (std::find_if(macAddr.begin(), macAddr.end(), [](uint8_t byte) { return (byte != 0); }) != macAddr.end())
+				break;
+		}
+
+		freeifaddrs(ifap);
+		return macAddr;
+	}
+	#endif
+
+	std::array<char, 18> GetHexMacAddr() {
+		std::array<uint8_t,  6> rawAddr = GetRawMacAddr();
+		std::array<   char, 18> hexAddr;
+
+		memset(hexAddr.data(), 0, hexAddr.size());
+		snprintf(hexAddr.data(), hexAddr.size(), "%02x:%02x:%02x:%02x:%02x:%02x", rawAddr[0], rawAddr[1], rawAddr[2], rawAddr[3], rawAddr[4], rawAddr[5]);
+		return hexAddr;
+	}
 } // namespace Platform
