@@ -11,27 +11,27 @@ enum WatchdogThreadnum {
 	WDT_MAIN  = 0,
 	WDT_LOAD  = 1,
 	WDT_AUDIO = 2,
-	WDT_COUNT = 3,
+	WDT_VFSI  = 3,
+	WDT_COUNT = 4,
 };
 
 namespace Watchdog
 {
-	//! Installs the watchdog thread
+	// Installs the watchdog thread
 	void Install();
 	void Uninstall();
 
-	//! Call this to reset the watchdog timer of the current thread
-	void ClearTimer(bool disable = false, Threading::NativeThreadId* _threadId = nullptr);
+	// Call this to reset the watchdog timer of the current thread
+	void ClearTimer(Threading::NativeThreadId* _threadId = nullptr, bool disable = false);
 	void ClearTimer(const WatchdogThreadnum num, bool disable = false);
-	void ClearTimer(const std::string& name, bool disable = false);
-	inline void ClearTimer(const char* name, bool disable = false) {
-		ClearTimer(std::string(name), disable);
-	}
-	void ClearPrimaryTimers(bool disable = false);
+	void ClearTimer(const char* name, bool disable = false);
+	void ClearTimers(bool disable = false, bool primary = false);
 
-	//! Call these in the threads you want to monitor
+	// Call these in the threads you want to monitor
 	void RegisterThread(WatchdogThreadnum num, bool primary = false);
 	bool DeregisterThread(WatchdogThreadnum num);
+	bool DeregisterCurrentThread();
+	bool HasThread(WatchdogThreadnum num);
 }
 
 #endif // _WATCHDOG_H
