@@ -3,7 +3,7 @@
 #ifndef DLLLIB_H
 #define DLLLIB_H
 
-#ifdef WIN32
+#ifdef _WIN32
 
 #include "System/Platform/SharedLib.h"
 #include <windows.h>
