@@ -5,12 +5,9 @@
 IAudioChannel::IAudioChannel()
 	: volume(1.0f)
 	, enabled(true)
-	, emmitsPerFrame(1000)
-	, emmitsThisFrame(0)
+	, emitsPerFrame(1000)
+	, emitsThisFrame(0)
 	, maxConcurrentSources(1024)
 {
 }
 
-IAudioChannel::~IAudioChannel()
-{
-}
