@@ -1,12 +1,12 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "SyncedFloat3.h"
-#include "System/myMath.h"
+#include "System/SpringMath.h"
 
 #if defined(SYNCDEBUG) || defined(SYNCCHECK)
 
-CR_BIND(SyncedFloat3, );
-CR_REG_METADATA(SyncedFloat3, (CR_MEMBER(x), CR_MEMBER(y), CR_MEMBER(z)));
+CR_BIND(SyncedFloat3, )
+CR_REG_METADATA(SyncedFloat3, (CR_MEMBER(x), CR_MEMBER(y), CR_MEMBER(z)))
 
 
 bool SyncedFloat3::IsInBounds() const
