@@ -28,10 +28,13 @@
 #define CR_BIND_DERIVED(TCls, TBase, ctor_args)
 #define CR_BIND(TCls, ctor_args)
 #define CR_BIND_DERIVED_INTERFACE(TCls, TBase)
+#define CR_BIND_DERIVED_INTERFACE_POOL(TCls, TBase, poolAlloc, poolFree)
+#define CR_BIND_DERIVED_POOL(TCls, TBase, ctor_args, poolAlloc, poolFree)
 #define CR_BIND_INTERFACE(TCls)
 #define CR_BIND_TEMPLATE(TCls, ctor_args)
 #define CR_REG_METADATA(TClass, Members)
 #define CR_REG_METADATA_SUB(TSuperClass, TSubClass, Members)
+#define CR_REG_METADATA_TEMPLATE(TCls, Members)
 #define CR_MEMBER(Member)
 #define CR_IGNORED(Member)
 #define CR_MEMBER_UN(Member)
