@@ -21,8 +21,8 @@ public:
 	slimsig::signal<void (void)> Clicked;
 
 private:
-	virtual void DrawSelf();
-	virtual bool HandleEventSelf(const SDL_Event& ev);
+	virtual void DrawSelf() override;
+	virtual bool HandleEventSelf(const SDL_Event& ev) override;
 
 	bool hovered;
 	bool clicked;
