@@ -9,6 +9,10 @@
 
 union SDL_Event;
 
+namespace Shader {
+	struct IProgramObject;
+}
+
 namespace agui
 {
 
@@ -18,7 +22,7 @@ class Gui
 {
 public:
 	Gui();
-	virtual ~Gui();
+	~Gui();
 
 	void Clean();
 	void Draw();
@@ -29,26 +33,40 @@ public:
 	void UpdateScreenGeometry(int screenx, int screeny, int screenOffsetX, int screenOffsetY);
 
 	bool MouseOverElement(const GuiElement*, int x, int y) const;
+	void SetColor(float r, float g, float b, float a);
+	void SetColor(const float* v) { SetColor(v[0], v[1], v[2], v[3]); }
+
+	enum DrawMode {
+		COLOR   = 0,
+		TEXTURE = 1,
+		FONT    = 2,
+	};
+
+	void SetDrawMode(DrawMode newMode);
+
+	Shader::IProgramObject* GetShader() { return shader; }
 
 private:
 	bool HandleEvent(const SDL_Event& ev);
-	InputHandler::SignalType::connection_type inputCon;
 
-	struct GuiItem
-	{
-		GuiItem(GuiElement* el, bool back) : element(el), asBackground(back) {};
+	struct GuiItem {
+		GuiItem(GuiElement* el, bool back) : element(el), asBackground(back) {}
 		GuiElement* element;
 		bool asBackground;
 	};
-	typedef std::list<GuiItem> ElList;
-	ElList elements;
-	ElList toBeRemoved;
-	ElList toBeAdded;
+
+	std::list<GuiItem> elements;
+	std::list<GuiItem> toBeRemoved;
+	std::list<GuiItem> toBeAdded;
+
+	DrawMode currentDrawMode = DrawMode::COLOR;
+
+	Shader::IProgramObject* shader;
+
+	InputHandler::SignalType::connection_type inputCon;
 };
 
 extern Gui* gui;
-void InitGui();
-void FreeGui();
 
 }
 
