@@ -13,19 +13,21 @@ namespace agui {
 class Window : public GuiElement
 {
 public:
-	Window(const std::string& title = "", GuiElement* parent = NULL);
+	Window(const std::string& title = "", GuiElement* parent = nullptr);
 
-	virtual void AddChild(GuiElement* elem);
+	virtual void AddChild(GuiElement* elem) override;
 	slimsig::signal<void (void)> WantClose;
 
 protected:
 	std::string title;
 
 private:
-	virtual void DrawSelf();
-	virtual bool HandleEventSelf(const SDL_Event& ev);
+	void DrawSelf() override;
+	void GeometryChangeSelf() override;
+	bool HandleEventSelf(const SDL_Event& ev) override;
+
+	float Opacity() const override;
 
-	virtual float Opacity() const;
 	bool dragging;
 	float dragPos[2];
 	float titleHeight;
