@@ -3,7 +3,8 @@
 Open Asset Import Library (assimp)
 ---------------------------------------------------------------------------
 
-Copyright (c) 2006-2016, assimp team
+Copyright (c) 2006-2017, assimp team
+
 
 All rights reserved.
 
@@ -51,7 +52,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #   include "lib/streflop/streflop_cond.h"
 #endif
 
-#include "./Compiler/pushpack1.h"
 #include "defs.h"
 
 #ifdef __cplusplus
@@ -126,7 +126,7 @@ public:
     const aiVector3t SymMul(const aiVector3t& o);
 
     TReal x, y, z;
-} PACK_STRUCT;
+};
 
 
 typedef aiVector3t<ai_real> aiVector3D;
@@ -135,16 +135,12 @@ typedef aiVector3t<ai_real> aiVector3D;
 
 struct aiVector3D {
     ai_real x, y, z;
-} PACK_STRUCT;
+};
 
 #endif // __cplusplus
 
-#include "./Compiler/poppack1.h"
-
 #ifdef __cplusplus
 
-
-
 #endif // __cplusplus
 
 #endif // AI_VECTOR3D_H_INC
