@@ -149,7 +149,7 @@ static inline T luaL_SpringOpt(lua_State* L, int idx, const T def, T(*lua_optFoo
 		return ret;
 
 	if (!lua_isnoneornil(L, idx))
-		LOG_L(L_WARNING, "Got wrong type for return argument #%d in \"%s::%s\" (%s expected, got %s)", luaS_absIndex(L, idx), spring_lua_getHandleName(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
+		LOG_L(L_WARNING, "[%s] wrong type for return argument %d in \"%s::%s\" (%s expected, got %s)", __func__, luaS_absIndex(L, idx), spring_lua_get_handle_name(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
 
 	return def;
 }
@@ -165,7 +165,7 @@ static inline std::string luaL_SpringOptString(lua_State* L, int idx, const std:
 		return ret;
 
 	if (!lua_isnoneornil(L, idx))
-		LOG_L(L_WARNING, "Got wrong type for return argument #%d in \"%s::%s\" (%s expected, got %s)", luaS_absIndex(L, idx), spring_lua_getHandleName(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
+		LOG_L(L_WARNING, "[%s(def=%s)] wrong type for return argument %d in \"%s::%s\" (%s expected, got %s)", __func__, def.c_str(), luaS_absIndex(L, idx), spring_lua_get_handle_name(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
 
 	return def;
 }
@@ -181,7 +181,7 @@ static inline const char* luaL_SpringOptCString(lua_State* L, int idx, const cha
 		return ret;
 
 	if (!lua_isnoneornil(L, idx))
-		LOG_L(L_WARNING, "Got wrong type for return argument #%d in \"%s::%s\" (%s expected, got %s)", luaS_absIndex(L, idx), spring_lua_getHandleName(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
+		LOG_L(L_WARNING, "[%s(def=%s)] wrong type for return argument %d in \"%s::%s\" (%s expected, got %s)", __func__, def, luaS_absIndex(L, idx), spring_lua_get_handle_name(L), caller, lua_typename(L, typeFoo), luaL_typename(L, idx));
 
 	if (len != nullptr)
 		*len = strlen(def);
@@ -247,5 +247,13 @@ static inline void LUA_UNLOAD_LIB(lua_State* L, std::string libname) {
      lua_pcall((L), 0, 0, 0);
 #endif
 
+#ifdef USING_CREG
+#  define SPRING_LUA_OPEN_LIB(L, lib) \
+	LUA_OPEN_LIB(L, lib); \
+	creg::RegisterCFunction(#lib, lib);
+#else
+#  define SPRING_LUA_OPEN_LIB(L, lib) LUA_OPEN_LIB(L, lib)
+#endif
+
 
 #endif // SPRING_LUA_INCLUDE
