@@ -12,9 +12,15 @@ public:
 	{
 		return handle;
 	}
+	std::string GetError() const;
 	static std::string escapeUrl(const std::string& url);
-
+	static void InitCurl();
+	static void KillCurl();
 private:
+	static bool VerifyFile(const std::string& path);
+	static bool ValidateCaFile(const std::string& cafile);
+
 	CURL* handle;
-	struct curl_slist* list;
+	char* errbuf;
+	curl_slist* list;
 };
