@@ -9,7 +9,6 @@
 #include "Logger.h"
 #include "SevenZipArchive.h"
 #include "ZipArchive.h"
-#include "lib/bencode/bencode.h"
 
 #include <zlib.h>
 #include <string.h>
@@ -19,7 +18,7 @@
 #include <dirent.h>
 #include <stdlib.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #include <shlobj.h>
 #include <math.h>
@@ -27,23 +26,23 @@
 #define SHGFP_TYPE_CURRENT 0
 #endif
 #else
+#include <unistd.h>
 #include <sys/statvfs.h>
 #include <errno.h>
 #endif
 
-static CFileSystem* singleton = NULL;
+static CFileSystem* singleton = nullptr;
 
 FILE* CFileSystem::propen(const std::string& filename,
-			  const std::string& mode) const
+			  const std::string& mode)
 {
-	FILE* ret;
-#ifdef WIN32
-	ret = _wfopen(s2ws(filename).c_str(), s2ws(mode).c_str());
+#ifdef _WIN32
+	FILE* ret = _wfopen(s2ws(filename).c_str(), s2ws(mode).c_str());
 #else
-	ret = fopen(filename.c_str(), mode.c_str());
+	FILE* ret = fopen(filename.c_str(), mode.c_str());
 #endif
-	if (ret == NULL) {
-		LOG_ERROR("Couldn't open %s", filename.c_str());
+	if (ret == nullptr) {
+		LOG_ERROR("Couldn't open %s: %s", filename.c_str(), strerror(errno));
 	}
 	return ret;
 }
@@ -51,18 +50,18 @@ FILE* CFileSystem::propen(const std::string& filename,
 bool CFileSystem::fileIsValid(const FileData* mod,
 			      const std::string& filename) const
 {
-	HashMD5 md5hash;
-	int bytes;
 	unsigned char data[IO_BUF_SIZE];
 	FILE* f = propen(filename, "rb");
 	gzFile inFile = gzdopen(fileno(f), "rb");
-	if (inFile == NULL) { // file can't be opened
+	if (inFile == nullptr) { // file can't be opened
 		fclose(f);
 		LOG_ERROR("Could not open file %s", filename.c_str());
 		return false;
 	}
+	HashMD5 md5hash;
 	md5hash.Init();
 	//	unsigned long filesize=0;
+	int bytes;
 	while ((bytes = gzread(inFile, data, IO_BUF_SIZE)) > 0) {
 		md5hash.Update((char*)data, bytes);
 		//		filesize=filesize+bytes;
@@ -133,7 +132,7 @@ bool CFileSystem::parseSdp(const std::string& filename, std::list<FileData>& fil
 		memcpy(fd.crc32, &c_crc32, 4);
 		fd.size = parse_int32(c_size);
 		files.push_back(fd);
-		
+
 		HashMD5 nameMd5;
 		nameMd5.Init();
 		nameMd5.Update(fd.name.data(), fd.name.size());
@@ -156,24 +155,15 @@ bool CFileSystem::parseSdp(const std::string& filename, std::list<FileData>& fil
 	return true;
 }
 
-CFileSystem::~CFileSystem()
-{
-}
-
 bool CFileSystem::setWritePath(const std::string& path)
 {
 
 	if (!path.empty()) {
-		if (!directoryExists(path)) {
-			LOG_ERROR("filesystem-writepath doesn't exist: %s", path.c_str());
-			return false;
-		}
 		springdir = path;
 	} else {
-#ifndef WIN32
-		char* buf;
-		buf = getenv("HOME");
-		if (buf != NULL) {
+#ifndef _WIN32
+		const char* buf = getenv("HOME");
+		if (buf != nullptr) {
 			springdir = buf;
 			springdir.append("/.spring");
 		} else { // no home: use cwd
@@ -182,7 +172,7 @@ bool CFileSystem::setWritePath(const std::string& path)
 		}
 #else
 		wchar_t my_documents[MAX_PATH];
-		HRESULT result = SHGetFolderPathW(NULL, CSIDL_PERSONAL, NULL,
+		HRESULT result = SHGetFolderPathW(nullptr, CSIDL_PERSONAL, nullptr,
 						  SHGFP_TYPE_CURRENT, my_documents);
 		if (result == S_OK) {
 			springdir = ws2s(my_documents);
@@ -196,32 +186,25 @@ bool CFileSystem::setWritePath(const std::string& path)
 		}
 	}
 	LOG_INFO("Using filesystem-writepath: %s", springdir.c_str());
-	return true;
-}
-
-CFileSystem::CFileSystem()
-    : portableDownload(false)
-{
+	return createSubdirs(springdir.c_str());
 }
 
 CFileSystem* CFileSystem::GetInstance()
 {
-	if (singleton == NULL)
+	if (singleton == nullptr) {
 		singleton = new CFileSystem();
+	}
 	return singleton;
 }
 
 void CFileSystem::Shutdown()
 {
-	CFileSystem* tmpFileSystem = singleton;
-	singleton = NULL;
-	delete tmpFileSystem;
-	tmpFileSystem = NULL;
+	delete singleton;
+	singleton = nullptr;
 }
 
 const std::string CFileSystem::getSpringDir()
 {
-	assert(!springdir.empty());
 	if (springdir.empty())
 		(setWritePath(""));
 	return springdir;
@@ -231,7 +214,7 @@ bool CFileSystem::directoryExists(const std::string& path)
 {
 	if (path.empty())
 		return false;
-#ifdef WIN32
+#ifdef _WIN32
 	const std::wstring wpath = s2ws(path);
 	DWORD dwAttrib = GetFileAttributesW(wpath.c_str());
 	return ((dwAttrib != INVALID_FILE_ATTRIBUTES) &&
@@ -246,8 +229,8 @@ bool CFileSystem::directoryExists(const std::string& path)
 bool CreateDir(const std::string& path)
 {
 	assert(!path.empty());
-#ifdef WIN32
-	return CreateDirectory(s2ws(path).c_str(), NULL);
+#ifdef _WIN32
+	return CreateDirectory(s2ws(path).c_str(), nullptr);
 #else
 	return mkdir(path.c_str(), 0755) == 0;
 #endif
@@ -261,7 +244,7 @@ bool CFileSystem::createSubdirs(const std::string& path)
 	}
 	for (size_t i = 2; i < path.size(); i++) {
 		char c = path.at(i);
-#ifdef WIN32
+#ifdef _WIN32
 		/* skip for example mkdir(C:\) */
 		if ((i == 2) && (c == PATH_DELIMITER))
 			continue;
@@ -287,18 +270,17 @@ bool CFileSystem::createSubdirs(const std::string& path)
 	return CreateDir(path);
 }
 
-void CFileSystem::getPoolFilename(const std::string& md5str,
-				  std::string& path)
+std::string CFileSystem::getPoolFilename(const std::string& md5str) const
 {
-	path = fileSystem->getSpringDir();
-	path += PATH_DELIMITER;
-	path += "pool";
-	path += PATH_DELIMITER;
-	path += md5str.at(0);
-	path += md5str.at(1);
-	path += PATH_DELIMITER;
-	path += md5str.substr(2);
-	path += ".gz";
+	return fileSystem->getSpringDir()
+	 + PATH_DELIMITER
+	 + "pool"
+	 + PATH_DELIMITER
+	 + md5str.at(0)
+	 + md5str.at(1)
+	 + PATH_DELIMITER
+	 + md5str.substr(2)
+	 + ".gz";
 }
 
 int CFileSystem::validatePool(const std::string& path, bool deletebroken)
@@ -310,58 +292,59 @@ int CFileSystem::validatePool(const std::string& path, bool deletebroken)
 	int res = 0;
 	std::list<std::string> dirs;
 	dirs.push_back(path);
-	int maxdirs = 257; // FIXME: unknown dirs in pool will break bar
+	const int maxdirs = 257; // FIXME: unknown dirs in pool will break bar
 	int finished = 0;
 	IHash* md5 = new HashMD5();
 	while (!dirs.empty()) {
-		struct dirent* dentry;
-		DIR* d;
 		const std::string dir = dirs.front();
 		dirs.pop_front();
-		d = opendir(dir.c_str());
-		while ((dentry = readdir(d)) != NULL) {
+		DIR* d = opendir(dir.c_str());
+		dirent* dentry;
+		while ((dentry = readdir(d)) != nullptr) {
 			LOG_PROGRESS(finished, maxdirs);
-			std::string absname = dir;
-			absname += PATH_DELIMITER;
-			absname += dentry->d_name;
-			if (dentry->d_name[0] != '.') { // don't check hidden files / . / ..
-#ifndef WIN32
-				if ((dentry->d_type & DT_DIR) != 0) { // directory
+			const std::string absname = dir + PATH_DELIMITER + dentry->d_name;
+			// don't check hidden files / . / ..
+			if (dentry->d_name[0] == '.') {
+				continue;
+			}
+#ifndef _WIN32
+			if ((dentry->d_type & DT_DIR) != 0) { // directory
 #else
-				struct stat sb;
-				stat(absname.c_str(), &sb);
-				if ((sb.st_mode & S_IFDIR) != 0) {
+			struct stat sb;
+			stat(absname.c_str(), &sb);
+			if ((sb.st_mode & S_IFDIR) != 0) {
 #endif
-					dirs.push_back(absname);
-				} else {
-					FileData filedata = FileData();
-					int len = absname.length();
-					if (len < 36) { // file length has at least to be
-							// <md5[0]><md5[1]>/<md5[2-30]>.gz
-						LOG_ERROR("Invalid file: %s", absname.c_str());
-					} else {
-						std::string md5str = "";
-						md5str.push_back(
-						    absname.at(len - 36)); // get md5 from path + filename
-						md5str.push_back(absname.at(len - 35));
-						md5str.append(absname.substr(len - 33, 30));
-						md5->Set(md5str);
-						for (unsigned i = 0; i < 16; i++) {
-							filedata.md5[i] = md5->get(i);
-						}
+				dirs.push_back(absname);
+				continue;
+			}
 
-						if (!fileIsValid(&filedata, absname)) { // check if md5 in filename
-											// is the same as in
-											// filename
-							LOG_ERROR("Invalid File in pool: %s", absname.c_str());
-							if (deletebroken) {
-								removeFile(absname);
-							}
-						} else {
-							res++;
-						}
-					}
+			const int len = absname.length();
+			if (len < 36) { // file length has at least to be
+					// <md5[0]><md5[1]>/<md5[2-30]>.gz
+				LOG_ERROR("Invalid file: %s", absname.c_str());
+				continue;
+			}
+
+			std::string md5str;
+			 // get md5 from path + filename
+			md5str.push_back(absname.at(len - 36));
+			md5str.push_back(absname.at(len - 35));
+			md5str.append(absname.substr(len - 33, 30));
+			md5->Set(md5str);
+			FileData filedata;
+			for (unsigned i = 0; i < 16; i++) {
+				filedata.md5[i] = md5->get(i);
+			}
+
+			if (!fileIsValid(&filedata, absname)) { // check if md5 in filename
+								// is the same as in
+								// filename
+				LOG_ERROR("Invalid File in pool: %s", absname.c_str());
+				if (deletebroken) {
+					removeFile(absname);
 				}
+			} else {
+				res++;
 			}
 		}
 		finished++;
@@ -375,30 +358,40 @@ int CFileSystem::validatePool(const std::string& path, bool deletebroken)
 
 bool CFileSystem::isOlder(const std::string& filename, int secs)
 {
+	if (secs <= 0)
+		return true;
 	struct stat sb;
 	if (stat(filename.c_str(), &sb) < 0) {
 		return true;
 	}
-	time_t t;
-#ifdef WIN32
+#ifdef _WIN32
 	SYSTEMTIME pTime;
 	FILETIME pFTime;
 	GetSystemTime(&pTime);
 	SystemTimeToFileTime(&pTime, &pFTime);
-	t = FiletimeToTimestamp(pFTime);
-#else
-	time(&t);
-#endif
-	LOG_DEBUG("%s is %d seconds old, redownloading at %d", filename.c_str(),
-		  (int)(t - sb.st_ctime), secs);
+	const time_t t = FiletimeToTimestamp(pFTime);
+	LOG_DEBUG("%s is %d seconds old, redownloading at %d", filename.c_str(), (int)(t - sb.st_ctime), secs);
 	return (t < sb.st_ctime + secs);
+
+#else
+	time_t t;
+	time(&t);
+	struct tm lt;
+	localtime_r(&sb.st_mtime, &lt);
+
+	const time_t filetime = mktime(&lt);
+	const double diff = difftime(t, filetime);
+
+	LOG_DEBUG("checking time: %s  %.0fs >  %ds res: %d", filename.c_str(), diff, secs, (bool)(diff > secs));
+	return (diff > secs);
+#endif
 }
 
 bool CFileSystem::fileExists(const std::string& path)
 {
 	if (path.empty())
 		return false;
-#ifdef WIN32
+#ifdef _WIN32
 	DWORD dwAttrib = GetFileAttributesW(s2ws(path).c_str());
 	return (dwAttrib != INVALID_FILE_ATTRIBUTES);
 #else
@@ -409,7 +402,7 @@ bool CFileSystem::fileExists(const std::string& path)
 
 bool CFileSystem::removeFile(const std::string& path)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	const bool res = _wunlink(s2ws(path).c_str()) == 0;
 #else
 	const bool res = unlink(path.c_str()) == 0;
@@ -422,7 +415,7 @@ bool CFileSystem::removeFile(const std::string& path)
 
 bool CFileSystem::removeDir(const std::string& path)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	const bool res = _wrmdir(s2ws(path).c_str()) == 0;
 #else
 	const bool res = rmdir(path.c_str()) == 0;
@@ -433,84 +426,12 @@ bool CFileSystem::removeDir(const std::string& path)
 	return res;
 }
 
-bool CFileSystem::parseTorrent(const char* data, int size, IDownload* dl)
-{
-	struct be_node* node = be_decoden(data, size);
-	//#ifdef DEBUG
-	//	be_dump(node);
-	//#endif
-	if (node == NULL) {
-		LOG_ERROR("couldn't parse torrent");
-		return false;
-	}
-	if (node->type != BE_DICT) {
-		LOG_ERROR("Error in torrent data");
-		be_free(node);
-		return false;
-	}
-	int i;
-	struct be_node* infonode = NULL;
-	for (i = 0; node->val.d[i].val; ++i) { // search for a dict with name info
-		if ((node->type == BE_DICT) && (strcmp(node->val.d[i].key, "info") == 0)) {
-			infonode = node->val.d[i].val;
-			break;
-		}
-	}
-	if (infonode == NULL) {
-		LOG_ERROR("couldn't find info node in be dict");
-		be_free(node);
-		return false;
-	}
-	for (i = 0; infonode->val.d[i].val;
-	     ++i) { // fetch needed data from dict and fill into dl
-		struct be_node* datanode;
-		datanode = infonode->val.d[i].val;
-		switch (datanode->type) {
-			case BE_STR: // current value is a string
-				if ((strcmp("name", infonode->val.d[i].key) == 0) &&
-				    (dl->name.empty())) { // set filename if not already set
-					dl->name = datanode->val.s;
-				} else if (!strcmp("pieces",
-						   infonode->val.d[i].key)) { // hash sum of a piece
-					const int count =
-					    be_str_len(datanode) / 20; // one sha1 sum is 5 * 4 bytes long
-					for (int i = 0; i < count; i++) {
-						struct IDownload::piece piece;
-						const unsigned char* data = (unsigned char*)&datanode->val.s[i * 20];
-						piece.sha = new HashSHA1();
-						if (!piece.sha->Set(data, 20)) {
-							LOG_ERROR("Error setting sha1");
-						}
-						piece.state = IDownload::STATE_NONE;
-						dl->pieces.push_back(piece);
-					}
-				}
-				break;
-			case BE_INT:						     // current value is a int
-				if (strcmp("length", infonode->val.d[i].key) == 0) { // filesize
-					dl->size = datanode->val.i;
-				} else if (!strcmp("piece length",
-						   infonode->val.d[i].key)) { // length of a piece
-					dl->piecesize = datanode->val.i;
-					LOG_DEBUG("dl->piecesize: %d", dl->piecesize);
-				}
-				break;
-			default:
-				break;
-		}
-	}
-	LOG_DEBUG("Parsed torrent data: %s %d", dl->name.c_str(), dl->piecesize);
-	be_free(node);
-	return true;
-}
-
 bool CFileSystem::dumpSDP(const std::string& filename)
 {
 	std::list<FileData> files;
 	if (!parseSdp(filename, files))
 		return false;
 	LOG_INFO("md5 (filename in pool)           crc32        size filename");
-	std::list<FileData*>::iterator it;
 	HashMD5 md5;
 	for (const FileData& fd: files) {
 		md5.Set(fd.md5, sizeof(fd.md5));
@@ -540,11 +461,9 @@ bool CFileSystem::validateSDP(const std::string& sdpPath)
 
 	bool valid = true;
 	for (FileData& fd : files) {
-
-		std::string filePath;
 		HashMD5 fileMd5;
 		fileMd5.Set(fd.md5, sizeof(fd.md5));
-		getPoolFilename(fileMd5.toString(), filePath);
+		const std::string filePath = getPoolFilename(fileMd5.toString());
 		if(!fileExists(filePath)) {
 			valid = false;
 			LOG_INFO("Missing file: %s", filePath.c_str());
@@ -562,11 +481,10 @@ bool CFileSystem::validateSDP(const std::string& sdpPath)
 }
 
 bool CFileSystem::extractEngine(const std::string& filename,
-				const std::string& version)
+				const std::string& version, const std::string& platform)
 {
 #ifdef ARCHIVE_SUPPORT
-	const std::string output = getSpringDir() + PATH_DELIMITER + "engine" +
-				   PATH_DELIMITER +
+	const std::string output = getSpringDir() + PATH_DELIMITER + "engine" + PATH_DELIMITER + platform + PATH_DELIMITER +
 				   CFileSystem::EscapeFilename(version);
 	if (!extract(filename, output)) {
 		LOG_DEBUG("Failed to extract %s %s", filename.c_str(), output.c_str());
@@ -600,6 +518,8 @@ bool CFileSystem::extract(const std::string& filename,
 
 	const unsigned int num = archive->NumFiles();
 	if (num <= 0) {
+		LOG_WARN("Empty archive:  %s", filename.c_str());
+		delete archive;
 		return false;
 	}
 	for (unsigned int i = 0; i < num; i++) {
@@ -612,7 +532,7 @@ bool CFileSystem::extract(const std::string& filename,
 			delete archive;
 			return false;
 		}
-#ifdef WIN32
+#ifdef _WIN32
 		for (unsigned int i = 0; i < name.length();
 		     i++) { // replace / with \ on win32
 			if (name[i] == '/')
@@ -628,13 +548,13 @@ bool CFileSystem::extract(const std::string& filename,
 		tmp += name.c_str(); // FIXME: concating UTF-16
 		createSubdirs(DirName(tmp));
 		if (fileSystem->fileExists(tmp)) {
-			LOG_ERROR("File already exists: %s", tmp.c_str());
+			LOG_WARN("File already exists: %s", tmp.c_str());
 			if (!overwrite)
 				continue;
 		}
 		LOG_INFO("extracting (%s)", tmp.c_str());
 		FILE* f = propen(tmp, "wb+");
-		if (f == NULL) {
+		if (f == nullptr) {
 			LOG_ERROR("Error creating %s", tmp.c_str());
 			delete archive;
 			return false;
@@ -642,7 +562,7 @@ bool CFileSystem::extract(const std::string& filename,
 		int res = 1;
 		if (!buf.empty())
 			res = fwrite(&buf[0], buf.size(), 1, f);
-#ifndef WIN32
+#ifndef _WIN32
 		fchmod(fileno(f), mode);
 #endif
 		if (res <= 0) {
@@ -666,7 +586,7 @@ bool CFileSystem::extract(const std::string& filename,
 bool CFileSystem::Rename(const std::string& source,
 			 const std::string& destination)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	return MoveFileW(s2ws(source).c_str(), s2ws(destination).c_str());
 #else
 	int res = rename(source.c_str(), destination.c_str());
@@ -684,7 +604,7 @@ std::string CFileSystem::DirName(const std::string& path)
 	}
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 long CFileSystem::FiletimeToTimestamp(const _FILETIME& time)
 {
 	LARGE_INTEGER date, adjust;
@@ -720,7 +640,7 @@ std::string CFileSystem::EscapeFilename(const std::string& str)
 
 unsigned long CFileSystem::getMBsFree(const std::string& path)
 {
-#ifdef WIN32
+#ifdef _WIN32
 	ULARGE_INTEGER freespace;
 	BOOL res = GetDiskFreeSpaceEx(s2ws(path).c_str(), &freespace, nullptr, nullptr);
 	if (!res) {
@@ -737,9 +657,20 @@ unsigned long CFileSystem::getMBsFree(const std::string& path)
 		return 0;
 	}
 	if (st.f_frsize) {
-		return (st.f_frsize * st.f_bavail) / (1024 * 1024);
+		return ((uint64_t)st.f_frsize * st.f_bavail) / (1024 * 1024);
 	}
-	return (st.f_bsize * st.f_bavail) / (1024 * 1024);
+	return ((uint64_t)st.f_bsize * st.f_bavail) / (1024 * 1024);
 #endif
 }
 
+
+long CFileSystem::getFileSize(const std::string& path)
+{
+	FILE* f = propen(path.c_str(), "rb");
+	if (f == nullptr)
+		return -1;
+	fseek(f, 0, SEEK_END);
+	long size = ftell(f);
+	fclose(f);
+	return size;
+}
